(***********************************************)
(*                                             *)
(*            ColourMaths Package              *)
(*         Copyright D.Bailey 2014             *)
(*       http://www.dbaileyconsultancy.co.uk   *)
(*                                             *)
(***********************************************)
(* ::Package:: *)

(* ::Title:: *)
(*ColourMaths package*)


(* ::Section:: *)
(*Package preamble*)


BeginPackage["ColourMaths`"];


Assert::usage="Assert[<rule name>,<expr1>==<expr2>,<var list>,<0 or mor comments>";

Use::usage="Use[myrule] returns a function that applies a rule Asserted with Assert.\n Variable changes, such as t->s can be applied as options";

Use\[Breve]Function::usage="Experimental";

AllPossibleReplacements::usage="Returns list of all ways that arg2\[Rule]arg3 can apply to arg1";

Dialog\[Breve]Box::usage="Dialog box builder";

dText::usage="Data structure used by Dialog\[Breve]Box";

dItem::usage="Data structure used by Dialog\[Breve]Box";

dTitle::usage="Data structure used by Dialog\[Breve]Box";

dInput::usage="Data structure used by Dialog\[Breve]Box";

dButtons::usage="Data structure used by Dialog\[Breve]Box";

dHButtons::usage="Data structure used by Dialog\[Breve]Box";

vList::usage="Data structure used by Dialog\[Breve]Box";

RtoF::usage="Converts a rule to the corresponding function to apply the rule";

RRtoF::usage="Converts a rule to the corresponding function to apply the rule exhaustively";

DBColour::usage="Used to colour selections - do not call explicitly";

DBNoColour::usage="Used to colour selections - do not call explicitly";

Change\[Breve]Variables::usage="Function to Change variables in a protected integral, e.g. Change\[Breve]Variables[u->ArcTan[x],x]";

ByParts::usage="Integrates by parts. With one argument it is assumed that the part of the integrand to be integrated is coloured red. Alternatively the part to be integrated can be supplied as the second argument";

SwapOperations::usage="Swaps various pairs of operations such as sums and integrals. This function must be applied to the exact object, not to a containing expression";

Setup\[Breve]Extraction::usage="Extract expression to a temporary notebook";

visualInterface::usage="Assign visualInterface[<form>] to the name of a dialog routine for point and click operations";

Return\[Breve]Extraction::usage="";

Pick\[Breve]Highlight\[Breve]Formula::usage="Pick\[Breve]Highlight\[Breve]Formula[nb] \[Dash] Displays a menu of highlight formulae in the given notebook and returns your selection. nb defaults to the current notebook.";

Highlight\[Breve]Formula::usage="Highlight\[Breve]Formula[name,expr] \[Dash] Creates named formula \[Dash] a process that is usually done using the \"Hightlight\" button";

Get\[Breve]Highlight\[Breve]Formula::usage="Get\[Breve]Highlight\[Breve]Formula[name] \[Dash] Returns a named formula from the current notebook. The relevant notebook can be specified as a second argument.";

ColourSort::usage="CS[] returns a function to sort partially coloured lists with colours at left.\nCS[f] returns a function to sort the arguments of terms of head f.";
CS=ColourSort;
CS1::usage="Function to sort partially coloured lists with colours at left, then remove colour";
ColourSortAndClear=CS1;

ColourMathsVersion::usage="ColourMathsVersion[] returns the version of the ColourMaths Package (also visible on the title of the palette)";

noSpell::usage="noSpell takes a string or a list of strings and turns them into symbols with the spelling message suppressed";

Integrate\[Breve]Inert::usage="Unevaluated form of definite integral - useful for various manipulations on integrals";

Sum\[Breve]Inert::usage="Unevaluated form of summation - useful for various manipulations on sums";

EllipsisForm::usage="Outputs sums and products as a sequence broken by ellipses\nThe \"Repeat\" option can be given to change the number of terms shown";

Product\[Breve]Inert::usage="Unevaluated form of product - useful for various manipulations on products";

Limit\[Breve]Inert::usage="Unevaluated form of Limit";

Define\[Breve]Adjustment::usage="";

Do\[Breve]Adjustment::usage="";

Make\[Breve]Infix\[Breve]Operator::usage="Make\[Breve]Infix\[Breve]Operator[template,char,z,fn,comment] \[Dash] create an infix operator \[Dash] for example Make\[Breve]Infix\[Breve]Operator[\!\(\*OverscriptBox[\(z\), \(^\)]\),\"\[CircleTimes]\",z,MyFunction,\"My opeator\"]";

Make\[Breve]Prefix\[Breve]Operator::usage="";

Diff::usage="Differentiation operator that can differentiate inside an inert integral";

Define\[Breve]Operator\[Breve]Relationship::usage="";

Make\[Breve]Operator::usage="";

Val1::usage="Val1[A,x,b] represents the value of A at x=b, Normally displayed as [A\!\(\*SubscriptBox[\(]\), \(\(\(x\)\(\\\\\)\)\\\  = \(\(\\\\\)\(\\\ \)\(b\)\)\)]\)";

DoIntegrals::usage="A function which converts inert integrals to unptotected ones";

DoInerts::usage="A function which converts all inert constructs to their actve forms \[Dash] thus causing them to evaluate";

EvalIntegrals::usage="A function which numerically evaluates integrals using substitutions in its arguments";

DoSummations::usage="A function which converts inert sums to unptotected ones";

DoProducts::usage="A function which converts protected products to unptotected ones";

DoLimits::usage="A function which converts inert limits to unptotected ones";

DoSpecialFunctions::usage="A function to convert special functions to mathematica form";

SiftOptions::usage="Analyse options to a function SiftOptions[specs_List,opts_List,parent_]";

Left\[Breve]Side::usage="Left\[Breve]Side[x] returns the left hand side of x if x is an equality/inequality";

Right\[Breve]Side::usage="Right\[Breve]Side[x] returns the right hand side of x if x is an equality/inequality";




Begin["`Private`"];


(* ::Section:: *)
(*Internal debugging switches of various sorts*)


Debug\[Breve]Print=SuperWidgetPackage`Debug\[Breve]Print;


Debug\[Breve]Evaluate\[Breve]Print=SuperWidgetPackage`Debug\[Breve]Evaluate\[Breve]Print;


mePrint[x_] := 
  If[DebugMakeExpression,Print[x/. u_String->FullForm[u]]];False
  

DebugMakeExpression=False


(* ::Section:: *)
(*Version*)


version=2.31;
ColourMathsVersion[]=version;


(* ::Section:: *)
(*Inert operations*)


inert\[Breve]background=RGBColor[.8,.8,1];


(*SetAttributes[Sum\[Breve]Inert,HoldAll];*)
(*SetAttributes[Product\[Breve]Inert,HoldAll];*)
SetAttributes[Limit\[Breve]Inert,Listable];
inert\[Breve]conversions={
   {Integrate,Integrate\[Breve]Inert},
   {Sum,Sum\[Breve]Inert},
   {Product,Product\[Breve]Inert},
   {Limit,Limit\[Breve]Inert}
};


Make\[Breve]Inert[HoldComplete[s_[body___]]]:=Module[{p,ans},
   inert\[Breve]worked=False;
   p=Position[inert\[Breve]conversions[[All,1]],s];
   ans=HoldComplete[s[body]];
   If[p==={},Return[ans]];
   inert\[Breve]worked=True;
   p=p[[1,1]];
   ans[[1,0]]=inert\[Breve]conversions[[p,2]];
(* Indefinite integrals are converted to definite ones *)
   If[ans[[1,0]]===Integrate\[Breve]Inert && ans[[1,2,0]]===Symbol,
      ans[[1,2]]={ans[[1,2]],0,ans[[1,2]]}];
   ans   
];

(*SetAttributes[Make\[Breve]Active,HoldFirst];*)
Make\[Breve]Active[HoldComplete[s_[body___]]]:=Module[{p,ans},
   p=Position[inert\[Breve]conversions[[All,2]],s];
   If[p==={},Return[s[body]]];
   ans=HoldComplete[s[body]];
   p=p[[1,1]];
   ans[[1,0]]=inert\[Breve]conversions[[p,1]];
   ans 
];


background\[Breve]adjust[RGBColor[r_,g_,b_]]:=RGBColor[r*0.93,g*0.93,b];

MakeBoxes[x_Integrate\[Breve]Inert,StandardForm]:=Module[{xx=Hold[x,StandardForm]},
   Block[{inert\[Breve]background=background\[Breve]adjust[inert\[Breve]background]},
      xx[[1,0]]=Integrate;
      xx[[0]]=MakeBoxes;
      TagBox[xx,Integrate\[Breve]Inert,DefaultBaseStyle->{Background->inert\[Breve]background},SelectWithContents->True]
   ]
];

MakeExpression[TagBox[x_,Integrate\[Breve]Inert,___],StandardForm]:=Module[{xx},
   xx=MakeExpression[x,StandardForm];
   xx[[1,0]]=Integrate\[Breve]Inert;
   xx
];

MakeBoxes[x_Sum\[Breve]Inert,StandardForm]:=Module[{xx=Hold[x,StandardForm]},
   Block[{inert\[Breve]background=background\[Breve]adjust[inert\[Breve]background]},
      xx[[1,0]]=Sum;
      xx[[0]]=MakeBoxes;
      TagBox[xx,Sum\[Breve]Inert,DefaultBaseStyle->{Background->inert\[Breve]background},SelectWithContents->True]
   ]
];
MakeExpression[TagBox[x_,Sum\[Breve]Inert,___],StandardForm]:=Module[{xx},
   xx=MakeExpression[x,StandardForm];
   xx[[1,0]]=Sum\[Breve]Inert;
   xx
];

MakeBoxes[x_Product\[Breve]Inert,StandardForm]:=Module[{xx=Hold[x,StandardForm]},
   Block[{inert\[Breve]background=background\[Breve]adjust[inert\[Breve]background]},
      xx[[1,0]]=Product;
      xx[[0]]=MakeBoxes;
      TagBox[xx,Product\[Breve]Inert,DefaultBaseStyle->{Background->inert\[Breve]background},SelectWithContents->True]
   ]
];
MakeExpression[TagBox[x_,Product\[Breve]Inert,___],StandardForm]:=Module[{xx},
   xx=MakeExpression[x,StandardForm];
   xx[[1,0]]=Product\[Breve]Inert;
   xx
];

MakeBoxes[x_Limit\[Breve]Inert,StandardForm]:=Module[{xx=Hold[x,StandardForm]},
   xx[[1,0]]=Limit;
   xx[[0]]=MakeBoxes;
   TagBox[xx,Limit\[Breve]Inert,DefaultBaseStyle->{Background->inert\[Breve]background},SelectWithContents->True]
];

MakeExpression[TagBox[x_,Limit\[Breve]Inert,___],StandardForm]:=Module[{xx},
   Block[{inert\[Breve]background=background\[Breve]adjust[inert\[Breve]background]},
      xx=MakeExpression[x,StandardForm];
      xx[[1,0]]=Limit\[Breve]Inert;
      xx
   ]
];


(* ::Section:: *)
(*Set up all the colours*)


(* ::Text:: *)
(*Create the colours and usage statements for all the functions derived from them*)


all\[Breve]colours={
   {ColourMaths`Red\[Breve]Colour,Red,"R"},
   {ColourMaths`Green\[Breve]Colour,Green,"G"},
   {ColourMaths`Blue\[Breve]Colour,Blue,"B"},
   {ColourMaths`Flag\[Breve]Colour,Magenta,"M"}, (* Reserved as the flag colour *)
   {ColourMaths`Purple\[Breve]Colour,Purple,"P"},
   {ColourMaths`Yellow\[Breve]Colour,Yellow,"Y"},
   {ColourMaths`Orange\[Breve]Colour,Orange,"O"}
};


SetOptions[SelectedNotebook[],ShowSyntaxStyles->False];
Module[{j=1,a,c,d,cc,ccl,cletter,f,ff,f1,ff1,colour\[Breve]applier},
   (* We build these up on the fly *)
   colourRemovalRules={};
   (* Loop over all colours *)
   While[j<= Length[all\[Breve]colours],
      c=all\[Breve]colours[[j,1]];
      cletter=all\[Breve]colours[[j,3]];
      cc=SymbolName[c];
      ccl=ToLowerCase[cc];
      (* Set up the usage of the colour functions themselves *)
      MessageName[Evaluate[c],"usage"]=
      "Pseudo function representing "<> ccl <>" expressions";              
      (*Supply the box definitions for colour functions *)
      
      With[{c=c,rgb=all\[Breve]colours[[j,2]]},
         c/: MakeBoxes[c[x_],StandardForm]:=
         TagBox[                            
            MakeBoxes[x,StandardForm],c,DefaultBaseStyle->{FontColor->rgb},SelectWithContents->True];
      ];            
      If[c=!=ColourMaths`Flag\[Breve]Colour,
         (* Set up RR,RR1,RedOperation,RedOperationAndClear etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>cletter];
         f1=ToExpression["ColourMaths`"<>cletter<>cletter<>"1"];
         ff=ToExpression["ColourMaths`"<>cc<>"Operation"];
         ff1=ToExpression["ColourMaths`"<>cc<>"OperationAndClear"];              
         MessageName[Evaluate[f],"usage"]=
         "Returns a function that specialises a rule to apply to "<>ccl<>
         " parts of equation, colour retained";              
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];              
         MessageName[Evaluate[f1],"usage"]=
         "Returns a function that specialises a rule to apply to "<>ccl<>" parts of equation, colour destroyed\n"<> 
         cletter<>cletter<>"1[] returns a rule to remove  "<>ccl<>
         " colouration";              
         MessageName[Evaluate[ff1],"usage"]=MessageName[Evaluate[f1],"usage"];
         (* Setup functions RedOperation etc. *)
         With[{f=f,f1=f1,ff=ff,ff1=ff1,c=c},
            f=ff;
            f1=ff1;                     
            ff[x___]:=ColourTransform[c,Flatten[{x}],c]/.D2->D1;           
            (*\[LightBulb]Identity\[LetterSpace]changed\[LetterSpace]here*)          
            ff1[x___]:=ColourTransform[c,Flatten[{x}],Identity]/.D2->D1/.Identity[y_]->y;
         ];             
         colour\[Breve]applier=ff1;
         (* Set up Rout, etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"out"];
         f1=ToExpression["ColourMaths`"<>cletter<>"out1"];
         ff=ToExpression["ColourMaths`Move"<>cc<>"Out"];
         ff1=ToExpression["ColourMaths`Move"<>cc<>"OutAndClear"];              
         MessageName[Evaluate[f],"usage"]="Move "<>ccl<>" expression out";              
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];              
         MessageName[Evaluate[f1],"usage"]=
         "Move "<>ccl<>" expression out, colour destroyed";
         
         MessageName[Evaluate[ff1],"usage"]=MessageName[Evaluate[f1],"usage"];
         With[{f=f,f1=f1,ff=ff,ff1=ff1,c=c},
            f=ff;
            f1=ff1;
            ff[x_]:= moveOut[x,c,c];
            ff1[x_]:= moveOut[x,c,Identity];
         ];              
         (* Set up RC, etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"C"];
         f1=ToExpression["ColourMaths`"<>cletter<>"C1"];
         ff=ToExpression["ColourMaths`Coalesce"<>cc];
         ff1=ToExpression["ColourMaths`Coalesce"<>cc<>"AndClear"];              
         MessageName[Evaluate[f],"usage"]=
         "Function to merge all "<>ccl<>" parts of an expression";              
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];              
         MessageName[Evaluate[f1],"usage"]=
         "Function to merge all "<>ccl<>
         "parts of an expression, colour destroyed";              
         MessageName[Evaluate[ff1],"usage"]=MessageName[Evaluate[f1],"usage"];
         With[{f=f,f1=f1,ff=ff,ff1=ff1,c=c},
            f=ff;
            f1=ff1;
            ff[x_]:= colourmerge[c,x];
            ff1[x_]:= colourmerge[c,x]/.c[u_]->u;
         ];              
         (* Set up CR, etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"I"];
         ff=ToExpression["ColourMaths`Insert"<>cc];              
         MessageName[Evaluate[f],"usage"]= 
         "Function to insert "<>ccl<>" colour within an expression\nFor example, f[a+b,c]+g[x]//C"<>
         cletter<>"[f[\[SelectionPlaceholder],\[Placeholder]]]\n would colour 'a+b' "<>
         cc<>".";              
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         With[{f=f,ff=ff,c=c},
            f=ff;
            ff[x_]:= colourinsert[c,x];
            ff1[x_]:= colourinsert[c,x]/.c[u_]->u;
         ];    
         (* Set up MR, etc. *)
         f=ToExpression["ColourMaths`M"<>cletter];
         ff=ToExpression["ColourMaths`Make"<>cc];                  
         MessageName[Evaluate[f],"usage"]= 
         "Function to make an expression "<>ccl<>" coloured within an expression\nFor example, f[a+b,c]+g[x]//M"<>
         cletter<>"[f[\[Placeholder],\[Placeholder]]]\n would colour 'f[a+b,c]' "<>
         cc<>".";                  
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         With[{f=f,ff=ff,c=c},
            f=ff;
            ff[x_]:= colourmake[c,x];
            ff1[x_]:= colourmake[c,x]/.c[u_]->u;
         ];
         
         (* Set up RPart, etc. *)
         (*f=ToExpression["ColourMaths`"<>cletter<>"Part"];*)
         ff=ToExpression["ColourMaths`"<>cletter<>"Part"];                  
         MessageName[Evaluate[f],"usage"]= 
         "Function to reduce an expression "<>ccl<>" to a Part of that expression. \nFor example, f[a+b,c]+g[x]//RPart[1]";
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         With[{(*f=f,*)ff=ff,c=c},
            (* f=ff;*)
            ff[x___]:= colourpart[c,x];            
         ];
         
         (* Set up Rover etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"over"];
         f1=ToExpression["ColourMaths`"<>cletter<>"over1"];
         ff=ToExpression["ColourMaths`Move"<>cc<>"Over"];
         ff1=ToExpression["ColourMaths`Move"<>cc<>"OverAndClear"];                  
         MessageName[Evaluate[f],"usage"]=
         "Move "<>ccl<>
         " expression over to the other side of an equation or inequality";
         
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];                  
         MessageName[Evaluate[f1],"usage"]=
         "Move "<>ccl<>
         " expression over to the other side of an equation or inequality, colour destroyed";                  
         MessageName[Evaluate[ff1],"usage"]=MessageName[Evaluate[f1],"usage"];
         With[{f=f,f1=f1,ff=ff,ff1=ff1,c=c},
            f=ff;
            f1=ff1;
            ff[x_]:= moveOver[x,c,c];
            ff1[x_]:= moveOver[x,c,Identity];
         ];      
         (* Set up Rgrow etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"grow"];
         ff=ToExpression["ColourMaths`"<>cc<>"Grow"];      
         MessageName[Evaluate[f],"usage"]=
         "Expand "<>ccl<>" expression to immediately containing expression";      
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         With[{f=f,ff=ff,c=c},
            f=ff;
            ff[x_]:=x/.g_[a___,c[b_],d___]->c[g[a,b,d]];
         ];      
         (*\[LightBulb]Radjust*) (* Set up Radjust etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"adjust"];
         ff=ToExpression["ColourMaths`"<>cc<>"Adjust"];      
         MessageName[Evaluate[f],"usage"]=
         "Adjust "<>cc<>" expression) "<>cc<>"[expr] - expr (adjust calls are normally created by the Adjust button)";      
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         With[{f=f,ff=ff,c=c},
            f=ff;
            With[{cc1=colour\[Breve]applier}, ff[x__]:=cc1[Do\[Breve]Adjustment[x]]];
         ];      
         (* Set up Radd etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"add"];
         ff=ToExpression["ColourMaths`"<>cc<>"Add"];      
         MessageName[Evaluate[f],"usage"]=
         "Add to "<>cc<>" expression(s) "<>cc<>"[expr] - expr";      
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         With[{f=f,ff=ff,c=c},
            f=ff;
            ff[x_]:=cadd[x,c];
         ];                  
         (* Set up Rmpy etc. *)
         f=ToExpression["ColourMaths`"<>cletter<>"mpy"];
         ff=ToExpression["ColourMaths`"<>cc<>"Multiply"];                  
         MessageName[Evaluate[f],"usage"]=
         "Multiply "<>cc<>" expression(s) by "<>cc<>"[expr]/expr";      
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         With[{f=f,ff=ff,c=c},
            f=ff;
            ff[x_]:=cmultiply[x,c];
         ];                  
         (* Set up Rin, etc. (Single colour case) *)
         If[cletter=="M",
            f=ToExpression["ColourMaths`"<>cletter<>"inside"],
            f=ToExpression["ColourMaths`"<>cletter<>"in"]
         ];
         f1=ToExpression["ColourMaths`"<>cletter<>"in1"];
         ff=ToExpression["ColourMaths`Move"<>cc<>"In"];
         ff1=ToExpression["ColourMaths`Move"<>cc<>"InAndClear"];
         
         MessageName[Evaluate[f],"usage"]="Move "<>ccl<>" expression inside";
         
         MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
         
         MessageName[Evaluate[f1],"usage"]=
         "Move "<>ccl<>" expression inside colour destroyed";
         
         MessageName[Evaluate[ff1],"usage"]=MessageName[Evaluate[f1],"usage"];
         With[{f=f,f1=f1,ff=ff,ff1=ff1,c=c},
            f=ff;
            f1=ff1;
            ff[x_]:= moveIn[x,c,Identity,c];
            ff1[x_]:= moveIn[x,c,Identity,Identity];
         ];
         
         (* Set up RGin, etc. (Double colour case) *)
         Module[{k,c2,cletter2,cc2,cc12},
            k=1;
            While[k <= Length[all\[Breve]colours],
               c2=all\[Breve]colours[[k,1]];
               cletter2=all\[Breve]colours[[k,3]];
               cc2=SymbolName[c2];
               ccl2=ToLowerCase[cc2];
               
               f=ToExpression["ColourMaths`"<>cletter<>cletter2<>"in"];                                    
               f1=ToExpression["ColourMaths`"<>cletter<>cletter2<>"in1"];                                    
               ff=ToExpression["ColourMaths`Move"<>cc<>"Inside"<>cc2];                                    
               ff1=ToExpression["ColourMaths`Move"<>cc<>"Inside"<>cc2<>"AndClear"];                                    
               MessageName[Evaluate[f],"usage"]=
               "Move "<>ccl<>" expression inside "<>ccl2;                                    
               MessageName[Evaluate[ff],"usage"]=MessageName[Evaluate[f],"usage"];
               MessageName[Evaluate[f1],"usage"]=                                      
               "Move "<>ccl<>"  expression inside  "<>ccl2<>", colour destroyed";                                    
               MessageName[Evaluate[ff1],"usage"]=MessageName[Evaluate[f1],"usage"];
               With[{f=f,f1=f1,ff=ff,ff1=ff1,c=c,c2=c2},
                  f=ff;
                  f1=ff1;
                  ff[x_]:= moveIn[x,c,c2,c];                                             
                  ff1[x_]:= moveIn[x,c,c2,Identity];
               ];                                    
               (* Entry in sort table *)
               With[{c=c,j=j},
                  cto[c[x_]]:=f[j,x];
                  colouredExpressionQ[c[_]]:=True;                                             
                  colourRemovalRules=Append[colourRemovalRules,c[x_]->x];
               ];                                    
               k++;
            ]
         ];
      ];
      j++;
   ];
   (* Final case for sort table *)
   cto[x_]:=f[j,x];
   colouredExpressionQ[_]:=False;
];


(* ::Section:: *)
(*Utility functions*)


Clear[Error];
SetAttributes[Error,HoldAll];
Error[mes_,vars___]:=Module[{},
   Message[mes,vars];
   Abort[];
];

Clear[SiftOptions];
SetAttributes[SiftOptions,HoldFirst];
SiftOptions[specs_List,opts_List,parent_]:=Module[{k,optionset,s,clr},
   SetAttributes[clr,HoldFirst];
   clr[{a_,b_}]:=Unset[b];
      		optionset=
   Join[Complement[Options[parent],opts,SameTest->(#1[[1]]===#2[[1]]&)],
      opts];
      					ReleaseHold[Map[clr,Hold[specs],{2}]];
   k=0;
   While[k < Length[specs],
            	  k++;
      s=Options[specs[[k,1]]]/.{(a_->b_)->(a->_),(a_:>b_)->(a:>_)};
      Evaluate[specs[[k,2]]]=Cases[optionset,Apply[Alternatives,s]];
      optionset=DeleteCases[optionset,Apply[Alternatives,s]];
   ];
      	If[optionset =!={},
      parent::badarg="`1` is an invalid option to "<>ToString[parent];
      Message[parent::badarg,optionset[[1,1]]]]
];




Left\[Breve]Side[str_String]:=Left\[Breve]Side[Get\[Breve]Highlight\[Breve]Formula[str]];
Left\[Breve]Side[op_[a_,b_]]:=a/;MemberQ[{Equal,Unequal,GreaterEqual,Greater,Less,LessEqual},op];

Right\[Breve]Side[str_String]:=Right\[Breve]Side[Get\[Breve]Highlight\[Breve]Formula[str]];
Right\[Breve]Side[op_[a_,b_]]:=b/;MemberQ[{Equal,Unequal,GreaterEqual,Greater,Less,LessEqual},op];


RtoF[Rule_]:=Function[Z,Z/.Rule];
RRtoF[Rule_]:=Function[Z,Z//.Rule];


(* ::Section:: *)
(*Highlighted formulae*)


(* Note that these definitions deliberately do not match! *)
MakeBoxes[Highlight\[Breve]Formula[name_String,expr_],StandardForm]:=TagBox[
   StyleBox[RowBox[{StyleBox["\""<>name<>"\"",FontSize->17(*,FontColor->Darker[Green]*)],"\n",
            TagBox[MakeBoxes[expr,StandardForm],HR2]}](*,Background->LightGreen*)],HR1[name]];

MakeExpression[TagBox[expr_,HR1[_]],StandardForm]:=MakeExpression[
   Part[Cases[expr,TagBox[_,HR2],\[Infinity]],1,1],StandardForm];

Get\[Breve]Highlight\[Breve]Formula[name_String]:=Get\[Breve]Highlight\[Breve]Formula[name,SelectedNotebook[]];
Get\[Breve]Highlight\[Breve]Formula[name_String,nb_]:=Module[{},
   xx=NotebookGet[nb];
   xx=Cases[xx,TagBox[_,HR1[name]],\[Infinity]];
   If[xx=={},Abort[]];
   ToExpression[Cases[xx,TagBox[expr_,HR2]->expr,\[Infinity]][[1]]]
];


Set\[Breve]Highlight\[Breve]Formula[]:=Module[{nb,str,xx,colour},
   nb=InputNotebook[];
   (*Note this seems to be the ideal way to read input from a dialog window *)
   str="";
   colour=LightGreen;
   DialogInput[{TextCell["Equation name: "],
         InputField[Dynamic[str],String],
         "Select background colouration",
         Row[
            Map[Button["",colour=#,Background->#,ImageSize->{30,30}]&,
               {LightGreen,LightBlue,LightYellow,Pink}
            ]
         ],
         DefaultButton[DialogReturn[{}]]
      }
   ];
   If[(! StringQ[str]) || str=="",Return[]];
   SelectionMove[nb,All,CellContents];
   xx=Highlight\[Breve]Formula[str,ToExpression[NotebookRead[nb],StandardForm]];
   NotebookWrite[nb,ToBoxes[xx]];
   SelectionMove[nb,All,Cell];
   SetOptions[NotebookSelection[nb],CellFrame->True,Background->colour];
];


Pick\[Breve]Highlight\[Breve]Formula[nbx_:Null]:=Module[{nb=nbx,xx},
   If[nb===Null,nb=SelectedNotebook[]];
   xx=NotebookGet[nb];
   xx=Cases[xx,TagBox[_,HR1[y_]]->y,\[Infinity]];
   If[xx==={},Abort[]];
   xx=ChoiceDialog["Select formula:",Map[#->#&,xx]];
   Get\[Breve]Highlight\[Breve]Formula[xx,nb]  
];


(* ::Section:: *)
(*Classification of operations*)


(*\[LightBulb]Operation\[LetterSpace]classification*)ProtectedMathOperationQ[Integrate\[Breve]Inert]=True;
ProtectedMathOperationQ[Sum\[Breve]Inert]=True;
ProtectedMathOperationQ[Product\[Breve]Inert]=True;
ProtectedMathOperationQ[Limit\[Breve]Inert]=True;
ProtectedMathOperationQ[Val1]=True;
ProtectedMathOperationQ[_]=False;

MathOperationQ[Integrate\[Breve]Inert]=True;
MathOperationQ[Sum\[Breve]Inert]=True;
MathOperationQ[Product\[Breve]Inert]=True;
MathOperationQ[Limit\[Breve]Inert]=True;
MathOperationQ[Val1]=True;

(*MathOperationQ[Integrate]=True;
MathOperationQ[Sum]=True;
MathOperationQ[Product]=True;
MathOperationQ[Limit]=True;
*)
MathOperationQ[_]=False;

(*LinearMathOperationQ[Product]=False;*)
LinearMathOperationQ[Product\[Breve]Inert]=False;
LinearMathOperationQ[O_]:=MathOperationQ[O]



(* ::Section:: *)
(*Ellipsis form*)


Clear[UseEllipses,e\[Breve]string];

UseEllipses[U_[A_,{k_,a_,b_}],n_]:=Module[{j},
   Which[a==-\[Infinity] && b==\[Infinity],
      EllipsisTag[Flatten[{
               " \[Ellipsis] ",
               Table[A/.k->j,{j,-n,n}],
               " \[Ellipsis] ",
               A,
               " \[Ellipsis] "
            }],U],
      
      b==\[Infinity],
      EllipsisTag[Flatten[{
               Table[A/. k:>a+j,{j,0,n-1}],
               " \[Ellipsis] ",
               A,
               " \[Ellipsis] "
            }],U],
      
      a==-\[Infinity],
      EllipsisTag[Flatten[{
               " \[Ellipsis] ",
               Table[A/. k:>b-n+1+j,{j,0,n-1}]
            }],U],
      
      True,
      If[a+4>b,Return[U[A,{k,a,b}]]];
      EllipsisTag[Flatten[{
               Table[A/. k:>a+j,{j,0,n-1}],
               " \[Ellipsis] ",
               A,
               " \[Ellipsis] ",
               Table[A/. k:>b-n+1+j,{j,0,n-1}]
            }],U]
   ]
];


Options[EllipsisForm]={"Repeat"->4};
EllipsisForm[x_,OptionsPattern[EllipsisForm]]:=Module[{n},
   n=OptionValue["Repeat"];
   x/.{t_Product\[Breve]Inert:>UseEllipses[t,n],t_Sum\[Breve]Inert->UseEllipses[t,n]}
];
(*Subscript[EllipsisForm, k_Integer][x_]:=x /.{t_Product\[Breve]Inert:>UseEllipses[t,k],t_Sum\[Breve]Inert->UseEllipses[t,k]};
EllipsisForm[k_Integer][x_]:=x /.{t_Product\[Breve]Inert:>UseEllipses[t,k],t_Sum\[Breve]Inert->UseEllipses[t,k]};*)

mb[-x_,_]:={"-",MakeBoxes[x,StandardForm]};
mb[x_,{1}]:=MakeBoxes[x,StandardForm];
mb[x_,_]:={"+",MakeBoxes[x,StandardForm]};
mp[x_String]:=MakeBoxes[x,StandardForm];
mp[x_]:=RowBox[{"(",MakeBoxes[x,StandardForm],")"}];
MakeBoxes[EllipsisTag[s_List,Sum\[Breve]Inert],StandardForm]:=Module[{},
   RowBox[Flatten[MapIndexed[mb,s]]]
];
MakeBoxes[EllipsisTag[s_List,Product\[Breve]Inert],StandardForm]:=Module[{},
   RowBox[Flatten[Map[mp,s]]]
];



Clear[ColourTransform,ct1];
trNames={};
ColourTransform::badargs="ColourTransform can only be applied to a rule - not to `1`";\

ColourTransform[C_,{},C_]:=Function[D2,D1];
ColourTransform[C_,{},C1_]:=
  Function[D2,D1]/. D1:>(D1/.C[x_]->C1[x]);
ColourTransform[C_,{F_,T___},C1_]:=ct1[ColourTransform[C,{T},C1],C,F];
ct1[x_,C_,a_->b_]:=
  x/. D1:>(D1/.C[u_] :> C[u/.a->b]);
ct1[x_,C_,a_:>b_]:=
  x/. D1:>(D1/.C[u_] :> C[u/.a:>b]);
ct1[x_,C_,(a_Symbol | a_Function)]:=
  x/. (D1 :> (D1/.C[y_]:>C[a[y]]));
ct1[x_,C_,a_String]:=
  Module[{k,s},
    With[{k=k},
        x/. (D1 :> (D1/.C[y_]:>
                    C[DBTransform[k,y]]))]/;
       (s=Cases[trNames,{n_,a}->n];If[s==={},False,k=s[[1]];True])];
ct1[_,_,a_]:= Error[ColourTransform::badargs,a];

noSpell[vl_String]:=Module[{},
       Off[General::"spell1"];
       Off[General::"spell"];
       Symbol[vl];
       On[General::"spell1"];
       On[General::"spell"];
       ];
noSpell[vl_List]:=(Map[noSpell,vl];Null);
noSpell[a_,b__]:=(Map[noSpell,{a,b}];Null);



Diff::trouble="Cannot differentiate with respect to a dummy variable";
Diff[Integrate\[Breve]Inert[f_,{x_,_,_}],x_]:=Error[Diff::trouble];
Diff[Sum\[Breve]Inert[f_,{x_,_,_}],x_]:=Error[Diff::trouble];
Diff[Integrate\[Breve]Inert[f_,l_],x_]:=Integrate\[Breve]Inert[Diff[f,x],l];
Diff[Sum\[Breve]Inert[f_,l_],x_]:=Sum\[Breve]Inert[Diff[f,x],l];
Diff[f_,x_]:=Module[{l,l1,l2,f1},
			l=Cases[f,_Integrate\[Breve]Inert | _Sum\[Breve]Inert,\[Infinity]];
		l1=Map[#->Module[{v},v[x]]&,l];
		l2=Map[q[Diff[#[[2]],x]->Diff[#[[1]],x],#[[2]]->#[[1]]]&,l1]/.q:>Sequence;
	f1=f/.l1;
		D[f1,x]/.l2
		]



Clear[dbGetSelectedExpression];
dbGetSelectedExpression[nb_]:=Module[{s,s1,s2,adjusted=False},
        While[True,              
      s=NotebookRead[nb] //. BoxData[x_] -> x//.FormBox[x_,_]->
              x //.StyleBox[x_,___]->x;
              If[SameQ[s,{}],Return[emptyexpression]];
              (*Return the box expression as a side effect *)
              boxExpression=s;
              s1=MakeExpression[s,StandardForm];
              s2=Evaluate[s1];
              If[! MatchQ[s2,ErrorBox[___]],Return[ReleaseHold[s2]]];
              If[adjusted,Beep[];Return[emptyexpression]];
              SelectionMove[nb,All,Expression];
              adjusted=True;
              ]
       ]


Clear[dbGetSelectedExpression1];
dbGetSelectedExpression1[nb_]:=Module[{s,s1,s2,adjusted=False},
       While[True,
              
      s=NotebookRead[nb]//. BoxData[x_] -> x//.FormBox[x_,_]->
              x //.StyleBox[x_,___]->x;
              (*Return the box expression as a side effect *)
              boxExpression=s;
              s2=MakeExpression[s,StandardForm];
              If[! MatchQ[s2,ErrorBox[___]],Return[s2]];
              If[adjusted,Beep[];Return[emptyexpression]];
              SelectionMove[nb,All,Expression];
              adjusted=True;
              ]
       ]


  MakeExpression[x_?mePrint,Form_]:=False;


MakeBoxes[Val1[A_,x_,a_],f_]:=Module[{ans1},
    ans1=TagBox[
        SubscriptBox[RowBox[{"[",MakeBoxes[A,f],"]"}],
          RowBox[{MakeBoxes[x,f],"=",MakeBoxes[a,f]}]],Val1];
    If[m===1,ans1,RowBox[{MakeBoxes[m,f],ans1}]]
    ]

MakeExpression[
    TagBox[SubscriptBox[RowBox[{"[",A_,"]"}],RowBox[{x_,"=",v_}]],Val1],f_]:=
  MakeExpression[RowBox[{"Val1","[",A,",",x,",",v,"]"}],f]

(*Clear[DBPasteOperation];
DBPasteOperation[x_]:=Module[{nb},
		nb=InputNotebook[];
		SelectionMove[nb,"After","CellContents"];
			NotebookApply[nb,"//"];
		NotebookApply[nb,x]
	]*)

Clear[DBModify];
SetAttributes[DBModify,HoldAll];DBModify[Congruent[A_,B_],P_] :=      
              Congruent[A//P,B//P]

Clear[Integrate\[Breve]Inert];
Integrate\[Breve]Inert[A_,{x_,a_,b_}]:= 
  A (b-a) /; (FreeQ[A,x] && FreeQ[A,Blank] && FreeQ[A,Q]);
Integrate\[Breve]Inert[A_,{x_,a_,
      b_}]:= -Integrate\[Breve]Inert[A,{x,b,a}] /;(FreeQ[a,Complex]&&FreeQ[b,Complex] && 
        b < a);
Integrate\[Breve]Inert[-A_,{x_,a_,b_}]:= -Integrate\[Breve]Inert[A,{x,a,b}];

invertprefer[-x_,x_]:=False;
invertprefer[x_,-x_]:=True;
invertprefer[a_,b_]:=OrderedQ[b,a]

Clear[invertsubstitution];
invertsubstitution[u_,us_,x_]:=
Sort[x/.
              (* Setting InverseFunctions to True suppresses warnings *)
            Solve[u==us,x,InverseFunctions->True]
              ,invertprefer][[1]]



Clear[Iv];
Clear[Iv1];
  Iv::impossible="This substitution is of the wrong form"
  Change\[Breve]Variables[x___]:=(Iv1[#,x]&);
  (* Complex case newvar=f[oldvar] *)
  Iv1[Integrate\[Breve]Inert[A_,{x_,a_,b_}],u_-> us_,x_]:= Module[{dux,xs},
               dux=D[us,x]/.x->xs;
               xs=invertsubstitution[u,us,x];
               Integrate\[Breve]Inert[(A/.x->xs)/dux,{u,us/.x->a,us/.x->b}]
               ]/;(! FreeQ[{us,a,b},Complex]);
  
  (* Real case newvar=f[oldvar] *)
  Iv1[Integrate\[Breve]Inert[A_,{x_,a_,b_}],u_Symbol-> us_,x_]:= Module[{dux,xs},
             dux=D[us,x]/.x->xs;
             xs=invertsubstitution[u,us,x];
             
      Integrate\[Breve]Inert[(A/.x->xs)/dux,{u,Limit[us,x->a,Direction->-1],
          Limit[us ,x->b,Direction->1]}]
             ];
  
  (* Real case oldvar=f[newvar] *)
  Iv1[Integrate\[Breve]Inert[A_,{x_,a_,b_}],x_Symbol-> xs_,u_]:= Module[{dxu,us},
       dxu=D[xs,u];      
       us=invertsubstitution[x,xs,u];
             
      Integrate\[Breve]Inert[(A/.x->xs)dxu,{u,Limit[us,x->a,Direction->-1],
          Limit[us ,x->b,Direction->1]}]
             ];
  Iv1[___]:=(Message[Iv::impossible];Abort[]);




swappable\[Breve]heads={Integrate\[Breve]Inert,Sum\[Breve]Inert,Limit\[Breve]Inert};

SwapOperations[S_[m_. S1_[A_,x1_],x_]]:=
  S1[S[m A,x],
        x1]/;MemberQ[swappable\[Breve]heads,S]&&MemberQ[swappable\[Breve]heads,S1]&&((! ListQ[x]) || 
        FreeQ[x1,x[[1]]])&&FreeQ[m,x1[[1]]];

(* These handle cases like the W.N.Bailey transforms *)
SwapOperations[Sum\[Breve]Inert[Sum\[Breve]Inert[A_,{n_,0,m_}],{m_,0,\[Infinity]}]]:=
  Sum\[Breve]Inert[Sum\[Breve]Inert[A,{m,n,\[Infinity]}],{n,0,\[Infinity]}];
SwapOperations[
    Sum\[Breve]Inert[Sum\[Breve]Inert[A_,{m_,n_,\[Infinity]}],{n_,0,\[Infinity]}]]:=
  Sum\[Breve]Inert[Sum\[Breve]Inert[A,{n,0,m}],{m,0,\[Infinity]}];

SwapOperations[x_]:=x;



(* ::Section:: *)
(*Integration by parts*)


SetAttributes[integrate1,HoldFirst];
(* These rules circumvent a bug in Mathematica which handles these wrongly*)
integrate1[D[f_,x_],x_]:=f;
integrate1[D[f_,{x_,k_}],x_]:=D[f,{x,k-1}];
integrate1[f_,x_]:=Integrate[f,x];
ByParts[Integrate\[Breve]Inert[A_. Red\[Breve]Colour[v_],{x_,a_,b_}]]:=Module[{s1},
       s1=integrate1[v,x];
       Limit[s1 A,x->b]-Limit[s1 A,x->a]-Integrate\[Breve]Inert[s1 D[A,x],{x,a,b}]
       ]
ByParts[Integrate\[Breve]Inert[A_,{x_,a_,b_}],v_]:=Module[{s,s1},
       s=Simplify[A/v];
       s1=integrate1[v,x];
       Limit[s1 s,x->b]-Limit[s1 s,x->a]-Integrate\[Breve]Inert[s1 D[s,x],{x,a,b}]
       ];


AllPossibleReplacements[expr_,lhs_,rhs_]:=
  Module[{counter,counter1,s,ans},
    counter=0;
    ans={};
    While[True,
      counter1=counter;
      s=expr/.lhs:>rhs/;(counter1--)==0;
      If[s===expr,Break[]];
      ans=Append[ans,
          s];
      counter++;
      ];
    ans
    ]

Assertion::notfound="Assertion `` not found";
Assertion[fact_]:=Module[{aa=Asserted[fact]},
    If[Head[aa]===Asserted,Message[Assertion::notfound,fact];Abort[]];
    (aa[[1]]==
        aa[[2]])
    ];

Use::badarg="This assertion does not exist";
Use::nomatch="Unable to apply this asertion";
specialUse=False;
Use[R_,rules___Rule]:=Use[#1,R,{rules}];
Use[expr_,R_,rules_]:=Module[{s,ans,use1},
       s=Asserted[R]/.rules;
       If[specialUse,
              use1=Use1s,
              use1=Use1
              ];
       If[Head[s]===Asserted,Error[Use::badarg]];
       If[(ans=use1[s,expr])=!=Null,ans,
              
          If[(ans=use1[{s[[2]],
                      s[[1]],
                      s[[3]],
                      Sequence@@Drop[s,3]},expr])=!=Null,ans,
                     If[!specialUse,Error[Use::nomatch]];expr
                     ]
              ]
       ]& 



Clear[Use3];(* Note here that NumericQ works for stuff like Sqrt[\[Pi]) *)
Use3[1]:=Null;
Use3[a_?NumericQ +b_]:=((#-a)&);
Use3[a_?NumericQ* b_]:=((#/a)&);
Use3[_]:=Null;

Clear[Use2];
Use2[s_]:=Module[{lhs,rhs,vars,comments,cond=True,a,b,c,g,n,k,m,nv,trivialF},
       lhs=s[[1]];
       rhs=s[[2]];
       While[True,
              trivialF=Use3[lhs];
              If[trivialF==Null,Break[]];
              lhs=trivialF[lhs];
              rhs=trivialF[rhs];
              ];
       vars=s[[3]];
       comments=Drop[s,3];
       k=Length[vars];
       (* Loop over variables *)
       While[k > 0,
              n=vars[[k]];
              If[FreeQ[lhs,Integrate\[Breve]Inert[_,{n,_,_}]],
                     (* 
          Get cases like 2n+1 and map one such case to ensure each variable \
can be found inside lhs in its raw state.
                     
                Note that this is made much more complex because Cases can \
match with or without optional terms *)
                     lhs=lhs/.a_. n+b_. ->g[a n+b];
                     
        c= Cases[lhs,
            g[a_. n+b_.]:>{a,
                  b}/;(NumberQ[a]&&
                    NumberQ[
                      b]&&(a !=1 || 
                        b != 0)),{1,\[Infinity]}];
                     If[c =!={},
                            
          lhs=lhs/.n->(n/c[[1,1]]-
                    c[[1,2]]/
                      c[[1,1]]);
                            
          lhs=lhs/. g[x_]:>g[Distribute[Simplify[x]]];
                            
          rhs=rhs/.n->(n/c[[1,1]]-
                    c[[1,2]]/
                      c[[1,1]])];
                     (* Add extra variables to match linear terms*)
                     
        c=Cases[lhs,
            g[a_. n+b_.]:>{a,
                  b}/;(NumberQ[a]&&
                    NumberQ[
                      b]&&(a !=1 || 
                        b != 0)),{1,\[Infinity]}];
                     lhs=lhs/.g->Identity;
                     m=Length[c];
                     While[m>0,
                            
          With[{nv=Unique[v],a=c[[m,1]],
              b=c[[m,2]],n=n},
                                   lhs=lhs/.n a+b->nv;
                                   vars=Append[vars,nv];
                                   cond=cond&&Hold[Simplify[(a n+b)-nv]==0]
                                   ];
                            m=m-1
                            ];
                     ];
              k=k-1;
              ];
       With[{cond=cond},{lhs,rhs/;cond,vars}]/.Hold[x_]->x
       ]



writeElement[str_,x_]:=Module[{},
		WriteString[str,"ColourMaths`Private`Asserted["];
				WriteString[str,
      ToString[x[[1]],InputForm,CharacterEncoding->None]];		
		WriteString[str,"]:=\n   "];
    WriteString[str,ToString[x[[2]],InputForm,CharacterEncoding->None]];
		WriteString[str,"\n\n"]
	];

dumpAssertions[]:=Module[{str},
		str=OpenWrite["c:\\maths\\packages\\formulae.m"];
	    Map[writeElement[str,#]&,
      DownValues[ColourMaths`Private`Asserted]/.HoldPattern->
            Hold/.Hold[_[x_]]->x];
		Close[str]
	];

(*DoIntegrals := RRtoF[(Integrate\[Breve]Inert[A_,{x_,a_,b_}] :>Integrate[A,{x,a,b}]) ];*)
DoIntegrals[x_]:=(x/.Integrate\[Breve]Inert->Integrate);

EvalIntegrals[subs___] := 
  RRtoF[(Integrate\[Breve]Inert[A_,{x_,a_,b_}] :>NIntegrate[A/.{subs},{x,a,b}]) ];

(*DoSummations := RRtoF[(Sum\[Breve]Inert[A_,{x_,a_,b_}] :>Sum[A,{x,a,b}]) ];*)
DoSummations[x_]:=(x/.Sum\[Breve]Inert->Sum);

(*DoProducts := RRtoF[(Product\[Breve]Inert[A_,{x_,a_,b_}] :>Product[A,{x,a,b}]) ];*)
DoProducts[x_]:=(x/.Product\[Breve]Inert->Product);

(*DoLimits:=RRtoF[Limit\[Breve]Inert[A_,a_->b_]:>Limit[A,a->b]];*)
DoLimits[x_]:=(x/.Limit\[Breve]Inert->Limit);

DoInerts[x_]:=(x/.{Integrate\[Breve]Inert->Integrate,x/.Sum\[Breve]Inert->Sum,Product\[Breve]Inert->Product,Limit\[Breve]Inert->Limit});



colourinsert[C_,x_]:=Module[{xx,xxr,v,pat,ans},
   (* pD\[Breve]Suppress\[Breve]Level++;*)           
   xx=x/.Blank->pat/.pat[]->Global`\[Placeholder] /.{ 
      Global`\[Placeholder]:>pat[Unique[v],Blank[]],
      Global`\[SelectionPlaceholder]->pat[v,Blank[]]};
   xxr=xx/.{ pat[v,Blank[]]->C[v],pat[s_,_]->s};
   xx=xx/.pat->Pattern;
   
   ans= With[{xx=xx,xxr=xxr},
      Function[u,u/.xx:>xxr]]/.C[v]:>
   C[v/.colourRemovalRules];
   (* pD\[Breve]Suppress\[Breve]Level--;*)
   ans
];


colourpart[C_,pp___]:=Module[{u},Function[u,colourpart1[u,C,pp]]]


colourpart1[exprx_,C_,pp___]:=Module[{p,p1,expr=exprx},
   p=Position[expr,_C];
   If[p=!={},
    p=First[p];
    (*p=First[p];*)
   p1=Join[p,{pp}];
   Part[expr,Sequence@@p]=Part[expr,Sequence@@p,1];
   Quiet[Check[Part[expr,Sequence@@p1]=C[Part[expr,Sequence@@p1]],Return[$Failed]]],
    Quiet[Check[Part[expr,pp]=C[Part[expr,pp]],Return[$Failed]]];
   ];
  expr
];


colourmake[C_,x_]:=Module[{xx,xxr,v,pat,ans},
(*    pD\[Breve]Suppress\[Breve]Level++;*)
    xx=x/.Blank->pat/.pat[]->Global`\[Placeholder] /.{ 
          Global`\[Placeholder]:>pat[Unique[v],Blank[]]};
           xxr=C[xx/.{ pat[v,Blank[]]->v,pat[s_,_]->s}];
           xx=xx/.pat->Pattern;
           
    ans= With[{xx=xx,xxr=xxr},
          Function[u,u/.xx:>xxr]]/.C[v]:>
          C[v/.colourRemovalRules];
(*    pD\[Breve]Suppress\[Breve]Level--;*)
    ans
           ]



Clear[colourspread];
colourspread[C_,x_]:=
	x/. {C[a_] b_->C[a b],C[a_]+b_->C[a+b],
			F_[C[a_]]->C[F[a]]}



Clear[colourmerge,cm1,cm2,cm3];
(*\[LightBulb]colourmerge*)colourmerge[C_,x_]:=Module[{Flag},colourmerge[C,x,Flag]];
colourmerge[C_,x_?AtomQ,Flag_]:= x;
colourmerge[C_,C_[x_],Flag_]:= (cm3[Flag];C[x/.C[a_]->a]);
colourmerge[C_,Alternatives[x_Plus,x_Times],Flag_] :=
	Module[{Flag1,ans},
	ans=Map[cm1[C,#1]&,x];
		Switch[Count[ans,f[True,_]],
			0,Map[#1[[2]]&,ans],
			1,cm3[Flag];Map[#1[[2]]&,ans],
				_,cm3[Flag];
      Map[cm2[C,#1]&,ans/.C[u_]->u]//.C[a_]+C[b_]->C[a+b]//.C[a_]C[b_]->
          C[a b]
	]];	 
colourmerge[C_,F_[x___],Flag_]:=
		Module[{Flag1,ans},
		 ans=Map[colourmerge[C,#1,Flag1]&,F[x],Heads->True];
		If[Flag1,cm3[Flag]];
		ans];
cm1[C_,x_] := Module[{Flag,ans},
		ans=colourmerge[C,x,Flag];
		If[Flag,Return[f[True,ans]]];
		f[False,ans]
	];
cm2[C_,f[True,x_]]:=C[x];
cm2[C_,f[_,x_]]:=x;
cm3[True]:=True;
cm3[x_]:=(x=True);

cadd[expr_,C_]:=Function[u,u/.C[s_]->C[s +expr]- expr];

cmultiply[expr_,C_]:=Function[u,u/.C[s_]->C[s expr]/expr];



(* ::Section:: *)
(*moveIn*)


(*\[LightBulb]moveIn*)Clear[moveIn];

moveIn[z_,C_,C1_,C2_] :=Module[{},
z/.{
       C[Log[e_]]+C1[Log[e1_]]:>Log[C[e]e1],       
       C[Log[e_]]-C1[Log[e1_]]:>Log[C[e]/e1],       
       C1[Log[e_]]-C[Log[e1_]]:>Log[e/C[e1]],

(* Merge the product of two integrals to form a double integral *)
       
       C[Integrate\[Breve]Inert[t1_,l1_]]C1[S_[t2_,l2_]]:>C[Integrate\[Breve]Inert[Integrate\[Breve]Inert[t1 t2,l2],l1]]/;
       (FreeQ[t1,l2[[1]]]&&FreeQ[t2,l1[[1]]] && l1[[1]]=!=l2[[1]]),

(* Merge the sum of two sums that use the same index *)

       m1_. C[Sum\[Breve]Inert[t1_,l1_]]+m2_. Sum\[Breve]Inert[t2_,l1_]:>C1[Sum\[Breve]Inert[m1 t1+m2 t2,l1]]/;
           FreeQ[{m1,m2},l1[[1]]],
 
       C[e_]C1[S_[t_,{v_,a_,b_}]]:>
              S[PowerExpand[C[e^(1/(b-a+1))]]t,{v,a,b}]/; ((S===Product || 
                    S=== Product\[Breve]Inert)&&FreeQ[e,v]),
       
       C[e_]^(p_.)C1[S_[t_,l_]]:>S[C[e]^p t,l]/;(MathOperationQ[S]&&FreeQ[e^p,l[[1]]]),
       
       C[S_[t_,l_]]C1[S_[t1_,l_]]:>
              C[S[t t1,l]]/;(S===Product || S===Product\[Breve]Inert),
       
       C[S_[t_,l_]]+C1[S_[t1_,l_]]:>
              C[S[t+t1,l]]/;LinearMathOperationQ[S],
       
       C[S_[t_,l_]]-C1[S_[t1_,l_]]:>
              C[S[t-t1,l]]/;LinearMathOperationQ[S],
       
       C[e_]+C1[S_[t_ ,{l_,a_,b_}]]:>S[t+C[e]/(b-a),{l,a,b}]/;
       (S===Integrate\[Breve]Inert || S===Integrate),
       
       C1[S_[t_ ,{l_,a_,b_}]]-C[e_]:>S[t+C[e]/(a-b),{l,a,b}]/;
       ((S===Integrate\[Breve]Inert || S===Integrate)&&FreeQ[e,l]),
       
       C1[S_[t_ ,l_]]/C[e_]:>S[t/C[e],l]/;
       (LinearMathOperationQ[S]&&FreeQ[e,l[[1]]]),
       
       C[e_]^p_.C1[x_Plus ]^p_.:>Distribute[C[e]x]^p,
       
       C[e_^p_]C1[e1_^p_] :>C[(e e1)^p],
       
       (*  Mapping Sum in this way allows the inside to evaluate and clean up   *)
       C1[x_Plus]^p_./C[e_] ^p_.:>Distribute[(1/C[e])x]^p,

       op1_[C[e_],e1_]:>Map[op1[C1[e],#]&,e1]/;(Quasi\[Breve]Left\[Breve]MultiplicationQ[op1]&&DistributesQ[op1,Head[e1]]),
       op1_[e1_,C[e_]]:>Map[op1[#,C1[e]]&,e1]/;(Quasi\[Breve]Left\[Breve]MultiplicationQ[op1]&&DistributesQ[op1,Head[e1]])
       }/.C->C2 //final\[Breve]cleanup
];


(* ::Section:: *)
(*moveOut*)


(*\[LightBulb]moveOut*)Clear[moveOut];
	moveOut[z_,C_,C1_]:=Module[{},    
   z/.{
      HoldPattern[D[a_ C[b_],{x_,k_}]]:>C1[b]D[a,{x,k}]/;FreeQ[b,x],
      
      Log[a_ C[b_]]:>Log[a]+Log[C1[b]],
      Log[a_ /C[b_]]:>Log[a]-C1[Log[b]],
      
      ((a_)*C[b_])^(p_):>a^p C1[b^p],
      a_^((b_.+C[c_]d1_.)d_.) :> a^(b d)C1[a^(c d d1)],
      	
      HoldPattern[Product\[Breve]Inert[t_ C[e_]^(p_.),{v_,a_,b_}]]:>
      C1[e^(p*(b - a + 1)) ]Product\[Breve]Inert[t,{v,a,b}]/;FreeQ[C[e]^(p),v],
      				
      S_[t_ C[e_]^(p_.),l_]:>C1[e]^p  S[t,l]/;
      				(LinearMathOperationQ[S]&&FreeQ[C[e]^p,l[[1]]]),
      
      Integrate\[Breve]Inert[t_ + C[e_]p_.,{l_,a_,b_}]:>p (b-a)C1[e] +Integrate\[Breve]Inert[t,{l,a,b}]/; FreeQ[p e,l],
      
      HoldPattern[Sum[t_ + C[e_]p_.,{l_,a_,b_}]]:>p (b-a+1)C1[e] +Sum[t,{l,a,b}]/;FreeQ[p e,l],
      Sum\[Breve]Inert[t_ + C[e_]p_.,{l_,a_,b_}]:>p (b-a+1)C1[e] +Sum\[Breve]Inert[t,{l,a,b}]/;FreeQ[p e,l],	
      		
      a_Plus+C[e_]^(p_.)x_. :> C1[e]^p(x+Distribute[a e^(-p)]),      
      a_+C[e_]^(p_.)x_. :> C1[e]^p(x+a e^(-p)),

      op2_[a_,op1_[C[e_],x_]] :> op1[C1[e],op2[x,op1[left\[Breve]inverse[op1][e],a]]]/;(Quasi\[Breve]Left\[Breve]MultiplicationQ[op1]&&DistributesQ[op1,op2]),
      op2_[a_,op1_[x_,C[e_]]] :> op1[op2[op1[a,right\[Breve]inverse[op1][C1[e]]],x],C1[e]]/;(Quasi\[Breve]Right\[Breve]MultiplicationQ[op1]&&DistributesQ[op1,op2])
      	} //final\[Breve]cleanup
];

chooseSimpler[x_,y_]:=If[LeafCount[x]>LeafCount[y],y,x];

tryDistribute[x_]:=Module[{xx=Distribute[x]},chooseSimpler[x,xx]];

Clear[pD,conditionop,moveOver];
conditionop[System`Less]:=True;
conditionop[System`LessEqual]:=True;
conditionop[System`Equal]:=True;
conditionop[System`Unequal]:=True;
conditionop[System`GreaterEqual]:=True;
conditionop[System`Greater]:=True;
conditionop[_]:=False;


(* ::Section:: *)
(*MoveOver*)


moveOver[F_[C_[x_]+b_.,c_],C_,C1_]:=final\[Breve]cleanup[F[b,c-C1[x]]]/;conditionop[F];
moveOver[F_[c_,C_[x_]+b_.],C_,C1_]:=final\[Breve]cleanup[F[c-C1[x],b]]/;conditionop[F];
moveOver[F_[-C_[x_]+b_.,c_],C_,C1_]:=final\[Breve]cleanup[F[b,c+C1[x]]]/;conditionop[F];
(* moveOver[F_[c_,-C_[x_]+b_.],C_,C1_]:=F[c+C1[x],b]/;conditionop[F];   *)
moveOver[F_[(C_)[x_]^(p_.)a_+b_.,c_],C_,C1_]:=
     final\[Breve]cleanup[F[a+b C1[x]^(-p),tryDistribute[c  C1[x]^(-p)]]]/;conditionop[F];
moveOver[F_[c_,(C_)[x_]^(p_.)a_+b_.],C_,C1_]:=
     final\[Breve]cleanup[F[tryDistribute[c C1[x]^(-p)],a+b C1[x]^(-p)]]/;conditionop[F];
moveOver[op_[C_[x_],y_]==z_,C_,C1_]:=final\[Breve]cleanup[(y==op[left\[Breve]inverse[op][C1[x]],z])]/;Quasi\[Breve]Left\[Breve]MultiplicationQ[op];
moveOver[op_[C_[x_],y_,w__]==z_,C_,C1_]:=final\[Breve]cleanup[(op[y,w]==op[left\[Breve]inverse[op][C1[x]],z])]/;Quasi\[Breve]Left\[Breve]MultiplicationQ[op];
moveOver[op_[y_,C_[x_]]==z_,C_,C1_]:=final\[Breve]cleanup[(y==op[z,right\[Breve]inverse[op][C1[x]]])]/;Quasi\[Breve]Right\[Breve]MultiplicationQ[op];
moveOver[op_[w__,y_,C_[x_]]==z_,C_,C1_]:=final\[Breve]cleanup[(op[w,y]==op[z,right\[Breve]inverse[op][C1[x]]])]/;Quasi\[Breve]Right\[Breve]MultiplicationQ[op];
moveOver[x_,_,_]:=final\[Breve]cleanup[x];


Clear[colourOrdered];
(* This table is now built on the fly *)
colourOrdered[x_,y_]:=OrderedQ[{cto[x],cto[y]}];
cs[l_List]:=Sort[l,colourOrdered];
ColourSort[f_:List]:=
  Function[u,u/.f[a___,b_?colouredExpressionQ,c___]:>(f@@cs[{a,b,c}])];
ColourSortAndClear[f_:List]:=
  Function[u,
    u/.f[a___,b_?colouredExpressionQ,
            c___]:>(f@@cs[{a,b,c}])/.colourRemovalRules];


(*\[LightBulb]Setup\[Breve]Extraction*)
Setup\[Breve]Extraction[C_]:=Module[{nb,nb1,s,s1,s2,s3,subst,style},
  nb=InputNotebook[];
  s3=dbGetSelectedExpression[nb];
  If[UnsameQ[s3,emptyexpression],DBColour[C]];
  SelectionMove[nb,All,CellContents];
  s=colourmerge[C,dbGetSelectedExpression[nb]];
  subst=42;
  s1=s /. C[x_] :> (subst=x;Q);
(* Create a temporary notebook in the same style as the parent *)  
  style=Options[nb,StyleDefinitions][[1]][[2]];
  nb1=NotebookCreate[WindowTitle->"Extracted fragment",
        StyleDefinitions->style];
(* Save the details indexed by the temp notebook *)  
  ExtractionRecord[nb1]:={nb,s1};
(* Write a button into the tem notebook to perform the return *)  
NotebookWrite[nb1,Cell[BoxData[MakeBoxes[Button["Return result",Return\[Breve]Extraction[]],StandardForm]],"Print"]];
    s=StyleBox[Apply[MakeBoxes,{subst,StandardForm}]];
(* Write the extracted expression into the temp notebook *)  
  NotebookWrite[nb1,Cell[BoxData[s],Evaluatable->True]];
  SetSelectedNotebook[nb1];
];

Clear[Return\[Breve]Extraction];
Return\[Breve]Extraction[]:=Module[{nb,nb1,s1,s2,s},
(*$$$$$$$  nb1=SelectedNotebook[];     *)
  nb1=InputNotebook[];
  nb=ExtractionRecord[nb1][[1]];
  s1=ExtractionRecord[nb1][[2]];
  s2=s1 /. Q -> %;
  SetSelectedNotebook[nb];
  s=BoxData[Apply[MakeBoxes,{s2,StandardForm}]];
  NotebookWrite[nb,s];
  NotebookClose[nb1];
  ]



(* ::Section:: *)
(*Create Palette *)


(*\[LightBulb]The\[LetterSpace]Palette*)
SetAttributes[P\[Breve]Button,HoldAll];
palette\[Breve]title="ColourMaths version "<>ToString[version];
extra\[Breve]operator\[Breve]buttons={}
Create\[Breve]Colour\[Breve]Palette[]:=Module[{nbs,k,extra\[Breve]buttons},
   (* First remove any existing colour palette *)
   nbs=Notebooks[];
   k=1;
   While[k <= Length[nbs],
      (*str=Options[nbs[[k]],WindowTitle];*)
      str="WindowTitle"/.NotebookInformation[nbs[[k]]];
      If[StringQ[str],          
         If[StringQ[str] && StringPosition[str,palette\[Breve]title,IgnoreCase->True]=!={},
            NotebookClose[nbs[[k]]];            
         ];
      ];
      k++;
   ];  
   extra\[Breve]buttons=Partition[PadRight[extra\[Breve]operator\[Breve]buttons,Ceiling[Length[extra\[Breve]operator\[Breve]buttons],4],P\[Breve]Button["","",Null,""]],4];
   P\[Breve]Button[id_,col_,action_,comment_]:=Tooltip[Button[Style[id,FontSize->13,Bold,col],action,ImageSize->Full,Method->"Queued"],comment];
   CreatePalette[{Grid[{{
                  P\[Breve]Button["Red",Red,DBColour[Red\[Breve]Colour],"Colour selection red"],
                  P\[Breve]Button["Green",Green,DBColour[Green\[Breve]Colour],"Colour selection green"],
                  P\[Breve]Button["Blue",Blue,DBColour[Blue\[Breve]Colour],"Colour selection blue"],
                  P\[Breve]Button["Remove",Black,DBNoColour[],"Remove colour from selection"]
               },
               {
                  P\[Breve]Button["Rin",Red,DBPostfix["Rin"],"Move red expression in"],
                  P\[Breve]Button["Gin",Green,DBPostfix["Gin"],"Move green expression in"],
                  P\[Breve]Button["Bin",Blue,DBPostfix["Bin"],"Move blue expression in"],
                  P\[Breve]Button["Inert",Black,DBInert[],"Forms the inert version of Integrals, sums, products, and limits"]
               },
               {
                  P\[Breve]Button["Rout",Red,DBPostfix["Rout"],"Move red expression out"],
                  P\[Breve]Button["Gout",Green,DBPostfix["Gout"],"Move green expression out"],
                  P\[Breve]Button["Bout",Blue,DBPostfix["Bout"],"Move blue expression out"],
                  P\[Breve]Button["Adjust",Black,ColourMaths`Private`Adjust\[Breve]Selection[],"Supplies interactive menu of rearrangements"]
               },
               {
                  P\[Breve]Button["Rover",Red,DBPostfix["Rover"],"Move red expression over comparison"],
                  P\[Breve]Button["Gover",Green,DBPostfix["Gover"],"Move red expression over comparison"],
                  P\[Breve]Button["Bover",Blue,DBPostfix["Bover"],"Move red expression over comparison"],
                  P\[Breve]Button["HighLight",Black,ColourMaths`Private`Set\[Breve]Highlight\[Breve]Formula[],"Highlights and names key formulae"]
               },
               {
                  P\[Breve]Button["Radd[\[SelectionPlaceholder]]",Red,DBPostfix["Radd[\[SelectionPlaceholder]]"],"Adds argument to red expression"],
                  P\[Breve]Button["Gadd[\[SelectionPlaceholder]]",Green,DBPostfix["Gadd[\[SelectionPlaceholder]]"],"Adds argument to green expression"],
                  P\[Breve]Button["Badd[\[SelectionPlaceholder]]",Blue,DBPostfix["Badd[\[SelectionPlaceholder]]"],"Adds argument to blue expression"],
                  P\[Breve]Button["Retrieve",Black,Pick\[Breve]Highlight\[Breve]Formula[],"Permits selection of highlight formulae"]
               },
               {
                  P\[Breve]Button["Rmpy[\[SelectionPlaceholder]]",Red,DBPostfix["Rmpy[\[SelectionPlaceholder]]"],"Multiplies argument to red expression"],
                  P\[Breve]Button["Gmpy[\[SelectionPlaceholder]]",Green,DBPostfix["Gmpy[\[SelectionPlaceholder]]"],"Multiplies argument to green expression"],
                  P\[Breve]Button["Bmpy[\[SelectionPlaceholder]]",Blue,DBPostfix["Bmpy[\[SelectionPlaceholder]]"],"Multiplies argument to blue expression"],
                  P\[Breve]Button["Flag",Magenta,DBColour[Flag\[Breve]Colour],"Flag"]
               },
               {
                  P\[Breve]Button["X",Red,Setup\[Breve]Extraction[Red\[Breve]Colour],"Colours selected expression red and extracts it to a new notebook"],
                  P\[Breve]Button["X",Green,Setup\[Breve]Extraction[Green\[Breve]Colour],"Colours selected expression green and extracts it to a new notebook"],
                  P\[Breve]Button["X",Blue,Setup\[Breve]Extraction[Blue\[Breve]Colour],"Colours selected expression blue and extracts it to a new notebook"],
                  P\[Breve]Button["","",Null,""]                 
               },
               Sequence@@extra\[Breve]buttons
            }]
      },Saveable->False,WindowTitle->palette\[Breve]title]
];


Create\[Breve]Colour\[Breve]Palette[];


(* ::Section:: *)
(*Operations invoked by butons*)


Hold\[Breve]Make\[Breve]Boxes[HoldComplete[z_]]:=MakeBoxes[z,StandardForm];


DBPostfix[f_]:=Module[{nb,s,s1,ss},
   nb=InputNotebook[];
   SelectionMove[nb,After,CellContents];
   NotebookWrite[nb,Evaluate["//"<>ToString[f]]];
];


DBInert[]:=Module[{nb,s,s1,ss},
   nb=InputNotebook[];
   s=dbGetSelectedExpression1[nb];
   If[SameQ[s,emptyexpression],Return[]];
   s=Make\[Breve]Inert[s];    
   NotebookWrite[nb,Evaluate[Hold\[Breve]Make\[Breve]Boxes[s]]];
];


(*\[LightBulb]DBColour*)DBColour[C_]:=Module[{nb,s,s1,ss},
   ss=Map[(#[[1]][x_]:>x)&,all\[Breve]colours];
   nb=InputNotebook[];
   s=dbGetSelectedExpression1[nb];
   If[SameQ[s,emptyexpression],Return[]];
   (* Get rid of any existing colour out of expression *)
   s1=s//.ss;
   s=C[s1] /. 
   C[HoldComplete[z_]]:>MakeBoxes[C[z],StandardForm];
   (* If the original expression was parenthesised, 
   parenthesise result *)
   If[MatchQ[boxExpression,RowBox[{"(",___}]],s=RowBox[{"(",s,")"}]];
   s=RowBox[{s}];
   NotebookWrite[nb,Evaluate[s]];
   SelectionMove[nb,All,Cell];
   SetOptions[nb,ShowSyntaxStyles->False]
   (*SetOptions[NotebookSelection[nb],ShowAutoStyles->False];*)
]

Clear[DBNoColour];
DBNoColour[]:=Module[{nb,s,s1,ss},
   	 nb=InputNotebook[];
   s=dbGetSelectedExpression1[nb];
   If[SameQ[s,emptyexpression],Return[]];
   		ss=Map[(#[[1]][x_]:>x)&,all\[Breve]colours];
   		s1=s//.ss;
   			If[s1 =!= s,
      			(* The replacement changed something - so we assume we are done *)
      			s=s1 /. HoldComplete[z_]:>MakeBoxes[z,StandardForm];
      NotebookWrite[nb,Evaluate[s]],
      			(* Nothing changed - so try to widen the selection a bit *)
      			SelectionMove[nb,All,Expression];
      			  If[s=!=dbGetSelectedExpression1[nb],DBNoColour[]];
      			]
   	]



(* ::Section:: *)
(*Dialog\[Breve]Box*)


SetAttributes[MakeVLISTBoxes,HoldFirst];
(* This function can switch to a smaller fornt for complex objects *)
MakeVLISTBoxes[x_]:=
  MakeBoxes[x,
      StandardForm]/;((! NumberQ[vlistFont]) || LeafCount[Hold[x]]<7 );
MakeVLISTBoxes[x_]:=
  StyleBox[MakeBoxes[x,StandardForm],FontSize->vlistFont];
vList/:MakeBoxes[x_vList,StandardForm]:=
  RowBox[Drop[Map[Seq["\n",MakeVLISTBoxes[#]]&,List@@x]/.Seq->Sequence,
      1]]

SetAttributes[dbx,Listable];
SetAttributes[dInput,HoldAll];
SetAttributes[buttonAction,HoldFirst];
tagCounter=0;
dbx[dText[s_String]]:=Cell[TextData[s],"Print"];
dbx[dItem[s_String]]:=Cell[TextData[s],"Print",CellDingbat->"\[FilledSquare]"];
dbx[dInput[var_Symbol,s_String,init_]]:=Module[{tag,
		cfr=Symbol["CellFrameLabels"],
			inh=Symbol["Inherited"]},
       Clear[var];
       tagCounter++;
       tag="V"<>ToString[tagCounter];
       AppendTo[variableList,{var,tag}];
       inputRequired=True;
       Cell[BoxData[MakeBoxes[init,StandardForm]],"Print",Editable->True,
      CellFrame->True,System`CellTags->tag,
      cfr->{{s,inh},{inh,inh}}]
       ];
dbx[dTitle[s_]]:=Module[{ans},
       If[dialogTitle==="",
              If[Head[s]===String,dialogTitle=s,dialogTitle=ToString[s[[1]]]
                     ]
              ];
       vlistFont=16;
       ans=Cell[BoxData[ToBoxes[s,StandardForm]],"Print",FontSize->30,
        TextAlignment->"Center",Background->LightBlue, 
        CellMargins->{{0,0},{10,0}}];
       Clear[vlistFont];
       ans
       ];
dbx[dText[s_]]:=Cell[BoxData[ToBoxes[s,StandardForm]]]

dbx[dButtons[b_List,options___Rule]]:=Module[{gbo},
       SiftOptions[{{GridBox,gbo}},{options},dButtons];
       Cell[
      BoxData[GridBox[Map[dbt,b],ColumnAlignments->{Center,Left},
          RowMinHeight->1.5,Sequence@@gbo]]]
       ];

dbx[dHButtons[b_List,options___Rule]]:=
Cell[BoxData[RowBox[Map[RowBox[{"   ",#,"   "}]&,Map[dbt1,b]]]],
       TextAlignment->"Center"];

dbt[{btext_String,action_,longtext_:""}]:=
{
       ButtonBox[
              StyleBox[btext,
                     FontSize->17],
              
      ButtonFunction:>ColourMaths`Private`buttonAction[action[]],
              ButtonEvaluator->Automatic,
              (* This should become optional at some point *)
              ButtonExpandable->False,
              Active->True
              ],
    StyleBox[ToBoxes[longtext,StandardForm],FontWeight->"Bold",
      FontSize->18]
       };
dbt1[{btext_String,action_}]:=
ButtonBox[
       StyleBox[btext,
              FontSize->17],
       ButtonFunction:>ColourMaths`Private`buttonAction[
        action[]],
       ButtonEvaluator->Automatic,
       (* This should become optional at some point *)
       ButtonExpandable->False,
       Active->True
       ];

buttonAction[action_]:=Module[{k,c},
       SetSelectedNotebook[dialogNotebook];
       k=1;
       While[k <= Length[variableList],
              NotebookLocate[variableList[[k,2]]];
              c=NotebookRead[dialogNotebook]/.Cell[BoxData[x_],___]->x;
              With[{p=variableList[[k,1]]},
                     Set[p,ToExpression[c]]];
              k++;
              ];
       NotebookClose[dialogNotebook];
       SetSelectedNotebook[dialogParent];
       variableList=Null;
       action
       ];
Dialog\[Breve]Box[x_List]:=Module[{xx},
       dialogParent=InputNotebook[];
       inputRequired=False;
       dialogTitle="";
       variableList={};
       xx=dbx[x];
       dialogNotebook=NotebookPut[Notebook[xx,
                     WindowClickSelect->inputRequired,
                     FontSize->18,
                     FontWeight->"Bold",
          WindowSize->{Automatic,All},
                               WindowElements->{"VerticalScrollBar"},
                               WindowToolbars->{},
                               ShowCellBracket->False,
                               WindowTitle->dialogTitle,
                               Editable->inputRequired,
                               WindowFloating->True,
                               CellMargins->{{10,0},{10,0}},
                               Background->LightYellow
                               ]];
           SetSelectedNotebook[dialogParent];
           ];

(*DBComment[c__]:=(usedComment={{c}};0);*)



(* ::Section:: *)
(*Extra operators*)


SetAttributes[precedence\[Breve]boxes,HoldAllComplete];
precedence\[Breve]boxes[x_,form_,prec_]:=Module[{},
   If[prec>Precedence1[Hold[x][[1,0]]],
      RowBox[{"(",MakeBoxes[x,form],")"}],
      MakeBoxes[x,form],
      MakeBoxes[x,form]
   ]
];

(*\[LightBulb]Make\[Breve]Infix\[Breve]Operator*)
Precedence1[x_]:=Precedence[x]; (* Augmented with specific cases as operators are defined *)
Make\[Breve]Infix\[Breve]Operator[template_,char_,z_,fn_Symbol,comment_:""]:=Module[{zb,bxs,bxs1,fns,prec},
   prec=Precedence[ToExpression["Hold[1 "<>char<>" 1]"][[1,0]]];
   Precedence1[fn]=prec;
   zb=MakeBoxes[z,StandardForm];
   bxs=MakeBoxes[template,StandardForm]/.zb->char;
   bxs1=bxs/.OverscriptBox[x_,y_]:>OverscriptBox[x,y,LimitsPositioning->False];
 bxs1=bxs1/.UnderscriptBox[x_, "_"]:>UnderscriptBox[x,"_",LimitsPositioning->False];
   extra\[Breve]operator\[Breve]buttons=Append[extra\[Breve]operator\[Breve]buttons,
      P\[Breve]Button[Evaluate[RawBoxes[bxs1]],"Black",NotebookApply[SelectedNotebook[],Evaluate[bxs]],comment]];
   With[{prec=prec,bxs=bxs,fns=ToString[fn]},
      MakeExpression[RowBox[{x_,bxs,y__}],StandardForm]:=MakeExpression[
         RowBox[{fns,"[",x,",",RowBox[{y}],"]"}],StandardForm];

      MakeBoxes[fn[x_,y_],StandardForm]:=RowBox[
         {precedence\[Breve]boxes[x,StandardForm,prec],bxs,precedence\[Breve]boxes[y,StandardForm,prec]}];

MakeBoxes[fn[x_,y_,w__],StandardForm]:=RowBox[
         {precedence\[Breve]boxes[x,StandardForm,prec],bxs,precedence\[Breve]boxes[fn[y,w],StandardForm,prec]}];
   ];
   Create\[Breve]Colour\[Breve]Palette[];
];

Make\[Breve]Prefix\[Breve]Operator[template_,char_,z_,fn_Symbol,comment_:""]:=Module[{zb,bxs,bxs1},
   prec=Precedence[ToExpression["Hold[1 "<>char<>" 1]"][[1,0]]];
   Precedence1[fn]=prec;
   zb=MakeBoxes[z,StandardForm];
   bxs=MakeBoxes[template,StandardForm]/.zb->char;
   bxs1=bxs/.OverscriptBox[x_,y_]:>OverscriptBox[x,y,LimitsPositioning->False];
 bxs1=bxs1/.UnderscriptBox[x_, "_"]:>UnderscriptBox[x,"_",LimitsPositioning->False];
   extra\[Breve]operator\[Breve]buttons=Append[extra\[Breve]operator\[Breve]buttons,
      P\[Breve]Button[Evaluate[RawBoxes[bxs1]],"Black",NotebookApply[SelectedNotebook[],Evaluate[bxs]],comment]];
   With[{prec=prec,bxs=bxs,fns=ToString[fn]},MakeExpression[RowBox[{bxs,y_}],StandardForm]:=MakeExpression[
         RowBox[{fns,"[",y,"]"}],StandardForm];
      MakeBoxes[fn[y_],StandardForm]:=RowBox[
         {bxs,precedence\[Breve]boxes[y,StandardForm,prec]}];
   ];
   Create\[Breve]Colour\[Breve]Palette[];
];


(*Options[Make\[Breve]Operator]={"Prefix"->False,"Precedence"->"*","Comment"->""};
Make\[Breve]Operator[template_,char_,z_,fn_Symbol,OptionsPattern[Make\[Breve]Operator]]:=Module[{zb,bxs,bxs1,sf,infic,comment,c},
   sf=OptionValue["Precedence"];
   If[OptionValue["Prefix"],
     comment="Prefix operator with precedence of "<>sf;
     sf=sf<>"b",
     
     comment="Infix operator with precedence of "<>sf;
     sf="a"<>sf<>"b";
     ];
   c=OptionValue["Comment"];
   If[c=!="",comment=comment<>": "<>c];
   zb=MakeBoxes[z,StandardForm];
   bxs=MakeBoxes[template,StandardForm]/.zb->char;
   bxs1=bxs/.OverscriptBox[x_,y_]:>OverscriptBox[x,y,LimitsPositioning->False];
   bxs=TagBox[bxs,xxx,SyntaxForm->sf];
   extra\[Breve]operator\[Breve]buttons=Append[extra\[Breve]operator\[Breve]buttons,
      P\[Breve]Button[Evaluate[RawBoxes[bxs1]],"Black",NotebookApply[SelectedNotebook[],bxs],comment]];
   With[{bxs=bxs,fns=ToString[fn]},MakeExpression[RowBox[{x_,bxs,y_}],StandardForm]:=MakeExpression[
         Debug\[Breve]Print@RowBox[{fns,"[",x,",",y,"]"}],StandardForm];
      MakeBoxes[fn[x_,y_],StandardForm]:=RowBox[
         {MakeBoxes[x,StandardForm],bxs,MakeBoxes[y,StandardForm]}];
   ];
   Create\[Breve]Colour\[Breve]Palette[];
];*)


(* ::Section:: *)
(*Adjustment operation*)


replace\[Breve]pattern\[Breve]variables[expr_]:=Module[{ppp,vars,repl,opt},
   vars=Union[Cases[expr//.{Pattern->ppp,Optional->opt},ppp[v_,___]->v,\[Infinity]]];
   repl=Map[(Rule@@#)&,Transpose[{vars,Map[Unique,vars]}]];
   expr/.repl
];


(*SetAttributes[Define\[Breve]Adjustment,HoldAll];*)
all\[Breve]adjustments={};

Define\[Breve]Adjustment[id_,conv_,comment_:""]:=Module[{},
all\[Breve]adjustments=Append[DeleteCases[all\[Breve]adjustments,adjustment[id,___]],adjustment[id,replace\[Breve]pattern\[Breve]variables[conv],comment]];
];


ColourMaths`\[CapitalGamma];  (* Force symbol into the correct context *)
Define\[Breve]Adjustment[ColourMaths`Factor\[Breve]Adjust,Factor];
Define\[Breve]Adjustment[ColourMaths`Factor1\[Breve]Adjust,Factor[#,GaussianIntegers->True]&];
Define\[Breve]Adjustment[ColourMaths`Together\[Breve]Adjust,Together];
Define\[Breve]Adjustment[ColourMaths`Apart\[Breve]Adjust,Apart];
Define\[Breve]Adjustment[ColourMaths`Expand\[Breve]Adjust,Expand];
Define\[Breve]Adjustment[ColourMaths`Expand\[Breve]Adjust,SwapOperations,"Subject to uniform convergence"];
Define\[Breve]Adjustment[ColourMaths`TrigExpand\[Breve]Adjust,TrigExpand];
Define\[Breve]Adjustment[ColourMaths`PowerExpand\[Breve]Adjust,PowerExpand];
Define\[Breve]Adjustment[ColourMaths`Power\[Breve]Swap\[Breve]Adjust,(a_-b_)^n_:>(-1)^n (b-a)^n,ToString[n]<> "must be an integer"];
Define\[Breve]Adjustment[ColourMaths`Trig\[Breve]Reciprocal\[Breve]Adjust,{
      Sin[x_]^p_. :>Blue\[Breve]Colour[Csc[x]]^(-p),
      Cos[x_]^p_. :>Blue\[Breve]Colour[Sec[x]]^(-p),
      Tan[x_]^p_. :>Blue\[Breve]Colour[Cot[x]]^(-p),
      Csc[x_]^p_. :>Blue\[Breve]Colour[Sin[x]]^(-p),
      Cot[x_]^p_. :>Blue\[Breve]Colour[Tan[x]]^(-p),
      Sec[x_]^p_. :>Blue\[Breve]Colour[Cos[x]]^(-p)
   },"Trig definitions"];

Define\[Breve]Adjustment[ColourMaths`Trig\[Breve]Swap\[Breve]Adjust,{
      Sin[x_]^p_.:>(1-Cos[x]^2)^(p/2),
      Cos[x_]^p_.:> (1-Sin[x]^2)^(p/2)
   },"Right angle triangle identities"];
Define\[Breve]Adjustment[ColourMaths`Log\[Breve]Adjust,{Log[a_]+Log[b_]:>Log[a b],Log[x_Times]:>(Plus@@Map[Log,List@@x]/.Log[1/y_]:>-Log[y]),Log[x_^p_]:>p Log[x]},""];
Define\[Breve]Adjustment[ColourMaths`Factorial1\[Breve]Adjust,n_!:>n (n-1)!];
Define\[Breve]Adjustment[ColourMaths`Factorial1\[Breve]Adjust,n_!:>n (n-1)!];
Define\[Breve]Adjustment[ColourMaths`Factorial2\[Breve]Adjust,n_!:>(n+1)!/(n+1)];
Define\[Breve]Adjustment[ColourMaths`Factorial3\[Breve]Adjust,n_!:>\[CapitalGamma][n+1]];
Define\[Breve]Adjustment[ColourMaths`Gamma1\[Breve]Adjust,\[CapitalGamma][n_]:>\[CapitalGamma][n+1]/n];
Define\[Breve]Adjustment[ColourMaths`Gamma2\[Breve]Adjust,\[CapitalGamma][n_]:>\[CapitalGamma][n-1](n-1)/;n=!=1];


final\[Breve]cleanup[x_]:=x/. Sum->Sum1 /. Sum1->Sum //. extra\[Breve]cleanups;


extra\[Breve]cleanups={HoldPattern[Identity[x_]]->x};
Define\[Breve]Unless\[Breve]Null[x_,y_]:=If[y===Null,Quiet[x=.],x=y];
(*\[LightBulb]Define\[Breve]Operator\[Breve]Relationship*)
Define\[Breve]Operator\[Breve]Relationship[op1_,op2_,left\[Breve]inv1_,right\[Breve]inv1_,inv2_]:=(
   Define\[Breve]Unless\[Breve]Null[left\[Breve]inverse[op1],left\[Breve]inv1];
   Define\[Breve]Unless\[Breve]Null[right\[Breve]inverse[op1],right\[Breve]inv1];
   Define\[Breve]Unless\[Breve]Null[left\[Breve]inverse[op2],inv2];
   Define\[Breve]Unless\[Breve]Null[right\[Breve]inverse[op2],inv2];
   Quasi\[Breve]Left\[Breve]MultiplicationQ[op1]=(left\[Breve]inv1=!=Null);
   Quasi\[Breve]Right\[Breve]MultiplicationQ[op1]=(right\[Breve]inv1=!=Null);
   Quasi\[Breve]Right\[Breve]MultiplicationQ[op2]=(inv2=!=Null);
   Quasi\[Breve]Left\[Breve]MultiplicationQ[op2]=(inv2=!=Null);
   SetAttributes[op1,{Flat,OneIdentity}];
   SetAttributes[op2,{Flat,OneIdentity}];
   (* op1[a_]:=a;
   If[op2=!=Plus,op2[a_]:=a];*)
   If[ValueQ[left\[Breve]inverse[op1]],extra\[Breve]cleanups=Append[extra\[Breve]cleanups,{left\[Breve]inv1[left\[Breve]inv1[a_]]:>Sequence@@{},op1[left\[Breve]inv1[a_],a_]:>Sequence@@{},op1[a_]:>a}]];
   If[ValueQ[right\[Breve]inverse[op1]],extra\[Breve]cleanups=Append[extra\[Breve]cleanups,{right\[Breve]inv1[right\[Breve]inv1[a_]]:>Sequence@@{},op1[a_,right\[Breve]inv1[a_]]->Sequence@@{}}]];
   
   If[ValueQ[left\[Breve]inverse[op2]],extra\[Breve]cleanups=Append[extra\[Breve]cleanups,op2[inv2[a_],a_]:>Sequence@@{}]];
   If[ValueQ[right\[Breve]inverse[op2]],extra\[Breve]cleanups=Append[extra\[Breve]cleanups,op2[a_,inv2[a_]]->Sequence@@{}]];
   extra\[Breve]cleanups=Flatten[extra\[Breve]cleanups];
   (*If[ValueQ[left\[Breve]inverse[op2]],extra\[Breve]cleanups=Append[extra\[Breve]cleanups,op2[l\[Breve]inv\[Breve]a,a]:>Sequence@@{}]];
   If[ValueQ[right\[Breve]inverse[op2]],extra\[Breve]cleanups=Append[extra\[Breve]cleanups,op2[a,r\[Breve]inv]->Sequence@@{}]];*)
   DistributesQ[op1,op2]=True;   
);


Accept\[Breve]Adjustment::nocolours="All the colours are already in use in this expression";
Accept\[Breve]Adjustment[trans_]:=Module[{C,c,s,cff,j,a\[Breve]func=Null,rule,special\[Breve]symbols,expr,ff},
   If[trans===0,Return[]];
   NotebookWrite[adjustNb,
      RowBox[{TagBox[ToBoxes[adjustExpr,StandardForm],C]}]];
   SelectionMove[adjustNb,All,CellContents];
   s=NotebookRead[adjustNb];
   j=1;
   While[j <= Length[all\[Breve]colours],
      c=all\[Breve]colours[[j,1]];
      If[Cases[s,TagBox[_,c],\[Infinity]]==={},
         (* Things like RR1 have a value, so we must proceed cautiously! *)
         (* cf=all\[Breve]colours[[j,3]];*)
         a\[Breve]func=all\[Breve]colours[[j,3]]<>"adjust";
         Break[];
      ];
      j++;
   ];
   rule=Cases[all\[Breve]adjustments,adjustment[trans,___]][[1,2]];
   special\[Breve]symbols=Map[(#->Global`\[SelectionPlaceholder])&,Union[Cases[rule,Flag\[Breve]Colour[x_],\[Infinity]]]];
(*   rules=rules /. Map[(#->Flag\[Breve]Colour[#])&,Union[Cases[rule,Flag\[Breve]Colour[x_],\[Infinity]]]];*)
   rule=Magenta\[Breve]Symbol[Magenta\[Breve]Symbol[x_]]->Magenta\[Breve]Symbol[x];
   If[a\[Breve]func===Null,Message[Adjust\[Breve]Selection::nocolours];Abort[]];      
   NotebookWrite[adjustNb,
      s/.TagBox[_,C]:>ToBoxes[c[adjustExpr],StandardForm]];
   SelectionMove[adjustNb,After,CellContents];
   NotebookWrite[adjustNb,"//"];
   expr=ff@@Prepend[special\[Breve]symbols,trans];
   expr=Hold[Evaluate[expr],StandardForm]/.ff->ToExpression[a\[Breve]func]/.Hold->MakeBoxes;
   NotebookWrite[adjustNb,expr];
];


Do\[Breve]Adjustment[name_,rules___]:=Module[{op,ff,sy,vars},
   op=Cases[all\[Breve]adjustments,adjustment[name,___]];
   If[op==={},Abort[]];
   vars=Cases[op,Flag\[Breve]Colour[x_]->x,\[Infinity]];
   op=First[op];
   vars=Map[(#->Flag\[Breve]Colour[#])&,Union[Cases[op,Flag\[Breve]Colour[x_]->x,\[Infinity]]]];
   op=op/.Flag\[Breve]Colour[x_]->sy[x]/.vars/.sy[x_]->Flag\[Breve]Colour[x];
   ff=op[[2]]/.{rules}/.Flag\[Breve]Colour[x_]->x;
   If[Head[ff]===Symbol || Head[ff]===Function,Return[ff]];
   With[{ff=ff},Replace[#,ff]&]
];


(*\[LightBulb]Adjust\[Breve]Selection*)Adjust\[Breve]Selection[]:=Module[{ce,x,x1,s,k,op,comment,boxContents={},found=False,transforms},
   adjustNb=InputNotebook[];
   s=dbGetSelectedExpression1[adjustNb];
   If[SameQ[s,emptyexpression],Return[]];
   (* Get rid of any existing colour out of expression *)
   s=s//. colourRemovalRules;
   adjustExpr=s/.HoldComplete[u_]->u;
   AppendTo[boxContents,
      dTitle[vList["Possible rearrangements of",adjustExpr]]];
   buttons={};
   transforms={};
   k=1;
   While[k <= Length[all\[Breve]adjustments],
      op=all\[Breve]adjustments[[k]];
      If[Head[op[[2]]]===Symbol || Head[op[[2]]]===Function,
         x1=adjustExpr//op[[2]],
         x1=Replace[adjustExpr,op[[2]]];
      ];      			      		             
      If[x1 =!=adjustExpr &&  ! MemberQ[transforms,x1],
         transforms=Append[transforms,x1];
         comment=op[[3]];
         found=True;With[{x=op[[1]]},            
            AppendTo[
               buttons,{"Accept",Accept\[Breve]Adjustment[x]&,
                  If[comment=="",
                  vList[x1],
                  vList[x1,comment]
                  ]
                  }];
         ];
      ];
      k++;
   ];
   AppendTo[boxContents,dButtons[buttons,RowLines->True]];
   vi=visualInterface[adjustExpr];
   If[Head[vi]=!=visualInterface,
      If[! found,Return[vi[adjustExpr]]];
      found=True;      
      AppendTo[boxContents,
         dButtons[{{"Accept",vi[adjustExpr]&,"Further transformations"}}]]
   ];
   AppendTo[boxContents,dHButtons[{{"Cancel",xxx}}]];
   If[found,
      Dialog\[Breve]Box[boxContents],
      
      Dialog\[Breve]Box[{dTitle["Error"],
            dText[vList["No transformation can be found for the expression",
                  adjustExpr]],
            dHButtons[{{"Ok",xx }}]}]
   ];
];


End[];


EndPackage[];
