(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    172190,       4873]
NotebookOptionsPosition[    158133,       4427]
NotebookOutlinePosition[    160108,       4490]
CellTagsIndexPosition[    159902,       4482]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["The ColourMaths` package ", "Chapter",
 CellChangeTimes->{
  3.5953501485181932`*^9, {3.5957460646131616`*^9, 3.5957460764608393`*^9}}],

Cell[CellGroupData[{

Cell["Introduction", "Section",
 PageWidth->WindowWidth],

Cell[TextData[{
 "The Colour Maths package was developed to solve a ",
 StyleBox["recurring", "FunctionName"],
 " problem among ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " users \[Dash] how to specify arbitrary algebraic transformations as \
elegantly and reliably as possible. Using this package a beginner user of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "  can specify complicated algebraic transformations with little or no grasp \
of functions such as Collect, Together, Part, etc. Furthermore, all the \
details of algebraic manipulations stay in the notebook for future reference \
and there is a much reduced chance of making incorrect algebraic operations. \
Version 2.3 contains mechanisms to enable a user to extend the scope of the \
package for his/her domain, and it contains a powerful mechanism to refer \
back to particular formulae (e.g. theorems) earlier in the notebook."
}], "Text",
 PageWidth->WindowWidth,
 CellChangeTimes->{
  3.595782904874304*^9, {3.601366602235796*^9, 3.6013667128711243`*^9}}],

Cell["", "Text"],

Cell[CellGroupData[{

Cell["Getting started", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["\<\
ColourMaths is supplied as a ZIP file containing all the files required in \
their appropriate directories .. It is vital that this directory structure is \
preserved. Copy the file either to the directory specified by $BaseDirectory, \
or $UserBaseDirectory (as a general rule, use the former if you are the sole \
user of the computer, use the latter if several people log on to your \
machine). Unzip the file ColourMaths.ZIP using a tool that preserves the \
directory structure and handles long names correctly, e.g. PKZIP (R)  Version \
2.50, or WinZip (R). 
As of version 2.20 of ColourMaths, the documentation is fully accessible via \
the Documentation Center.\
\>", "Text",
 CellChangeTimes->{{3.595349978222453*^9, 3.5953499820056696`*^9}, {
   3.595350015704597*^9, 3.5953500697176867`*^9}, {3.5953512383015256`*^9, 
   3.595351258765696*^9}, 3.595746111843863*^9}],

Cell[TextData[StyleBox["Once the package is in place it can be read In the \
usual way:",
 FontFamily->"Times New Roman"]], "Text",
 CellChangeTimes->{{3.5953494065607557`*^9, 3.595349419761511*^9}, {
   3.595350082166398*^9, 3.595350082701429*^9}, 3.596118823516538*^9}],

Cell[BoxData[
 RowBox[{"<<", "ColourMaths`"}]], "Input",
 CellChangeTimes->{{3.595350089223802*^9, 3.595350098735346*^9}},
 CellLabel->"In[1]:="],

Cell[TextData[{
 "ColourMaths",
 " will create a palette immediately it is read in. Using this palette you \
can specify algebraic manipulations using colour. Simply select a whole \
subexpression and press one of the three coloured square buttons (red, green, \
blue) at the top of the palette. If you selected an invalid subexpression you \
will receive a beep, otherwise you will produce a coloured expression. It is \
possible to colour any number of subexpressions with the same or different \
colours. To remove a colour simply make a selection which includes the colour \
you wish to remove and hit the black button (other methods will be explained \
later).\n",
 StyleBox["Hint",
  FontSlant->"Italic"],
 ": To ensure that you select a whole subexpression click repeatedly on the \
expression you wish to select. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will select successively larger subexpressions until you reach the one you \
require."
}], "Text",
 CellChangeTimes->{3.595350156900673*^9, 3.5957461171311655`*^9}],

Cell[TextData[{
 "The colour acts as a tag to enable arbitrary algebraic transformations to \
be specified. Using ",
 "ColourMaths",
 " you should be able to avoid re-typing expressions in almost all situations \
- avoiding the mistakes that this can produce. Your notebooks will also \
contain a full record of your work.\nThe palette also contains ",
 StyleBox["some",
  FontVariations->{"Underline"->True}],
 " of the colour manipulation functions described below. These paste at the \
end of the cell, since that is usually whee they are required. To avoid \
creating a gigantic palette, not all functions or options can be accessed in \
this way. For example, the RR function pastes on the assumption that you will \
apply it to a rule rather than a function (although you can, of course, edit \
the pasted code to change it). In general there is no substitute for reading \
the remaining sections of this manual!"
}], "Text",
 CellChangeTimes->{3.5953501644611053`*^9, 3.5957461245115876`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["What is a coloured expression?", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["\<\
Although it is possible to colour text in a notebook using the Format menu, \
this colour is not transmitted to the Kernel and is purely decorative. When \
you select a subexpression and press one of the colour buttons something \
quite different occurs. The subexpression is surrounded by a function 'Red\
\[Breve]Colour', 'Green\[Breve]Colour', or 'Blue\[Breve]Colour' as \
appropriate. You can see this function by examining the FullForm:\
\>", "Text",
 CellChangeTimes->{{3.595351447148471*^9, 3.5953514701867886`*^9}, 
   3.6013667802729793`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "+", 
   TagBox["b",
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], "+", "c"}], " ", "//", "FullForm"}]], "Input",
 CellChangeTimes->{{3.595351498991436*^9, 3.5953515168684587`*^9}, {
  3.5957847116006427`*^9, 3.5957847190140667`*^9}},
 CellLabel->"In[8]:="],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Plus", "[", 
    RowBox[{"a", ",", "c", ",", 
     RowBox[{"Red\\[Breve]Colour", "[", "b", "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.595351517504495*^9, 3.595784725521439*^9},
 CellLabel->"Out[8]//FullForm="]
}, Open  ]],

Cell["\<\
The colour package programs the FrontEnd so that it normally displays these \
colour functions by colouring their arguments. Once you understand the \
structure of coloured expressions you can even manipulate them directly:\
\>", "Text",
 CellChangeTimes->{3.596118840727523*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "+", 
   TagBox["b",
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]}], "+", "c"}], "/.", 
  RowBox[{
   RowBox[{"Red\[Breve]Colour", "[", "x_", "]"}], "->", 
   RowBox[{"Green\[Breve]Colour", "[", 
    RowBox[{"2", " ", "x"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.595351546226138*^9, 3.5953516211664243`*^9}, {
   3.5953516719833307`*^9, 3.5953516941545987`*^9}, 3.5953518179826813`*^9},
 CellLabel->"In[9]:="],

Cell[BoxData[
 RowBox[{"a", "+", "c", "+", 
  TagBox[
   RowBox[{"2", " ", "b"}],
   ColourMaths`Green\[Breve]Colour,
   DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
   SelectWithContents->True]}]], "Output",
 CellChangeTimes->{3.5953516279838142`*^9, 3.5953516981648283`*^9, 
  3.5953518221349187`*^9, 3.595784736769082*^9},
 CellLabel->"Out[9]="]
}, Open  ]],

Cell[TextData[{
 "Because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not 'know' anything about these functions except how to display them, \
they can be useful for preventing unwanted evaluation. For example, consider \
the following expression:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["\[Pi]", 
   RowBox[{
    SuperscriptBox["E", 
     RowBox[{"I", " ", "\[Pi]", " ", "x"}]], "-", 
    SuperscriptBox["E", 
     RowBox[{
      RowBox[{"-", "I"}], " ", "\[Pi]", " ", "x"}]]}]], "//", 
  "ExpToTrig"}]], "Input",
 CellLabel->"In[10]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
  RowBox[{"Csc", "[", 
   RowBox[{"\[Pi]", " ", "x"}], "]"}]}]], "Output",
 CellChangeTimes->{3.595351844304187*^9},
 CellLabel->"Out[10]="]
}, Open  ]],

Cell[TextData[{
 "Perhaps you feel more comfortable with the sine function rather than the \
cosecant function (reciprocal sine). By using colour we can prevent ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " performing this transformation:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox["\[Pi]", 
   TagBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", "x"}]], "-", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "\[ImaginaryI]"}], " ", "\[Pi]", " ", "x"}]]}],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]}]], "//", 
  "ExpToTrig"}]], "Input",
 CellChangeTimes->{{3.595351935704415*^9, 3.59535204087043*^9}},
 CellLabel->"In[10]:="],

Cell[BoxData[
 FractionBox["\[Pi]", 
  TagBox[
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "\[ImaginaryI]"}], " ", "\[Pi]", " ", "x"}]]}], "+", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", "x"}]]}],
   ColourMaths`Red\[Breve]Colour,
   DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
   SelectWithContents->True]]], "Output",
 CellChangeTimes->{{3.5953520290587544`*^9, 3.5953520442596235`*^9}, 
   3.5957847479237204`*^9},
 CellLabel->"Out[10]="]
}, Open  ]],

Cell["\<\
In larger algebraic calculations in which a long sequence of colour \
operations are applied this concept of preventing evaluations can become \
vital.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The colour manipulation functions", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell[TextData[{
 "Once portions of an expression have been tagged they are usually \
manipulated using the colour manipulation functions provided by  ColourMaths. \
Because these functions are intended to be used frequently, and in \
combination, they are given a short name as well as a more traditional ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " name. Those functions which remove the colour after doing their work have \
a full name which ends in 'AndClear', and a short form which ends in the \
digit '1'. These functions are known as 'non-colour preserving variants'.\n"
}], "Text",
 CellChangeTimes->{
  3.595352076245453*^9, 3.5957461257636595`*^9, {3.601366810934733*^9, 
   3.6013668115507684`*^9}}],

Cell[BoxData[
 FormBox[
  StyleBox[
   FrameBox[
    StyleBox[
     AdjustmentBox[GridBox[{
        {
         StyleBox[
          RowBox[{"Function", " ", "name"}], "TI"], 
         StyleBox["Abbreviation", "TI"], 
         StyleBox["Action", "TI"]},
        {
         StyleBox["RedOperation",
          GridBoxOptions->{
          GridBoxDividers->{
           "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}}}], 
         StyleBox["RR",
          GridBoxOptions->{
          GridBoxDividers->{
           "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}}}], 
         StyleBox["\<\"Restricts functions/rules to red expressions\"\>", 
          "Text"]},
        {
         RowBox[{"RedOperationAndClear", " "}], "RR1", 
         RowBox[{
          RowBox[{"Restricts", " ", 
           RowBox[{"functions", "/", "rules"}], " ", "to", " ", "red"}], "\n", 
          RowBox[{"expressions", ",", " ", 
           RowBox[{"and", " ", "removes", " ", "colour"}]}]}]},
        {
         RowBox[{"GreenOperation", " "}], "GG", 
         StyleBox["\<\"Restricts functions/rules to green expressions\"\>", 
          "Text"]},
        {
         RowBox[{"GreenOperationAndClear", " "}], "GG1", 
         RowBox[{
          RowBox[{"Restricts", " ", 
           RowBox[{"functions", "/", "rules"}], " ", "to", " ", "green"}], 
          "\n", 
          RowBox[{"expressions", ",", " ", 
           RowBox[{"and", " ", "removes", " ", "colour"}]}]}]},
        {
         RowBox[{"BlueOperation", " "}], "BB", 
         StyleBox["\<\"Restricts functions/rules to blue expressions\"\>", 
          "Text"]},
        {
         RowBox[{"BlueOperationAndClear", " "}], "BB1", 
         RowBox[{
          RowBox[{"Restricts", " ", 
           RowBox[{"functions", "/", "rules"}], " ", "to", " ", "blue"}], 
          "\n", 
          RowBox[{"expressions", ",", " ", 
           RowBox[{"and", " ", "removes", " ", "colour"}]}]}]},
        {
         RowBox[{"RedGrow", " "}], "Rgrow", 
         RowBox[{
         "Expands", " ", "the", " ", "red", " ", "colouration", " ", "to", 
          " ", "cover", " ", "the", " ", "containing", " ", "expression"}]},
        {
         RowBox[{"GreenGrow", " "}], "Ggrow", 
         RowBox[{
         "Expands", " ", "the", " ", "green", " ", "colouration", " ", "to", 
          " ", "cover", " ", "the", " ", "containing", " ", "expression"}]},
        {
         RowBox[{"BlueGrow", " "}], "Bgrow", 
         RowBox[{
         "Expands", " ", "the", " ", "blue", " ", "colouration", " ", "to", 
          " ", "cover", " ", "the", " ", "containing", " ", "expression"}]},
        {
         RowBox[{"CoalesceRed", " "}], 
         "RC", "\<\"Coalesces red parts of an expression\"\>"},
        {
         RowBox[{"CoalesceRedAndClear", " "}], 
         "RC1", "\<\"Coalesces red parts of an \
expression,\:f39fand\:f39fremoves\:f39fcolour\"\>"},
        {"CoalesceGreen", 
         "GC", "\<\"Coalesces green parts of an expression\"\>"},
        {
         RowBox[{"CoalesceGreenAndClear", " "}], 
         "GC1", "\<\"Coalesces green parts of an \
expression,\:f39fand\:f39fremoves\:f39fcolour\"\>"},
        {"CoalesceBlue", 
         "BC", "\<\"Coalesces blue parts of an expression\"\>"},
        {
         RowBox[{"CoalesceBlueAndClear", "  "}], "BC1", 
         RowBox[{"\<\"Coalesces blue parts of an expression\"\>", ",", " ", 
          RowBox[{"and", " ", "removes", " ", "colour"}]}]},
        {
         RowBox[{"MoveRedOut", " "}], 
         "Rout", "\<\"Moves red expression out one layer\"\>"},
        {
         RowBox[{"MoveRedOutAndClear", " "}], "Rout1", 
         RowBox[{"\<\"Moves red expression out one layer\"\>", ",", "  ", 
          RowBox[{"and", " ", "removes", " ", "colour"}]}]},
        {
         RowBox[{"MoveGreenOut", " "}], 
         "Gout", "\<\"Moves green expression out one layer\"\>"},
        {
         RowBox[{"MoveGreenOutAndClear", " "}], "Gout1", 
         RowBox[{"\<\"Moves green expression out one layer\"\>", ",", "  ", 
          RowBox[{"and", " ", "removes", " ", "colour"}]}]},
        {
         RowBox[{"MoveBlueOut", " "}], 
         "Bout", "\<\"Moves blue expression out one layer\"\>"},
        {
         RowBox[{"MoveBlueOutAndClear", "  "}], "Bout1", 
         RowBox[{"\<\"Moves blue expression out one layer\"\>", ",", "  ", 
          RowBox[{"and", " ", "removes", " ", "colour"}]}]},
        {
         RowBox[{"MoveRedOver", " "}], "Rover", 
         RowBox[{"\<\"Moves red expression over ==\"\>", " ", "and", " ", 
          "inequalities"}]},
        {"MoveRedOverAndClear", "Rover1", 
         RowBox[{
          RowBox[{
           RowBox[{"Moves", " ", "red", " ", "expression", " ", "over"}], " ",
            "==", " ", 
           RowBox[{"and", " ", "inequalities"}]}], ",", "  ", 
          RowBox[{"and", "  ", "removes", " ", "colour"}]}]},
        {
         RowBox[{"MoveGreenOver", " "}], "Gover", 
         RowBox[{"\<\"Moves green expression over ==\"\>", " ", "and", " ", 
          "inequalities"}]},
        {
         RowBox[{"MoveGreenOverAndClear", " "}], "Gover1", 
         RowBox[{
          RowBox[{
           RowBox[{"Moves", " ", "green", " ", "expression", " ", "over"}], 
           " ", "==", 
           RowBox[{"and", " ", "inequalities"}]}], ",", "  ", 
          RowBox[{"and", "  ", "removes", " ", "colour"}]}]},
        {
         RowBox[{"MoveBlueOver", " "}], "Bover", 
         RowBox[{"\<\"Moves blue expression over ==\"\>", " ", "and", " ", 
          "inequalities"}]},
        {"MoveBlueOverAndClear", "Bover1", 
         RowBox[{
          
          RowBox[{"\<\"Moves blue expression over ==\"\>", " ", "and", " ", 
           "inequalities"}], ",", "  ", 
          RowBox[{"and", "  ", "removes", " ", "colour"}]}]},
        {
         RowBox[{"\<\"MoveRedIn\"\>", " "}], 
         "Rin", "\<\"Moves red expression inside another\"\>"},
        {
         RowBox[{"MoveRedInAndClear", " "}], "Rin1", 
         RowBox[{"\<\"Moves red expression inside another\"\>", " ", "and", 
          " ", "removes", " ", "the", " ", "colour"}]},
        {"MoveGreenIn", 
         "Gin", "\<\"Moves green expression inside another\"\>"},
        {
         RowBox[{"MoveGreenInAndClear", " "}], "Gin1", 
         RowBox[{"\<\"Moves green\:f39fexpression inside another\"\>", " ", 
          "and", " ", "removes", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveBlueIn", " "}], 
         "Bin", "\<\"Moves blue expression inside another\"\>"},
        {
         RowBox[{"MoveBlueInAndClear", " "}], "Bin1", 
         RowBox[{"\<\"Moves blue expression inside another\"\>", " ", "and", 
          " ", "removes", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveRedInsideRed", " "}], "RRin", 
         RowBox[{
         "Moves", " ", "red", " ", "expression", " ", "inside", " ", "red", 
          " ", "one"}]},
        {
         RowBox[{"MoveRedInsideRedAndClear", " "}], "RRin1", 
         RowBox[{
         "Moves", " ", "red", " ", "expression", " ", "inside", " ", "red", 
          " ", "one", " ", "and", " ", "remove", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveRedInsideGreen", " "}], "RGin", 
         RowBox[{
         "Moves", " ", "red", " ", "expression", " ", "inside", " ", "green", 
          " ", "one"}]},
        {
         RowBox[{"MoveRedInsideGreenAndClear", " "}], "RGin1", 
         RowBox[{
         "Moves", " ", "red", " ", "expression", " ", "inside", " ", "green", 
          " ", "one", " ", "and", " ", "remove", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveRedInsideBlue", " "}], "RBin", 
         RowBox[{
         "Moves", " ", "red", " ", "expression", " ", "inside", " ", "blue", 
          " ", "one"}]},
        {
         RowBox[{"MoveRedInsideBlueAndClear", " "}], "RBin1", 
         RowBox[{
         "Moves", " ", "red", " ", "expression", " ", "inside", " ", "blue", 
          " ", "one", " ", "and", " ", "remove", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveGreenInsideRed", " "}], "GRin", 
         RowBox[{
         "Moves", " ", "green", " ", "expression", " ", "inside", " ", "red", 
          " ", "one"}]},
        {
         RowBox[{"MoveGreenInsideRedAndClear", " "}], "GRin1", 
         RowBox[{
         "Moves", " ", "green", " ", "expression", " ", "inside", " ", "red", 
          " ", "one", " ", "and", " ", "remove", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveGreenInsideGreen", " "}], "GGin", 
         RowBox[{
         "Moves", " ", "green", " ", "expression", " ", "inside", " ", 
          "green", " ", "one"}]},
        {
         RowBox[{"MoveGreenInsideGreenAndClear", " "}], "GGin1", 
         RowBox[{
         "Moves", " ", "green", " ", "expression", " ", "inside", " ", 
          "green", " ", "one", " ", "and", " ", "remove", " ", "the", " ", 
          "colour"}]},
        {
         RowBox[{"MoveGreenInsideBlue", " "}], "GBin", 
         RowBox[{
         "Moves", " ", "green", " ", "expression", " ", "inside", " ", "blue",
           "  ", "one", " ", "and", " ", "remove", " ", "the", " ", 
          "colour"}]},
        {
         RowBox[{"MoveGreenInsideBlueAndClear", " "}], "GBin1", 
         RowBox[{
         "Moves", " ", "green", " ", "expression", " ", "inside", " ", "blue",
           " ", "one"}]},
        {
         RowBox[{"MoveBlueInsideRed", " "}], "BRin", 
         RowBox[{
         "Moves", " ", "blue", " ", "expression", " ", "inside", " ", "red", 
          " ", "one"}]},
        {
         RowBox[{"MoveBlueInsideRedAndClear", " "}], "BRin1", 
         RowBox[{
         "Moves", " ", "blue", " ", "expression", " ", "inside", " ", "red", 
          " ", "one", " ", "and", " ", "remove", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveBlueInsideGreen", " "}], "BGin", 
         RowBox[{
         "Moves", " ", "blue", " ", "expression", " ", "inside", " ", "green",
           " ", "one"}]},
        {
         RowBox[{"MoveBlueInsideGreenAndClear", " "}], "BGin1", 
         RowBox[{
         "Moves", " ", "blue", " ", "expression", " ", "inside", " ", "green",
           " ", "one", " ", "and", " ", "remove", " ", "the", " ", "colour"}]},
        {
         RowBox[{"MoveBlueInsideBlue", " "}], "BBin", 
         RowBox[{
         "Moves", " ", "blue", " ", "expression", " ", "inside", " ", "blue", 
          " ", "one"}]},
        {
         RowBox[{"MoveBlueInsideBlueAndClear", " "}], "BBin1", 
         RowBox[{
         "Moves", " ", "blue", " ", "expression", " ", "inside", " ", "blue", 
          " ", "one", " ", "and", " ", "remove", " ", "the", " ", "colour"}]},
        {
         RowBox[{"RedAdd", " "}], "Radd", 
         RowBox[{
          RowBox[{
           RowBox[{"Adds", " ", "expr"}], "-", 
           RowBox[{"Red\[Breve]Colour", "[", "expr", "]"}]}], " ", "\n", "\t", 
          RowBox[{"to", " ", "the", " ", "red", " ", "expression"}]}]},
        {
         RowBox[{"GreenAdd", " "}], "Gadd", 
         RowBox[{
          RowBox[{
           RowBox[{"Adds", " ", "expr"}], "-", 
           RowBox[{"Green\[Breve]Colour", "[", "expr", "]"}]}], " ", "\n", 
          "\t", 
          RowBox[{"to", " ", "the", " ", "green", " ", "expression"}]}]},
        {
         RowBox[{"BlueAdd", " "}], "Badd", 
         RowBox[{
          RowBox[{
           RowBox[{"Adds", " ", "expr"}], "-", 
           RowBox[{"Blue\[Breve]Colour", "[", "expr", "]"}]}], " ", "\n", 
          "\t", 
          RowBox[{"to", " ", "the", " ", "blue", " ", "expression"}]}]},
        {
         RowBox[{"RedMultiply", " "}], "Rmpy", 
         RowBox[{
         "Multiplies", " ", "the", " ", "red", " ", "expression", " ", "by", 
          " ", "expr", " ", 
          RowBox[{"Red\[Breve]Colour", "[", 
           SuperscriptBox["expr", 
            RowBox[{"-", "1"}]], "]"}]}]},
        {
         RowBox[{"GreenMultiply", " "}], "Gmpy", 
         RowBox[{
         "Multiplies", " ", "the", " ", "green", " ", "expression", " ", "by",
           " ", "expr", " ", 
          RowBox[{"Gree", "[", 
           RowBox[{"\[Breve]Colour", "[", 
            SuperscriptBox["expr", 
             RowBox[{"-", "1"}]], "]"}]}]}]},
        {
         RowBox[{"BlueMultiply", " "}], "Bmpy", 
         RowBox[{
          RowBox[{
          "Multiplies", " ", "the", " ", "blue", " ", "expression", " ", "by",
            " ", "expr", " ", 
           RowBox[{"Blue\[Breve]Colour", "[", 
            SuperscriptBox["expr", 
             RowBox[{"-", "1"}]], "]"}]}], "\n", "\t"}]},
        {
         RowBox[{"ColourSort", " "}], 
         StyleBox["CS",
          GridBoxOptions->{
          GridBoxDividers->{
           "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}}}], 
         StyleBox[
          RowBox[{
           RowBox[{"CS", "[", "f", "]"}], " ", "returns", " ", "a", " ", 
           "function", " ", "that", " ", "sorts", " ", "coloured", " ", 
           "parameters", " ", "of", " ", 
           RowBox[{"f", ".", " ", 
            RowBox[{"CS", "[", "]"}]}], " ", "returns", " ", "a", " ", 
           "function", " ", "that", " ", "colour", " ", "sorts", " ", 
           RowBox[{"lists", "."}]}],
          GridBoxOptions->{
          GridBoxDividers->{
           "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}}}]},
        {"ColourSortAndClear", 
         StyleBox["CS1",
          GridBoxOptions->{
          GridBoxDividers->{
           "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}}}], 
         StyleBox[
          RowBox[{
           RowBox[{"CS1", " ", "[", "f", "]"}], " ", "and", " ", 
           RowBox[{"CS1", "[", "]"}], " ", "operate", " ", "as", " ", 
           RowBox[{"CS", "[", "f", "]"}], " ", "and", " ", 
           RowBox[{"CS", "[", "]"}], " ", "except", " ", "that", " ", "all", 
           " ", "colour", " ", "is", " ", "removed", " ", "after", " ", "the",
            " ", "operation", " ", 
           RowBox[{"completes", "."}]}],
          GridBoxOptions->{
          GridBoxDividers->{
           "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}}}]}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}}],
      BoxMargins->{{1, 1}, {1, 1}}],
     GridBoxOptions->{
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
       "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}},
     GridBoxItemSize->{"Columns" -> {
         Scaled[0.35], 
         Scaled[0.2], {
          Scaled[0.4]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}}}]], "3ColumnBox"], TraditionalForm]], "Text",
 PageWidth->WindowWidth,
 CellChangeTimes->{{3.595783146198107*^9, 3.595783207060588*^9}},
 GridBoxOptions->{GridBoxSpacings->{"Columns" -> {
     Offset[0.27999999999999997`], {
      Offset[0.5599999999999999]}, 
     Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
     Offset[0.2], 
     Offset[0.8], 
     Offset[0.8], {
      Offset[0.4]}, 
     Offset[0.2]}, "RowsIndexed" -> {}}},
 CellTags->"Action.Definition"],

Cell["\<\
The following functions are used to manipulate inert sums, integrals, \
products, etc. and to define and use assertions. Many of them are frequently \
used inside colour functions such as RR1 to restrict their action.\
\>", "Text"],

Cell[BoxData[
 FormBox[
  StyleBox[
   FrameBox[
    StyleBox[
     AdjustmentBox[GridBox[{
        {
         StyleBox[
          RowBox[{"Function", " ", "name"}], "TI"], 
         StyleBox[
          RowBox[{"No", " ", "of", " ", "arguments"}], "TI"], 
         StyleBox["Action", "TI"]},
        {
         RowBox[{"DoIntegrals", " "}], "0", 
         StyleBox[
          RowBox[{
          "A", " ", "function", " ", "that", " ", "converts", " ", "inactive",
            " ", "integrals", " ", "into", " ", "the", " ", "form", " ", 
           "recognised", " ", "by", " ", 
           RowBox[{"Mathematica", "."}]}], "Text"]},
        {
         RowBox[{"DoSummations", " "}], "0", 
         StyleBox[
          RowBox[{
          "A", " ", "function", " ", "that", " ", "converts", " ", "inactive",
            " ", "summations", " ", "into", " ", "the", " ", "form", " ", 
           "recognised", " ", "by", " ", 
           RowBox[{"Mathematica", "."}]}], "Text"]},
        {
         RowBox[{"SwapOperations", " "}], "0", 
         StyleBox[
          RowBox[{
          "A", " ", "function", " ", "that", " ", "swaps", " ", "nested", " ",
            "inert", " ", 
           RowBox[{"summations", "/", 
            RowBox[{"integrals", "."}]}]}], "Text"]},
        {
         RowBox[{"ByParts", " "}], 
         RowBox[{"0", " ", "or", " ", "1"}], "xxxx"},
        {
         RowBox[{"Change\[Breve]Variables", " "}], "2", 
         RowBox[{
         "Returns", " ", "a", " ", "function", " ", "that", " ", "performs", 
          " ", "a", " ", "variable", " ", "substitution", " ", "on", " ", 
          "an", " ", "inert", " ", 
          RowBox[{"integral", "."}]}]},
        {"DoSpecialFunctions", 
         StyleBox["0",
          GridBoxOptions->{
          GridBoxDividers->{
           "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}}}], 
         RowBox[{
         "Returns", " ", "a", " ", "function", " ", "that", " ", "causes", 
          " ", "the", " ", "inert", " ", "forms", " ", "of", " ", "special", 
          " ", "functions", " ", "to", " ", "be", " ", "replaced", " ", "by", 
          " ", "the", " ", "Mathematica", " ", 
          RowBox[{"equivalent", "."}]}]}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}}],
      BoxMargins->{{1, 1}, {1, 1}}],
     GridBoxOptions->{
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
       "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}},
     GridBoxItemSize->{"Columns" -> {
         Scaled[0.35], 
         Scaled[0.2], {
          Scaled[0.4]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}}}]], "3ColumnBox"], TraditionalForm]], "Input",
 CellChangeTimes->{{3.5953548567844906`*^9, 3.595354867277091*^9}, {
  3.59535490623932*^9, 3.5953549379681344`*^9}, {3.596118891718439*^9, 
  3.5961188976377773`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Selective application of rules and functions", "SubsectionIcon",
 ShowGroupOpener->True],

Cell["\<\
Using colour it is possible to restrict the application of a rule, rule set, \
or function to just the coloured parts of an expression. This is done using \
the functions RR, GG, BB or their non colour preserving variants RR1, GG1, \
BB1. Here is an example in which the built-in function TrigToExp is applied \
selectively to the red subexpression:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Cos", "[", 
    RowBox[{"a", " ", "x"}], "]"}], "+", 
   TagBox[
    RowBox[{"Cos", "[", 
     RowBox[{"b", " ", "x"}], "]"}],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]}], "+", 
   RowBox[{"Cos", "[", 
    RowBox[{"c", " ", "x"}], "]"}]}], "//", 
  RowBox[{"RR", "[", "TrigToExp", "]"}]}]], "Input",
 CellChangeTimes->{{3.595355087113665*^9, 3.5953550927869897`*^9}},
 CellLabel->"In[11]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cos", "[", 
   RowBox[{"a", " ", "x"}], "]"}], "+", 
  RowBox[{"Cos", "[", 
   RowBox[{"c", " ", "x"}], "]"}], "+", 
  RowBox[{"(", 
   TagBox[
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", "b", " ", "x"}]]}], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"\[ImaginaryI]", " ", "b", " ", "x"}]]}]}],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], ")"}]}]], "Output",
 CellChangeTimes->{3.5953551114720583`*^9, 3.595784766555786*^9},
 CellLabel->"Out[11]="]
}, Open  ]],

Cell["\<\
Here s an example using a rule. Note carefully that RR always returns a pure \
function whatever its argument(s), and so should be preceded by the '//' \
operator:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Cos", "[", 
    RowBox[{"a", " ", "x"}], "]"}], "+", 
   TagBox[
    RowBox[{"Cos", "[", 
     RowBox[{"b", " ", "x"}], "]"}],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]}], "+", 
   RowBox[{"Cos", "[", 
    RowBox[{"c", " ", "x"}], "]"}]}], "//", 
  RowBox[{"RR", "[", 
   RowBox[{"x", "->", "y"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5953550994803724`*^9, 3.595355104054634*^9}},
 CellLabel->"In[12]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cos", "[", 
   RowBox[{"a", " ", "x"}], "]"}], "+", 
  RowBox[{"Cos", "[", 
   RowBox[{"c", " ", "x"}], "]"}], "+", 
  TagBox[
   RowBox[{"Cos", "[", 
    RowBox[{"b", " ", "y"}], "]"}],
   ColourMaths`Red\[Breve]Colour,
   DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
   SelectWithContents->True]}]], "Output",
 CellChangeTimes->{3.5953551244337997`*^9, 3.5957847693149443`*^9},
 CellLabel->"Out[12]="]
}, Open  ]],

Cell["\<\
These functions can take multiple arguments, which are used in turn, so the \
previous two transformations could be combined thus:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Cos", "[", 
    RowBox[{"a", " ", "x"}], "]"}], "+", 
   TagBox[
    RowBox[{"Cos", "[", 
     RowBox[{"b", " ", "x"}], "]"}],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]}], "+", 
   RowBox[{"Cos", "[", 
    RowBox[{"c", " ", "x"}], "]"}]}], "//", 
  RowBox[{"RR1", "[", 
   RowBox[{"TrigToExp", ",", 
    RowBox[{"x", "->", "y"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.595355137075523*^9, 3.5953551428578534`*^9}},
 CellLabel->"In[13]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{
     RowBox[{"-", "\[ImaginaryI]"}], " ", "b", " ", "y"}]]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{"\[ImaginaryI]", " ", "b", " ", "y"}]]}], "+", 
  RowBox[{"Cos", "[", 
   RowBox[{"a", " ", "x"}], "]"}], "+", 
  RowBox[{"Cos", "[", 
   RowBox[{"c", " ", "x"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5953551479661455`*^9, 3.595784772325116*^9},
 CellLabel->"Out[13]="]
}, Open  ]],

Cell["\<\
Since we have presumably finished with the red colour at this point, we have \
used the non colour preserving variant RR1.\
\>", "Text"],

Cell[TextData[{
 "Notice that there is no reason why the function or rule need represent a \
mathematical identity, so these functions can be used to perform many kinds \
of operations. As just one example, suppose we are manipulating special \
functions and decide to use a private notation except when ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " evaluation is required. We might define a rule set such as this:"
}], "Text"],

Cell[BoxData[
 RowBox[{"myrule", ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", "x_", "]"}], "->", 
     RowBox[{"Gamma", "[", "x", "]"}]}], ",", 
    RowBox[{
     RowBox[{"\[Zeta]", "[", "z_", "]"}], "->", 
     RowBox[{"Zeta", "[", "z", "]"}]}]}], "}"}]}]], "Input",
 CellLabel->"In[14]:="],

Cell["\<\
Using colour we could apply this rule set exactly where required:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"\[CapitalGamma]", "[", "a", "]"}], 
    TagBox[
     RowBox[{"\[CapitalGamma]", "[", "4", "]"}],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]}]}], 
   RowBox[{"\[CapitalGamma]", "[", "c", "]"}]], "//", 
  RowBox[{"RR1", "[", "myrule", "]"}]}]], "Input",
 CellChangeTimes->{{3.5953552023032537`*^9, 3.595355211560783*^9}},
 CellLabel->"In[15]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"6", " ", 
   RowBox[{"\[CapitalGamma]", "[", "a", "]"}]}], 
  RowBox[{"\[CapitalGamma]", "[", "c", "]"}]]], "Output",
 CellChangeTimes->{{3.5953552203352847`*^9, 3.595355226440634*^9}, 
   3.5957847831357346`*^9},
 CellLabel->"Out[15]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Collecting coloured expressions", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["Consider the following expression:", "Text",
 PageWidth->WindowWidth,
 CellTags->"Precedence of Operators"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["a", "2"], "+", 
  SuperscriptBox["b", "2"], "-", 
  SuperscriptBox["c", "2"], "-", 
  SuperscriptBox["d", "2"]}]], "Input"],

Cell[TextData[{
 "Clearly, from a mathematical point of view ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "2"], "-", 
    SuperscriptBox["d", "2"]}], TraditionalForm]]],
 " is a subexpression. However, because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has not printed these parts of the expression together, there is no way to \
select, and hence colour this subexpression. The solution is to use the RC \
(RedCoalesce), GC, and BC functions. These functions will force terms \
together thus:"
}], "Text",
 PageWidth->WindowWidth],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   TagBox[
    SuperscriptBox["a", "2"],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], "+", 
   SuperscriptBox["b", "2"], "-", 
   SuperscriptBox["c", "2"], "-", 
   TagBox[
    SuperscriptBox["d", "2"],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True]}], "//", "RC"}]], "Input",
 CellChangeTimes->{{3.595784982167118*^9, 3.5957850187822123`*^9}},
 CellLabel->"In[17]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["b", "2"], "-", 
  SuperscriptBox["c", "2"], "+", 
  RowBox[{"(", 
   TagBox[
    RowBox[{
     SuperscriptBox["a", "2"], "-", 
     SuperscriptBox["d", "2"]}],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], ")"}]}]], "Output",
 CellChangeTimes->{3.595784975504737*^9, 3.5957850226954365`*^9},
 CellLabel->"Out[17]="]
}, Open  ]],

Cell["\<\
Typically these functions are combined with one of the other colour \
manipulation functions, for example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    TagBox[
     StyleBox[
      SuperscriptBox["a", "2"],
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True], "+", 
    SuperscriptBox["b", "2"], "-", 
    SuperscriptBox["c", "2"], "-", 
    TagBox[
     StyleBox[
      SuperscriptBox["d", "2"],
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True]}], "//", "RC"}], "//", 
  RowBox[{"RR1", "[", "Factor", "]"}]}]], "Input",
 CellChangeTimes->{{3.5957854177900343`*^9, 3.5957854275335917`*^9}},
 CellLabel->"In[26]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["b", "2"], "-", 
  SuperscriptBox["c", "2"], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "d"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "d"}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.5957850301128607`*^9, 3.5957854314418154`*^9},
 CellLabel->"Out[26]="]
}, Open  ]],

Cell["\<\
Without coalescing the two red expressions Factor gets applied to them \
separately and achieves nothing.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   TagBox[
    StyleBox[
     SuperscriptBox["a", "2"],
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], "+", 
   SuperscriptBox["b", "2"], "-", 
   SuperscriptBox["c", "2"], "-", 
   TagBox[
    StyleBox[
     SuperscriptBox["d", "2"],
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True]}], "//", 
  RowBox[{"RR1", "[", "Factor", "]"}]}]], "Input",
 CellChangeTimes->{{3.595785482329726*^9, 3.5957854875120225`*^9}},
 CellLabel->"In[29]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["a", "2"], "+", 
  SuperscriptBox["b", "2"], "-", 
  SuperscriptBox["c", "2"], "-", 
  SuperscriptBox["d", "2"]}]], "Output",
 CellChangeTimes->{3.595785445561623*^9, 3.595785490778209*^9},
 CellLabel->"Out[29]="]
}, Open  ]],

Cell["\<\
Although non colour preserving versions of these functions exist, they are \
not often used because in many instances (such as the above) without the \
colour the expression collapses to its original form.\
\>", "Text"],

Cell["\<\
Rather than merge two coloured expressions, it is sometimes useful to expand \
a coloured region to cover the immediate containing expression. This can be \
done with the functions Rgrow, Ggrow, and Bgrow. Possibly the most \
application of these functions is to colour negative components of additions. \
For example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "-", 
   TagBox[
    StyleBox["b",
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True]}], "//", "FullForm"}]], "Input",
 CellChangeTimes->{3.595785515563627*^9},
 CellLabel->"In[31]:="],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Plus", "[", 
    RowBox[{"a", ",", 
     RowBox[{"Times", "[", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"Red\\[Breve]Colour", "[", "b", "]"}]}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{
  3.595783242249601*^9, {3.595785510617344*^9, 3.595785518369787*^9}},
 CellLabel->"Out[31]//FullForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a", "-", 
    TagBox[
     StyleBox["b",
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True]}], "//", "Rgrow"}], "//", 
  "FullForm"}]], "Input",
 CellChangeTimes->{3.5957855580770583`*^9},
 CellLabel->"In[33]:="],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Plus", "[", 
    RowBox[{"a", ",", 
     RowBox[{"Red\\[Breve]Colour", "[", 
      RowBox[{"Times", "[", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "b"}], "]"}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{{3.595785553001768*^9, 3.5957855591251183`*^9}},
 CellLabel->"Out[33]//FullForm="]
}, Open  ]],

Cell["\<\
Without this device there would be no way to pick out the subexpression (-b) \
as opposed to the subexpression b.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moving expressions 'out'", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["\<\
The functions Rout, Gout, and Bout are used to move subexpressions further \
out in an expression. The context of moving an expression out of another is \
well defined, but its exact meaning depends on context. Here are some \
examples:\
\>", "Text",
 PageWidth->WindowWidth,
 CellTags->"Must use palette"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    TagBox["a",
     ColourMaths`Green\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
     SelectWithContents->True], " ", "b"}], "+", 
   RowBox[{"a", " ", "c", " ", "d"}]}], "//", "Gout1"}]], "Input",
 CellChangeTimes->{{3.595785589033829*^9, 3.5957855966972675`*^9}},
 CellLabel->"In[35]:="],

Cell[BoxData[
 RowBox[{"a", " ", 
  RowBox[{"(", 
   RowBox[{"b", "+", 
    RowBox[{"c", " ", "d"}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.5957855794822826`*^9, 3.5957855992534137`*^9}},
 CellLabel->"Out[35]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   RowBox[{"a", " ", 
    TagBox[
     StyleBox["b",
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True], " ", "c"}], "]"}], "//", "Rout"}]], "Input",
 CellChangeTimes->{3.5957856105790615`*^9},
 CellLabel->"In[36]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", 
   RowBox[{"a", " ", "c"}], "]"}], "+", 
  RowBox[{"Log", "[", 
   TagBox["b",
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], "]"}]}]], "Output",
 CellChangeTimes->{3.595785613094205*^9},
 CellLabel->"Out[36]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox["E", 
   RowBox[{
    RowBox[{"a", " ", "b"}], "+", 
    TagBox[
     RowBox[{"c", "+", "d"}],
     ColourMaths`Blue\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
     SelectWithContents->True], "+", 
    RowBox[{"e", " ", "f"}]}]], "//", "Bout"}]], "Input",
 CellChangeTimes->{{3.5957856302581873`*^9, 3.595785642390881*^9}},
 CellLabel->"In[37]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[ExponentialE]", 
   RowBox[{
    RowBox[{"a", " ", "b"}], "+", 
    RowBox[{"e", " ", "f"}]}]], " ", 
  TagBox[
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{"c", "+", "d"}]],
   ColourMaths`Blue\[Breve]Colour,
   DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
   SelectWithContents->True]}]], "Output",
 CellChangeTimes->{3.595785646485115*^9},
 CellLabel->"Out[37]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox["a", 
     TagBox[
      RowBox[{"x", "+", "1"}],
      ColourMaths`Green\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
      SelectWithContents->True]], "+", 
    FractionBox["b", 
     TagBox[
      StyleBox[
       RowBox[{
        SuperscriptBox["x", "2"], "-", "1"}],
       FontColor->RGBColor[1, 0, 0]],
      ColourMaths`Red\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
      SelectWithContents->True]]}], "//", 
   RowBox[{"RR1", "[", "Factor", "]"}]}], "//", "Gout1"}]], "Input",
 CellChangeTimes->{{3.595785663648097*^9, 3.5957856864974036`*^9}},
 CellLabel->"In[39]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"a", "+", 
   FractionBox["b", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "x"}]]}], 
  RowBox[{"1", "+", "x"}]]], "Output",
 CellChangeTimes->{{3.5957856684513717`*^9, 3.5957856906916437`*^9}},
 CellLabel->"Out[39]="]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Moving expressions 'in'", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["\<\
Although it is common to talk about moving expressions inside others, the \
concept can be rather more ambiguous than the reverse operation. Colour Math \
supports several such operations to deal with the different cases. You can \
always use the most general form, which will work in all cases, if desired.
The simplest case is where the 'move in' operation is totally unambiguous, \
for example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   TagBox[
    StyleBox[
     SqrtBox["a"],
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], " ", 
   SqrtBox["b"]}], "//", "Rin"}]], "Input",
 CellChangeTimes->{3.5957857382423635`*^9},
 CellLabel->"In[41]:="],

Cell[BoxData[
 TagBox[
  SqrtBox[
   RowBox[{"a", " ", "b"}]],
  ColourMaths`Red\[Breve]Colour,
  DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{{3.5957857315819826`*^9, 3.5957857419215736`*^9}},
 CellLabel->"Out[41]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}], 
   TagBox[
    StyleBox["c",
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], " ", "d"}], "//", "Rin1"}]], "Input",
 CellChangeTimes->{3.5957857486169567`*^9},
 CellLabel->"In[42]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"a", " ", "c"}], "+", 
    RowBox[{"b", " ", "c"}]}], ")"}], " ", "d"}]], "Output",
 CellChangeTimes->{3.595785750630072*^9},
 CellLabel->"Out[42]="]
}, Open  ]],

Cell["\<\
An intermediate case exists in which it suffices to mark two subexpressions \
with the same colour to specify a 'move in' operation. For example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Log", "[", "a", "]"}], "+", 
   TagBox[
    StyleBox[
     RowBox[{"Log", "[", "b", "]"}],
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], "+", 
   RowBox[{"Log", "[", "c", "]"}], "+", 
   TagBox[
    StyleBox[
     RowBox[{"Log", "[", "d", "]"}],
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True]}], "//", "RRin1"}]], "Input",
 CellChangeTimes->{{3.5957857557113624`*^9, 3.5957857601056137`*^9}},
 CellLabel->"In[43]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", "a", "]"}], "+", 
  RowBox[{"Log", "[", "c", "]"}], "+", 
  RowBox[{"Log", "[", 
   RowBox[{"b", " ", "d"}], "]"}]}]], "Output",
 CellChangeTimes->{3.595785761524695*^9},
 CellLabel->"Out[43]="]
}, Open  ]],

Cell["\<\
In the most general case it is necessary to use two distinct colours:\
\>", "Text"],

Cell[BoxData[""], "Text",
 PageWidth->WindowWidth,
 GridBoxOptions->{GridBoxSpacings->{"Columns" -> {
     Offset[0.27999999999999997`], {
      Offset[0.5599999999999999]}, 
     Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
     Offset[0.2], 
     Offset[0.8], 
     Offset[0.8], {
      Offset[0.4]}, 
     Offset[0.2]}, "RowsIndexed" -> {}}},
 CellTags->"Action.Definition"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   TagBox[
    StyleBox[
     RowBox[{"a", "+", "b"}],
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], 
   RowBox[{"(", 
    RowBox[{"c", "+", "d"}], ")"}], 
   TagBox[
    RowBox[{"e", "+", "f"}],
    ColourMaths`Blue\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
    SelectWithContents->True]}], "//", "RBin1"}]], "Input",
 CellChangeTimes->{{3.5957857679700637`*^9, 3.5957858064772663`*^9}},
 CellLabel->"In[44]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"c", "+", "d"}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "+", "b"}], ")"}], " ", "e"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "+", "b"}], ")"}], " ", "f"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.595785809945464*^9},
 CellLabel->"Out[44]="]
}, Open  ]],

Cell["", "Caption",
 PageWidth->WindowWidth],

Cell["Here are a few more examples:", "Text"],

Cell[BoxData[""], "Text",
 PageWidth->WindowWidth,
 GridBoxOptions->{GridBoxSpacings->{"Columns" -> {
     Offset[0.27999999999999997`], {
      Offset[0.5599999999999999]}, 
     Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
     Offset[0.2], 
     Offset[0.8], 
     Offset[0.8], {
      Offset[0.4]}, 
     Offset[0.2]}, "RowsIndexed" -> {}}},
 CellTags->"WorkingForm.Definition"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   TagBox["s",
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], " ", 
   TagBox[
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "0"}], "\[Infinity]"], 
     RowBox[{"f", "[", "i", "]"}]}],
    ColourMaths`Sum\[Breve]Inert,
    DefaultBaseStyle->{
     Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
    SelectWithContents->True]}], "//", "Rin"}]], "Input",
 CellChangeTimes->{{3.595785873946125*^9, 3.595785913351379*^9}},
 CellLabel->"In[45]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"i", "=", "0"}], "\[Infinity]"], 
   RowBox[{
    RowBox[{"f", "[", "i", "]"}], " ", 
    TagBox["s",
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True]}]}],
  ColourMaths`Sum\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.5957859154935017`*^9},
 CellLabel->"Out[45]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a", " ", 
   TagBox[
    StyleBox["b",
     FontColor->RGBColor[1, 0, 0]],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["c", "2"], "+", 
     SuperscriptBox["d", "2"]}], ")"}]}], "//", "Rin1"}]], "Input",
 CellChangeTimes->{3.595785930626367*^9},
 CellLabel->"In[46]:="],

Cell[BoxData[
 RowBox[{"a", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"b", " ", 
     SuperscriptBox["c", "2"]}], "+", 
    RowBox[{"b", " ", 
     SuperscriptBox["d", "2"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.5957859319284415`*^9},
 CellLabel->"Out[46]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moving expressions 'over'", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["\<\
The functions Rover, Gover, and Bover move subexpressions to the other side \
of equalities and inequalities. The precise meaning depends on the context. \
Note that it is easy to do invalid things with inequalities - such as \
dividing both sides by something negative - so it is vital not to use these \
functions blindly. The manipulation of equalities is much safer. Here are \
some examples:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "2"], "+", 
    SuperscriptBox["y", "2"], "-", 
    TagBox["a",
     ColourMaths`Blue\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
     SelectWithContents->True]}], "==", "0"}], "//", "Bover"}]], "Input",
 CellChangeTimes->{{3.595786011190975*^9, 3.595786017624343*^9}},
 CellLabel->"In[48]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["x", "2"], "+", 
   SuperscriptBox["y", "2"]}], "\[Equal]", 
  TagBox["a",
   ColourMaths`Blue\[Breve]Colour,
   DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
   SelectWithContents->True]}]], "Output",
 CellChangeTimes->{{3.5957859972121754`*^9, 3.595786020281495*^9}},
 CellLabel->"Out[48]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"(", 
      RowBox[{"x", "+", "y"}], ")"}], 
     TagBox[
      StyleBox[
       RowBox[{"x", "-", "y"}],
       FontColor->RGBColor[1, 0, 0]],
      ColourMaths`Red\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
      SelectWithContents->True]], "==", "p"}], "//", "Rover"}], "//", 
  "Rin1"}]], "Input",
 CellChangeTimes->{3.5957860351943483`*^9},
 CellLabel->"In[49]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"x", "+", "y"}], "\[Equal]", 
  RowBox[{"p", " ", 
   RowBox[{"(", 
    RowBox[{"x", "-", "y"}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.5957860392405796`*^9},
 CellLabel->"Out[49]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"a", " ", "b"}], "+", 
    RowBox[{
     TagBox["b",
      ColourMaths`Green\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
      SelectWithContents->True], " ", "c", " ", "d"}]}], "==", "e"}], "//", 
  "Gover1"}]], "Input",
 CellChangeTimes->{{3.5957860528633585`*^9, 3.5957860589577074`*^9}},
 CellLabel->"In[50]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "+", 
   RowBox[{"c", " ", "d"}]}], "\[Equal]", 
  FractionBox["e", "b"]}]], "Output",
 CellChangeTimes->{3.595786061290841*^9},
 CellLabel->"Out[50]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multiply top and bottom by ...", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell[TextData[{
 "Frequently it is useful to manipulate an expression by adding and \
subtracting the same term, or by multiplying and dividing the same term. The \
result is unchanged of course, but the aim is to combine this maneuver with \
other operations to achieve something useful. You will encounter two problems \
if you try to perform such operations with 'raw' ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":\n\[Bullet] You may want to perform the manipulation deep inside an \
expression, not on the top level.\n\[Bullet] Until you have performed some \
further operations ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is quite likely to 'simplify' your expression back to its original form."
}], "Text",
 CellChangeTimes->{3.596118942767359*^9}],

Cell["\<\
To solve these problems a set of six operations are supplied which operate as \
in the following example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{
     TagBox["a",
      ColourMaths`Green\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
      SelectWithContents->True], ",", 
     TagBox[
      StyleBox["b",
       FontColor->RGBColor[1, 0, 0]],
      ColourMaths`Red\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
      SelectWithContents->True]}], "]"}], "//", 
   RowBox[{"Rmpy", "[", "2", "]"}]}], "//", 
  RowBox[{"Gadd", "[", "n", "]"}]}]], "Input",
 CellChangeTimes->{{3.5957860865552855`*^9, 3.595786098811987*^9}},
 CellLabel->"In[51]:="],

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "n"}], "+", 
    RowBox[{"(", 
     TagBox[
      RowBox[{"a", "+", "n"}],
      ColourMaths`Green\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
      SelectWithContents->True], ")"}]}], ",", 
   FractionBox[
    TagBox[
     RowBox[{"2", " ", "b"}],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True], "2"]}], "]"}]], "Output",
 CellChangeTimes->{3.5957861037382684`*^9},
 CellLabel->"Out[51]="]
}, Open  ]],

Cell["\<\
The desired operation is performed inside the coloured expression, and the \
inverse operation is applied outside the colour (so that the value of the \
expression, disregarding colour, remains the same). Since the result would at \
once collapse if the colour were removed before other transformations were \
applied, no non colour preserving version of these functions is supplied.\
\>", "Text"],

Cell["Here is a more interesting example:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         TagBox[
          StyleBox[
           RowBox[{"1", "+", "q", "+", 
            SuperscriptBox["q", "2"], "+", 
            SuperscriptBox["q", "3"]}],
           FontColor->RGBColor[1, 0, 0]],
          ColourMaths`Red\[Breve]Colour,
          DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
          SelectWithContents->True], ")"}], " ", 
        RowBox[{"(", 
         TagBox[
          RowBox[{"1", "+", "q", "+", 
           SuperscriptBox["q", "2"]}],
          ColourMaths`Green\[Breve]Colour,
          DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
          SelectWithContents->True], ")"}], 
        RowBox[{"(", 
         TagBox[
          RowBox[{"1", "+", "q"}],
          ColourMaths`Blue\[Breve]Colour,
          DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
          SelectWithContents->True], ")"}]}], "//", 
       RowBox[{"Rmpy", "[", 
        RowBox[{"1", "-", "q"}], "]"}]}], "//", 
      RowBox[{"Gmpy", "[", 
       RowBox[{"1", "-", "q"}], "]"}]}], "//", 
     RowBox[{"Bmpy", "[", 
      RowBox[{"1", "-", "q"}], "]"}]}], "//", 
    RowBox[{"RR1", "[", "Expand", "]"}]}], "//", 
   RowBox[{"GG1", "[", "Expand", "]"}]}], "//", 
  RowBox[{"BB1", "[", "Expand", "]"}]}]], "Input",
 CellChangeTimes->{{3.595786113799844*^9, 3.595786139520315*^9}, {
  3.595786212408484*^9, 3.5957862927670803`*^9}, {3.5957864035824184`*^9, 
  3.595786404758486*^9}},
 CellLabel->"In[56]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox["q", "2"]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox["q", "3"]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox["q", "4"]}], ")"}]}], 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", "q"}], ")"}], "3"]]], "Output",
 CellChangeTimes->{
  3.595786141832447*^9, {3.595786264293452*^9, 3.595786293409117*^9}, 
   3.595786406022558*^9},
 CellLabel->"Out[56]="]
}, Closed]],

Cell["\<\
Notice how the coloured product or sum is ready for another colour \
manipulation operation, in this case Expand.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extracting expressions to another notebook", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["\<\
The philosophy of using coloured expressions is perform as much as possible \
without re-writing or pasting operations, which do not leave any record in \
the notebook. However, sometimes it is convenient to extract part of an \
expression and manipulate it on its own in a separate notebook, pasting the \
result back in at the end of the process. If you do this you must accept that \
your notebook will not contain a full record of your manipulations.
To extract a subexpression first colour it in the usual way and then press \
the large 'X' on the palette of the appropriate colour. Note that there are \
no functions corresponding to these palette items. A new notebook will be \
created with the subexpression pasted into it. This notebook will also \
contain a button marked 'Return' which you should press to return the last \
result that you produce. Do not press this button until you have evaluated an \
expression of some sort. 
When you press the 'Return' button the temporary notebook will close and the \
result will be pasted back into your full expression.\
\>", "Text",
 CellChangeTimes->{3.5961189596333237`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions with many parameters", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True,
 CellChangeTimes->{3.5961189720780354`*^9}],

Cell[TextData[{
 "Certain mathematical functions, such as the (generalised) hypergeometric \
functions and the MeijerG function depend on many parameters, which are \
arranged in lists. Consider for example the function \
HypergeometricPFQ[{3/2,2},{1,2},z]. Here we have two lists of parameters, and \
it is a basic fact \[Dash] obvious from the definition of these functions \
\[Dash] that the order of parameters within each list is irrelevant. Suppose \
now that you wish to apply a transformation rule to a hypergeometric \
function, but the order of the parameters in the lists does not match those \
in the rule. Because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " applies transformation rules in an essentially blind way it is necessary \
to sort the order of parameters before applying such a rule (Alternatively \
you could define many rules to cover each parameter ordering, but this \
quickly becomes unmanageable).\nTo handle such problems using colour, a set \
of functions are supplied to sort the order of parameters in lists by colour. \
Lists are sorted into the order red, then green, then blue, then uncoloured \
expressions. For example:\n"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      TagBox["a",
       ColourMaths`Blue\[Breve]Colour,
       DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
       SelectWithContents->True], ",", 
      TagBox["b",
       ColourMaths`Green\[Breve]Colour,
       DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
       SelectWithContents->True], ",", "1", ",", 
      TagBox[
       StyleBox["2",
        FontColor->RGBColor[1, 0, 0]],
       ColourMaths`Red\[Breve]Colour,
       DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
       SelectWithContents->True]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"d", ",", "e", ",", "f"}], "}"}], ",", "z"}], "]"}], "//", 
  RowBox[{"CS", "[", "]"}]}]], "Input",
 CellChangeTimes->{{3.5957864654829593`*^9, 3.595786516565881*^9}},
 CellLabel->"In[58]:="],

Cell[BoxData[
 RowBox[{"F", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     TagBox["2",
      ColourMaths`Red\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
      SelectWithContents->True], ",", 
     TagBox["b",
      ColourMaths`Green\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
      SelectWithContents->True], ",", 
     TagBox["a",
      ColourMaths`Blue\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
      SelectWithContents->True], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"d", ",", "e", ",", "f"}], "}"}], ",", "z"}], "]"}]], "Output",
 CellChangeTimes->{{3.5957864979968185`*^9, 3.595786519472047*^9}},
 CellLabel->"Out[58]="]
}, Open  ]],

Cell["\<\
As usual, the sorting operation can be followed by removing the colour. In \
this case all three colours are removed:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      TagBox["a",
       ColourMaths`Blue\[Breve]Colour,
       DefaultBaseStyle->{FontColor -> RGBColor[0, 0, 1]},
       SelectWithContents->True], ",", 
      TagBox["b",
       ColourMaths`Green\[Breve]Colour,
       DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
       SelectWithContents->True], ",", "1", ",", 
      TagBox[
       StyleBox["2",
        FontColor->RGBColor[1, 0, 0]],
       ColourMaths`Red\[Breve]Colour,
       DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
       SelectWithContents->True]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"d", ",", "e", ",", "f"}], "}"}], ",", "z"}], "]"}], "//", 
  RowBox[{"CS1", "[", "]"}]}]], "Input",
 CellChangeTimes->{3.595786567935819*^9},
 CellLabel->"In[59]:="],

Cell[BoxData[
 RowBox[{"F", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "b", ",", "a", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"d", ",", "e", ",", "f"}], "}"}], ",", "z"}], "]"}]], "Output",
 CellChangeTimes->{3.5957865712220073`*^9},
 CellLabel->"Out[59]="]
}, Open  ]],

Cell[TextData[{
 StyleBox["Hint",
  FontSlant->"Italic"],
 ": If you wish to manipulate special functions such as hypergeometric \
functions you may find it useful to use a notation, such as that above, which \
is not recognised by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". This prevents the system applying spurious 'simplifications' such as \
converting hypergeometric functions to Bessel functions. Later, when \
required, the rule F->HypergeometricPFQ will switch to standard ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notation if desired."
}], "Text",
 CellChangeTimes->{{3.596118995691386*^9, 3.596119001988746*^9}}],

Cell["\<\
By default these functions sort lists only. However, sometimes it is more \
useful to apply these functions in a more general context by specifying the \
head of the term to be sorted. For example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", 
   RowBox[{"a", ",", 
    TagBox[
     StyleBox["b",
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True], ",", "c"}], "]"}], "//", 
  RowBox[{"CS1", "[", "g", "]"}]}]], "Input",
 CellChangeTimes->{3.595786579926505*^9},
 CellLabel->"In[60]:="],

Cell[BoxData[
 RowBox[{"g", "[", 
  RowBox[{"b", ",", "a", ",", "c"}], "]"}]], "Output",
 CellChangeTimes->{3.5957865821546326`*^9},
 CellLabel->"Out[60]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A word of caution", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell[TextData[{
 "As you probably realise, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not automatically expand terms such as ",
 Cell[BoxData[
  SqrtBox[
   SuperscriptBox["x", "2"]]]],
 ". The reason for this is that if x is negative or complex this is not \
valid. Many mathematical operations are only valid under certain conditions. \
Since colour manipulations are designed for manual use, the system 'assumes' \
that you know what you are doing. Of course, many of the traditional ways of \
manipulating ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expressions using Take, Part, etc. do not make these checks either. Here \
are some of the issues to watch for when using colour transformations:\n\
\[Bullet] Operations involving exponents are performed under the same \
assumptions as are used by PowerExpand.\n\[Bullet] Likewise, operations \
involving logarithms uses the transformation Log[a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "]"}], TraditionalForm]]],
 "\[DoubleLongLeftRightArrow]Log[a] Log[b]. This can be invalid if complex \
numbers are involved.\n\[Bullet] If you select part of an expression by \
dragging the mouse (as opposed to repeated clicking) it is possible to select \
parts of an expression which appear to be legal sub-expressions, but which \
are not. Here is a simple example:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "+", 
   RowBox[{"b", 
    TagBox[
     StyleBox[
      RowBox[{"-", "c"}],
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True]}], "+", "d"}], "//", "FullForm"}]], "Input",
 CellChangeTimes->{3.5957865973515015`*^9},
 CellLabel->"In[61]:="],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Plus", "[", 
    RowBox[{"a", ",", "d", ",", 
     RowBox[{"Times", "[", 
      RowBox[{"b", ",", 
       RowBox[{"Red\\[Breve]Colour", "[", 
        RowBox[{"Times", "[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "c"}], "]"}], "]"}]}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.595786598563571*^9},
 CellLabel->"Out[61]//FullForm="]
}, Open  ]],

Cell["\<\
Here the legal expression, -c, was extracted from the outer expression in an \
illegal way - essentially because drag selection can easily be used to select \
invalid sub-expressions. The best way to avoid this problem is to select \
using repeated clicks (at least where confusion might exist) and use Rgrow \
etc. to pickup a minus sign:\
\>", "Text",
 CellChangeTimes->{3.5961190332155323`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a", "+", "b", "-", 
    TagBox[
     StyleBox["c",
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True], "+", "d"}], "//", "Rgrow"}], "//", 
  "FullForm"}]], "Input",
 CellChangeTimes->{3.5957866186647205`*^9},
 CellLabel->"In[62]:="],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Plus", "[", 
    RowBox[{"a", ",", "b", ",", "d", ",", 
     RowBox[{"Red\\[Breve]Colour", "[", 
      RowBox[{"Times", "[", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "c"}], "]"}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.595786620004797*^9},
 CellLabel->"Out[62]//FullForm="]
}, Open  ]],

Cell["\<\

\[Bullet] Care should be taken when moving a multiplicative factor to the \
other side of an inequality. Negative multipliers can give incorrect results.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Adding your own extensions", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["\<\
Because the colour functions are represented in the Kernel as functions with \
no definition, it is easy to create rules which use colour to perform an \
operation of your own. For example, suppose you needed to perform 2-term \
series expansions on subexpressions regularly. You could try something like:\
\>", "Text",
 PageWidth->WindowWidth],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", "x"}], ")"}], "3"], ",", 
    TagBox[
     StyleBox[
      RowBox[{"Exp", "[", "x", "]"}],
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True]}], "]"}], "//", 
  RowBox[{"RR1", "[", 
   RowBox[{"u_", ":>", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{"u", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "2"}], "}"}]}], "]"}], "]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{3.5957866378768196`*^9},
 CellLabel->"In[63]:="],

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}], "3"], ",", 
   RowBox[{"1", "+", "x", "+", 
    FractionBox[
     SuperscriptBox["x", "2"], "2"]}]}], "]"}]], "Output",
 CellChangeTimes->{3.5957866396289196`*^9},
 CellLabel->"Out[63]="]
}, Open  ]],

Cell["\<\
However, this is very clumsy to use on a regular basis, so an alternative is \
to define a function to operate on the colour directly:\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RedExpand", "[", "u_", "]"}], ":=", 
  RowBox[{"u", "/.", 
   RowBox[{
    RowBox[{"Red\[Breve]Colour", "[", "s_", "]"}], ":>", 
    RowBox[{"Normal", "[", 
     RowBox[{"Series", "[", 
      RowBox[{"s", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "2"}], "}"}]}], "]"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.595786670107663*^9, 3.59578667162775*^9}},
 CellLabel->"In[2]:="],

Cell["\<\
Now you can use this function directly whenever it is needed simply by \
colouring one or more subexpressions red:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", "x"}], ")"}], "3"], ",", 
    TagBox[
     StyleBox[
      RowBox[{"Exp", "[", "x", "]"}],
      FontColor->RGBColor[1, 0, 0]],
     ColourMaths`Red\[Breve]Colour,
     DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
     SelectWithContents->True]}], "]"}], "//", "RedExpand"}]], "Input",
 CellChangeTimes->{3.5957866489044504`*^9},
 CellLabel->"In[3]:="],

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}], "3"], ",", 
   RowBox[{"1", "+", "x", "+", 
    FractionBox[
     SuperscriptBox["x", "2"], "2"]}]}], "]"}]], "Output",
 CellChangeTimes->{{3.595786650088518*^9, 3.595786676977056*^9}, 
   3.595869158178726*^9},
 CellLabel->"Out[3]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Defining inert mathematical operations", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell["", "Text"],

Cell["Consider the following summation:", "Text"],

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"n", "=", "1"}], "1000"], 
  SubscriptBox["a", "n"]}]], "Input"],

Cell[TextData[{
 "If you give such a sum to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", it will 'evaluate' it - producing an expression with 1000 terms, which is \
almost certainly not what is required! Summations with infinite or variable \
limits will either evaluate or remain unevaluated, however there is no \
built-in way to represent a sum without letting the system take a shot at \
evaluating it. Clearly, there are many cases, such as when one is trying to \
prove results about a sum, where one does not want this evaluation to even be \
attempted. The same is true for definite integrals, products, and limits. To \
facilitate the manipulation of these objects, the ColourMaths package \
contains inert variants of these constructs. These can be created by \
selecting the expression in question and pressing the \"Inert\" button on the \
palette."
}], "Text",
 CellChangeTimes->{{3.5957867075838065`*^9, 3.5957867307191296`*^9}, {
  3.5958704760871067`*^9, 3.5958704769741573`*^9}, {3.595870646053828*^9, 
  3.5958706905963755`*^9}}],

Cell[BoxData[GridBox[{
   {
    StyleBox[
     RowBox[{"Active", " ", "head"}],
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox[
     RowBox[{"Inert", " ", "head"}],
     FontColor->RGBColor[0, 0, 1]]},
   {"Integrate", "Integrate\[Breve]Inert"},
   {"Sum", "Sum\[Breve]Inert"},
   {"Product", "Product\[Breve]Inert"},
   {"Limit", "Limit\[Breve]Inert"}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]], "Input",
 CellChangeTimes->{{3.595786747914113*^9, 3.5957867756486998`*^9}}],

Cell["\<\
All passive integrals and sums are assumed to be definite, indefinite \
integrals are converted to definite intert integrals by the \"Inert\" button. \
An indefinite operation is specified by using a variable as the upper limit.
Inert constructs have a shaded background to distinguish them from normal \
operations. Where inert operations are nested inside other inert operations, \
this shading is made deeper.\
\>", "Text",
 CellChangeTimes->{{3.5958707140757184`*^9, 3.5958708574749203`*^9}}],

Cell[TextData[{
 "Clearly it would be possible to add further inert operations to this list, \
such as differentiation. As we will see in the next section, these inert \
operations allow us to represent theorems involving sums, integrals etc. \
(such as the binomial theorem) whereas if the sum in the binomial theorem \
were represented using Sum, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " would simply apply the binomial theorem and evaluate the sum! "
}], "Text",
 CellChangeTimes->{{3.5958707140757184`*^9, 3.5958707716440115`*^9}}],

Cell[TextData[{
 "Although it would be possible to represent these objects using ",
 StyleBox["HoldForm", "ObjectText"],
 " (the invisible form of ",
 StyleBox["Hold", "ObjectText"],
 "), in practice this is not very convenient as it suppresses other \
evaluations which may well be required. For example:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HoldForm", "[", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"n", ",", "8"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "0", ",", "1000"}], "}"}]}], "]"}], "]"}], "/.", 
  RowBox[{
   RowBox[{"a", "[", 
    RowBox[{"p_", ",", "q_"}], "]"}], ":>", 
   RowBox[{
    RowBox[{
     SuperscriptBox["2", "q"], 
     RowBox[{"b", "[", "p", "]"}]}], "/;", 
    RowBox[{"EvenQ", "[", "q", "]"}]}]}]}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"n", "=", "0"}], "1000"], 
   RowBox[{
    SuperscriptBox["2", "8"], " ", 
    RowBox[{"b", "[", "n", "]"}]}]}],
  HoldForm]], "Output"]
}, Open  ]],

Cell["\<\
Compare this with the same operations performed with an inert sum and without \
HoldForm:\
\>", "Text",
 CellChangeTimes->{{3.5958709884664125`*^9, 3.595871024578478*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "1000"], 
    RowBox[{"a", "[", 
     RowBox[{"n", ",", "8"}], "]"}]}],
   ColourMaths`Sum\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "/.", 
  RowBox[{
   RowBox[{"a", "[", 
    RowBox[{"p_", ",", "q_"}], "]"}], ":>", 
   RowBox[{
    RowBox[{
     SuperscriptBox["2", "q"], 
     RowBox[{"b", "[", "p", "]"}]}], "/;", 
    RowBox[{"EvenQ", "[", "q", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.595870942451781*^9, 3.595870952033329*^9}},
 CellLabel->"In[7]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"n", "=", "0"}], "1000"], 
   RowBox[{"256", " ", 
    RowBox[{"b", "[", "n", "]"}]}]}],
  ColourMaths`Sum\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.595870954538472*^9},
 CellLabel->"Out[7]="]
}, Open  ]],

Cell[TextData[{
 "Somewhat curiously,  ",
 StyleBox["Sum", "ObjectText"],
 " and ",
 StyleBox["Product", "ObjectText"],
 " have attribute ",
 StyleBox["HoldAll", "ObjectText"],
 ", whereas Integrate and ",
 StyleBox["Limit", "ObjectText"],
 " do not. None of the passive forms have been defined to hold their \
arguments - it is assumed that their dummy arguments do not have definitions \
outside of the construct. Since we often want to perform extensive \
manipulations of summations it is convenient to allow evaluation to take \
place."
}], "Text"],

Cell[TextData[{
 "In general, nested sums, and integrals can have the order of operations \
reversed subject to certain conditions regarding uniform convergence. A \
function ",
 StyleBox["SwapOperations", "ObjectText"],
 " is supplied to achieve this on the inert forms:"
}], "Text",
 CellChangeTimes->{{3.5958710582104015`*^9, 3.595871062249633*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"n", "=", "0"}], "1"], 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       RowBox[{
        SubscriptBox["f", "n"], "[", "x", "]"}], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     ColourMaths`Integrate\[Breve]Inert,
     DefaultBaseStyle->{
      Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
     SelectWithContents->True]}],
   ColourMaths`Sum\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "//", "SwapOperations"}]], "Input",
 CellChangeTimes->{{3.5958711162787232`*^9, 3.5958711989494514`*^9}},
 CellLabel->"In[8]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "1"], 
   RowBox[{
    RowBox[{"(", 
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"n", "=", "0"}], "1"], 
       RowBox[{
        SubscriptBox["f", "n"], "[", "x", "]"}]}],
      ColourMaths`Sum\[Breve]Inert,
      DefaultBaseStyle->{
       Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
      SelectWithContents->True], ")"}], 
    RowBox[{"\[DifferentialD]", "x"}]}]}],
  ColourMaths`Integrate\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.595871205705838*^9},
 CellLabel->"Out[8]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{"(", 
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"n", "=", "0"}], "1"], 
        RowBox[{
         SubscriptBox["f", "n"], "[", "x", "]"}]}],
       ColourMaths`Sum\[Breve]Inert,
       DefaultBaseStyle->{
        Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
       SelectWithContents->True], ")"}], 
     RowBox[{"\[DifferentialD]", "x"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "//", "SwapOperations"}]], "Input",
 CellChangeTimes->{3.595871227909108*^9},
 CellLabel->"In[9]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"n", "=", "0"}], "1"], 
   TagBox[
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      RowBox[{
       SubscriptBox["f", "n"], "[", "x", "]"}], 
      RowBox[{"\[DifferentialD]", "x"}]}]}],
    ColourMaths`Integrate\[Breve]Inert,
    DefaultBaseStyle->{
     Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
    SelectWithContents->True]}],
  ColourMaths`Sum\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.5958712317593284`*^9},
 CellLabel->"Out[9]="]
}, Open  ]],

Cell["\<\
Obviously this is a purely mechanical process, it could not even begin to \
check for uniform convergence, since in general it would not 'know' the \
meaning of many of the functions in use. The user must add this to his \
derivation if required.\
\>", "Text"],

Cell["\<\
Two common hand manipulations of integrals is to change the variable of \
integration or to perform an integration by parts.  Here we see a variable \
substitution with a subsequent simplification of the integrand:\
\>", "Text",
 CellChangeTimes->{3.595947927050331*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "t"}], ")"}], 
       RowBox[{
        RowBox[{"-", "1"}], "+", "q"}]], " ", 
      SuperscriptBox["t", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "p"}]]}], 
     RowBox[{"\[DifferentialD]", "t"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], " ", "//", 
  RowBox[{"Iv", "[", 
   RowBox[{
    RowBox[{"s", "->", 
     RowBox[{"t", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}]}]}], ",", "t"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.595947592171177*^9, 3.5959476200527716`*^9}, {
  3.5959476786391225`*^9, 3.59594771793537*^9}, {3.595947886567015*^9, 
  3.5959478872380533`*^9}},
 CellLabel->"In[5]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["s", 
         RowBox[{"1", "+", "s"}]], ")"}], 
       RowBox[{
        RowBox[{"-", "1"}], "+", "p"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", 
         FractionBox["s", 
          RowBox[{"1", "+", "s"}]]}], ")"}], 
       RowBox[{
        RowBox[{"-", "1"}], "+", "q"}]]}], 
     RowBox[{
      FractionBox["s", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "+", "s"}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", 
           FractionBox["s", 
            RowBox[{"1", "+", "s"}]]}], ")"}], "2"]}]], "+", 
      FractionBox["1", 
       RowBox[{"1", "-", 
        FractionBox["s", 
         RowBox[{"1", "+", "s"}]]}]]}]], 
    RowBox[{"\[DifferentialD]", "s"}]}]}],
  ColourMaths`Integrate\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.595947935278801*^9},
 CellLabel->"Out[5]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         FractionBox["s", 
          RowBox[{"1", "+", "s"}]], ")"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "p"}]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "-", 
          FractionBox["s", 
           RowBox[{"1", "+", "s"}]]}], ")"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "q"}]]}], 
      RowBox[{
       FractionBox["s", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", "s"}], ")"}], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", 
            FractionBox["s", 
             RowBox[{"1", "+", "s"}]]}], ")"}], "2"]}]], "+", 
       FractionBox["1", 
        RowBox[{"1", "-", 
         FractionBox["s", 
          RowBox[{"1", "+", "s"}]]}]]}]], 
     RowBox[{"\[DifferentialD]", "s"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], " ", "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.595947853842143*^9, 3.595947857313342*^9}},
 CellLabel->"In[4]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "+", "s"}]], ")"}], "q"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["s", 
         RowBox[{"1", "+", "s"}]], ")"}], "p"]}], "s"], 
    RowBox[{"\[DifferentialD]", "s"}]}]}],
  ColourMaths`Integrate\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.5959478595914726`*^9},
 CellLabel->"Out[4]="]
}, Open  ]],

Cell["\<\
The integration by parts operation assumes that the part to be integrated is \
picked out in red:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "a", "b"], 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", "n"], " ", 
      TagBox[
       RowBox[{"Sin", "[", "x", "]"}],
       ColourMaths`Red\[Breve]Colour,
       DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
       SelectWithContents->True]}], 
     RowBox[{"\[DifferentialD]", "x"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], " ", "//", "ByParts"}]], "Input",
 CellChangeTimes->{{3.5959479934501286`*^9, 3.595948044916072*^9}},
 CellLabel->"In[6]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["a", "n"], " ", 
   RowBox[{"Cos", "[", "a", "]"}]}], "-", 
  RowBox[{
   SuperscriptBox["b", "n"], " ", 
   RowBox[{"Cos", "[", "b", "]"}]}], "+", 
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "a", "b"], 
    RowBox[{
     RowBox[{"n", " ", 
      SuperscriptBox["x", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "n"}]], " ", 
      RowBox[{"Cos", "[", "x", "]"}]}], 
     RowBox[{"\[DifferentialD]", "x"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True]}]], "Output",
 CellChangeTimes->{3.595948045977133*^9},
 CellLabel->"Out[6]="]
}, Open  ]],

Cell[TextData[{
 "One problem with inert mathematical operations such as ",
 StyleBox["Integral", "ObjectText"],
 " is that it is impossible to define differentiation with respect to a \
parameter in a consistent way. Here is a simple definition assuming the \
limits are constants:"
}], "Text"],

Cell[BoxData[
 RowBox[{"Integrate\[Breve]Inert", "  ", "/:", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"Integrate\[Breve]Inert", "[", 
     RowBox[{"F_", ",", " ", 
      RowBox[{"{", 
       RowBox[{"x_", ",", " ", "a_", ",", " ", "b_"}], "}"}]}], "]"}], ",", 
    " ", "k_"}], "]"}], ":=", 
  RowBox[{"Integrate\[Breve]Inert", "[", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{"F", ",", " ", "k"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"x", ",", " ", "a", ",", " ", "b"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5960297900280085`*^9, 3.596029808302054*^9}},
 CellLabel->"In[2]:="],

Cell["\<\
Using this definition, we seem to be able to differentiate inert integrals:\
\>", "Text",
 CellChangeTimes->{{3.5960304893390074`*^9, 3.5960305130503635`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   TagBox[
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", 
      FractionBox["\[Pi]", "2"]], 
     RowBox[{
      SqrtBox[
       RowBox[{"1", "-", 
        RowBox[{
         SuperscriptBox["k", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]], 
      RowBox[{"\[DifferentialD]", "x"}]}]}],
    ColourMaths`Integrate\[Breve]Inert,
    DefaultBaseStyle->{
     Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
    SelectWithContents->True], ",", "k"}], "]"}]], "Input",
 CellChangeTimes->{{3.596029672331277*^9, 3.596029720150012*^9}, 
   3.5960297823095675`*^9},
 CellLabel->"In[3]:="],

Cell[BoxData[
 RowBox[{"-", 
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     FractionBox["\[Pi]", "2"]], 
    RowBox[{
     FractionBox[
      RowBox[{"k", " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "x", "]"}], "2"]}], 
      SqrtBox[
       RowBox[{"1", "-", 
        RowBox[{
         SuperscriptBox["k", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]]], 
     RowBox[{"\[DifferentialD]", "x"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True]}]], "Output",
 CellChangeTimes->{3.5960303915604143`*^9},
 CellLabel->"Out[3]="]
}, Open  ]],

Cell["\<\
However, if the inert integral is embedded inside a larger expression, things \
go wrong. It would seem that D uses the chain rule a little too eagerly.\
\>", "Text",
 CellChangeTimes->{{3.5960305192267165`*^9, 3.5960305972171774`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"k", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", 
       FractionBox["\[Pi]", "2"]], 
      RowBox[{
       SqrtBox[
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox["k", "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     ColourMaths`Integrate\[Breve]Inert,
     DefaultBaseStyle->{
      Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
     SelectWithContents->True]}], ",", "k"}], "]"}]], "Input",
 CellChangeTimes->{3.5960298589319496`*^9},
 CellLabel->"In[4]:="],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     FractionBox["\[Pi]", "2"]], 
    RowBox[{
     SqrtBox[
      RowBox[{"1", "-", 
       RowBox[{
        SuperscriptBox["k", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]], 
     RowBox[{"\[DifferentialD]", "x"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "-", 
  FractionBox[
   RowBox[{
    SuperscriptBox["k", "2"], " ", "\[Pi]", " ", 
    SuperscriptBox[
     RowBox[{"Sin", "[", "x", "]"}], "2"]}], 
   RowBox[{"2", " ", 
    SqrtBox[
     RowBox[{"1", "-", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]]}]]}]], "Output",
 CellChangeTimes->{3.596030412711624*^9},
 CellLabel->"Out[4]="]
}, Open  ]],

Cell["\<\
For this reason, the ColourMaths package supplies the Diff operation which \
fixes this problem.\
\>", "Text",
 CellChangeTimes->{{3.596030619049426*^9, 3.596030654889476*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Diff", "[", 
  RowBox[{
   RowBox[{"k", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", 
       FractionBox["\[Pi]", "2"]], 
      RowBox[{
       SqrtBox[
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox["k", "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     ColourMaths`Integrate\[Breve]Inert,
     DefaultBaseStyle->{
      Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
     SelectWithContents->True]}], ",", "k"}], "]"}]], "Input",
 CellChangeTimes->{{3.5960299368264055`*^9, 3.5960299374664416`*^9}},
 CellLabel->"In[5]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "k"}], " ", 
   TagBox[
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", 
      FractionBox["\[Pi]", "2"]], 
     RowBox[{
      FractionBox[
       RowBox[{"k", " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "x", "]"}], "2"]}], 
       SqrtBox[
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox["k", "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]]], 
      RowBox[{"\[DifferentialD]", "x"}]}]}],
    ColourMaths`Integrate\[Breve]Inert,
    DefaultBaseStyle->{
     Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
    SelectWithContents->True]}], "+", 
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     FractionBox["\[Pi]", "2"]], 
    RowBox[{
     SqrtBox[
      RowBox[{"1", "-", 
       RowBox[{
        SuperscriptBox["k", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "x", "]"}], "2"]}]}]], 
     RowBox[{"\[DifferentialD]", "x"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True]}]], "Output",
 CellChangeTimes->{3.596030451184825*^9},
 CellLabel->"Out[5]="]
}, Open  ]],

Cell[TextData[{
 "The ColourMaths  package also contains inert forms for several common \
Special functions and related mathematical objects. In most cases these also \
have rules to display them in a traditional form from StandardForm. These \
functions do not display well in TraditionalForm, but by switching them to \
the active ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " equivalent it is possible to obtain an expression suitable for display in \
TraditionalForm. "
}], "Text",
 CellChangeTimes->{{3.5959624826758647`*^9, 3.5959625078993073`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
The Adjust button \[Dash] Visual algebra\
\>", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True,
 CellChangeTimes->{{3.5960338853652487`*^9, 3.5960338860582886`*^9}, {
  3.5960339294507704`*^9, 3.5960339418994823`*^9}}],

Cell["\<\
Most of us are familiar with doing algebra in a linear fashion. The 'point \
and click' culture has  yet to make much impact on this process. However,  \
The Adjust button offers a step in that direction! Simply select the part of \
an expression to be manipulated, and press the Adjust button to see a menu of \
possible manipulations:\
\>", "Text",
 CellChangeTimes->{{3.5960339625876656`*^9, 3.596034076673191*^9}, {
  3.5960344558788805`*^9, 3.5960344565989213`*^9}}],

Cell[BoxData[
 RowBox[{"f", "[", 
  StyleBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", "^", "2"}], "+", 
      RowBox[{"2", " ", "a", " ", "b"}], "+", 
      RowBox[{"b", "^", "2"}]}], ")"}]}],
   Background->GrayLevel[0.85]], "]"}]], "Input",
 CellChangeTimes->{{3.596034108282999*^9, 3.5960341287071667`*^9}, {
  3.596034175123822*^9, 3.596034178908038*^9}}],

Cell["\<\
Note that I have used the shading to represent the selection \[Dash] it isn't \
part of the expression. Here is what the system offers at this point:\
\>", "Text",
 CellChangeTimes->{{3.5960344687826185`*^9, 3.5960346235014677`*^9}}],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f1zHdVhxnG1nf4Xncww03SYtukAnWg6DmmnoTNJfohn2ppSBhNcmgjo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    "], {{0, 261}, {620, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{620, 261},
  PlotRange->{{0, 620}, {0, 261}}]], "Input"],

Cell["\<\
Accepting one or other of the two choices will not simply alter the \
expression (which would leave no indication of how the transformation was \
produced). Instead, it annotates the expression, so that the desired result \
is obtained by evaluation:\
\>", "Text",
 CellChangeTimes->{{3.596034638141305*^9, 3.596034722829149*^9}, {
  3.596034789972989*^9, 3.5960348120842543`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   TagBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "+", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["a", "2"], "+", 
       RowBox[{"2", " ", "a", " ", "b"}], "+", 
       SuperscriptBox["b", "2"]}], ")"}]}],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
    SelectWithContents->True], "]"}], "//", 
  RowBox[{
  "Red\[Breve]ColourAdjust", "[", "Apart\[Breve]Adjust", "]"}]}]], "Input",
 CellChangeTimes->{{3.5960347606363115`*^9, 3.596034761050335*^9}},
 CellLabel->"In[10]:="],

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{
   SuperscriptBox["a", "3"], "+", 
   RowBox[{"3", " ", 
    SuperscriptBox["a", "2"], " ", "b"}], "+", 
   RowBox[{"3", " ", "a", " ", 
    SuperscriptBox["b", "2"]}], "+", 
   SuperscriptBox["b", "3"]}], "]"}]], "Output",
 CellChangeTimes->{3.596034772571994*^9},
 CellLabel->"Out[10]="]
}, Open  ]],

Cell["\<\
This illustrates the concept, but to obtain really interesting results, we \
need to supply some transformations that are specific to our area of \
interest. For example:\
\>", "Text",
 CellChangeTimes->{{3.5960348685314827`*^9, 3.5960349366913815`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Define\[Breve]Adjustment", "[", 
   RowBox[{"Binomial\[Breve]Theorem", ",", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", "x_"}], ")"}], "p_"], "\[RuleDelayed]", 
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"k", "=", "0"}], "\[Infinity]"], 
       RowBox[{
        SuperscriptBox["x", "k"], " ", 
        RowBox[{"Binomial", "[", 
         RowBox[{"p", ",", 
          TagBox["k",
           ColourMaths`Flag\[Breve]Colour,
           DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 1]}]}], "]"}]}]}],
      ColourMaths`Sum\[Breve]Inert,
      DefaultBaseStyle->{
       Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
      SelectWithContents->True]}], ",", "\"\<Binomial theorem\>\""}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.593888053871479*^9, 3.593888067977286*^9}, {
   3.5938883019906707`*^9, 3.5938883241439376`*^9}, {3.5938883547716894`*^9, 
   3.593888473690491*^9}, {3.593888710431032*^9, 3.593888723295768*^9}, {
   3.593935015037504*^9, 3.5939350374267845`*^9}, 3.5939351513473005`*^9, 
   3.5939731127175665`*^9, {3.594233111986506*^9, 3.5942331194679337`*^9}, {
   3.594841279239688*^9, 3.5948412917584047`*^9}},
 CellLabel->"In[11]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Define\[Breve]Adjustment", "[", 
   RowBox[{"Beta\[Breve]Function\[Breve]Definition", ",", 
    RowBox[{
     RowBox[{"Integrate\[Breve]Inert", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"t_", "^", 
         RowBox[{"(", 
          RowBox[{"p_", " ", "-", " ", "1"}], ")"}]}], "*", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "-", " ", "t_"}], ")"}], "^", 
         RowBox[{"(", 
          RowBox[{"q_", " ", "-", " ", "1"}], ")"}]}]}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"t_", ",", " ", "0", ",", " ", "1"}], "}"}]}], "]"}], 
     "\[RuleDelayed]", 
     RowBox[{"\[CapitalBeta]", "[", 
      RowBox[{"p", ",", "q"}], "]"}]}], ",", 
    "\"\<Beta function definition\>\""}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5942300272110667`*^9, 3.594230168762163*^9}, 
   3.5942302047392206`*^9, 3.5942303176816807`*^9, {3.5942303727428303`*^9, 
   3.594230403141569*^9}, {3.5942304446509433`*^9, 3.5942304533874426`*^9}, {
   3.594230632209671*^9, 3.594230640658154*^9}},
 CellLabel->"In[21]:="],

Cell["\<\
The Define\[Breve]Adjustment function takes an arbitrary transformation name \
(analogous to Apart\[Breve]Adjust above), which must be unique, a \
transformation rule, and any additional comments. The last argument can be \
omitted.\
\>", "Text",
 CellChangeTimes->{{3.5960425623015413`*^9, 3.5960426790922213`*^9}}],

Cell["\<\
Here is a typical use of these definitions. Select the shaded part of the \
expression, and press the Adjust button. You should see the following::\
\>", "Text",
 CellChangeTimes->{{3.596042696164198*^9, 3.5960427159883313`*^9}, {
   3.596042932723728*^9, 3.5960429863227935`*^9}, 3.596119087517638*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "1"], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
    SuperscriptBox["t", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "b"}]], 
    RowBox[{"\[DifferentialD]", "t"}]}]}],
  ColourMaths`Integrate\[Breve]Inert,
  DefaultBaseStyle->{Background -> RGBColor[0.8, 0.8, 1]}]], "Input",
 CellChangeTimes->{{3.5960428505490274`*^9, 3.596042869140091*^9}}],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnQmY3cSBoJ85c06Sye5kMpMlQ2azYeIkH5CNZ7fJJJMh8yXZD/xld2EY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    "], {{0, 306}, {621, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{621, 306},
  PlotRange->{{0, 621}, {0, 306}}]], "Input"],

Cell["\<\
This produces the following. As you can see, it is quite clear what step was \
chosen, and it could be re-executed at a later date provided the above call \
to Define\[Breve]Adjustment had been entered.\
\>", "Text",
 CellChangeTimes->{{3.596043308760236*^9, 3.5960434057357826`*^9}}],

Cell["\<\
However, there is a complication here. The summation in the binomial \
expansion requires a dummy index \[Dash] a problem which is very common in \
really useful transformations. It is for this reason that the dummy index, k, \
was coloured with the 'flag colour' when it was entered. The idea is that you \
fill in a suitable index to use thus:\
\>", "Text",
 CellChangeTimes->{{3.596043418046487*^9, 3.596043601053954*^9}}],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{
       RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
     SuperscriptBox["t", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "a"}]], " ", 
     TagBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"t", " ", "z"}]}], ")"}], 
       RowBox[{"-", "b"}]],
      ColourMaths`Red\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
      SelectWithContents->True], 
     RowBox[{"\[DifferentialD]", "t"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{Background -> RGBColor[0.8, 0.8, 1]}], "//", 
  RowBox[{"Red\[Breve]ColourAdjust", "[", 
   RowBox[{"Binomial\[Breve]Theorem", ",", 
    RowBox[{
     TagBox["k",
      ColourMaths`Flag\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 1]},
      SelectWithContents->True], "\[Rule]", "\[SelectionPlaceholder]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5960428505490274`*^9, 3.596042869140091*^9}, {
  3.59604325096093*^9, 3.5960432511989436`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{
       RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
     SuperscriptBox["t", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "a"}]], " ", 
     TagBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"t", " ", "z"}]}], ")"}], 
       RowBox[{"-", "b"}]],
      ColourMaths`Red\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
      SelectWithContents->True], 
     RowBox[{"\[DifferentialD]", "t"}]}]}],
   ColourMaths`Integrate\[Breve]Inert,
   DefaultBaseStyle->{Background -> RGBColor[0.8, 0.8, 1]}], "//", 
  RowBox[{"Red\[Breve]ColourAdjust", "[", 
   RowBox[{"Binomial\[Breve]Theorem", ",", 
    RowBox[{
     TagBox["k",
      ColourMaths`Flag\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 1]},
      SelectWithContents->True], "\[Rule]", "m"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5960428505490274`*^9, 3.596042869140091*^9}, {
   3.59604325096093*^9, 3.5960432511989436`*^9}, 3.5960436153147697`*^9},
 CellLabel->"In[13]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "1"], 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{
       RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
     SuperscriptBox["t", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "a"}]], " ", 
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"m", "=", "0"}], "\[Infinity]"], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "t"}], " ", "z"}], ")"}], "m"], " ", 
        RowBox[{"Binomial", "[", 
         RowBox[{
          RowBox[{"-", "b"}], ",", "m"}], "]"}]}]}],
      ColourMaths`Sum\[Breve]Inert,
      DefaultBaseStyle->{
       Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
      SelectWithContents->True]}], 
    RowBox[{"\[DifferentialD]", "t"}]}]}],
  ColourMaths`Integrate\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.59604361811593*^9},
 CellLabel->"Out[13]="]
}, Open  ]],

Cell["\<\
Now we swap the integral and sum (deferring any convergence checks until \
later), and move things about.\
\>", "Text",
 CellChangeTimes->{{3.5960437208538065`*^9, 3.5960437492444305`*^9}, {
   3.5960438823650446`*^9, 3.5960438919565926`*^9}, 3.5961190997833395`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   TagBox[
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "-", "t"}], ")"}], 
        RowBox[{
         RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
       SuperscriptBox["t", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "a"}]], " ", 
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"m", "=", "0"}], "\[Infinity]"], 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "t"}], " ", "z"}], ")"}], "m"], " ", 
          TagBox[
           RowBox[{"Binomial", "[", 
            RowBox[{
             RowBox[{"-", "b"}], ",", "m"}], "]"}],
           ColourMaths`Red\[Breve]Colour,
           DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
           SelectWithContents->True]}]}],
        ColourMaths`Sum\[Breve]Inert,
        DefaultBaseStyle->{
         Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
        SelectWithContents->True]}], 
      RowBox[{"\[DifferentialD]", "t"}]}]}],
    ColourMaths`Integrate\[Breve]Inert,
    DefaultBaseStyle->{
     Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
    SelectWithContents->True], "//", "SwapOperations"}], "//", 
  "Rout1"}]], "Input",
 CellChangeTimes->{{3.596043762916212*^9, 3.5960437672424593`*^9}, {
  3.5960438603747864`*^9, 3.596043868318241*^9}},
 CellLabel->"In[16]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"m", "=", "0"}], "\[Infinity]"], 
   RowBox[{
    RowBox[{"Binomial", "[", 
     RowBox[{
      RowBox[{"-", "b"}], ",", "m"}], "]"}], " ", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", "t"}], ")"}], 
         RowBox[{
          RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
        SuperscriptBox["t", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "a"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "t"}], " ", "z"}], ")"}], "m"]}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}],
     ColourMaths`Integrate\[Breve]Inert,
     DefaultBaseStyle->{
      Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
     SelectWithContents->True]}]}],
  ColourMaths`Sum\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.5960439203762183`*^9},
 CellLabel->"Out[16]="]
}, Open  ]],

Cell["\<\
Now a few simple rearrangements leave the expression with a recognisable beta \
function integral\
\>", "Text",
 CellChangeTimes->{{3.596112609784133*^9, 3.596112648087324*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"m", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     RowBox[{"Binomial", "[", 
      RowBox[{
       RowBox[{"-", "b"}], ",", "m"}], "]"}], " ", 
     TagBox[
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", "1"], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", "t"}], ")"}], 
         RowBox[{
          RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
        SuperscriptBox["t", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "a"}]], " ", 
        TagBox[
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "t"}], " ", "z"}], ")"}], "m"],
         ColourMaths`Green\[Breve]Colour,
         DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
         SelectWithContents->True], 
        RowBox[{"\[DifferentialD]", "t"}]}]}],
      ColourMaths`Integrate\[Breve]Inert,
      DefaultBaseStyle->{
       Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
      SelectWithContents->True]}]}],
   ColourMaths`Sum\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "//", 
  RowBox[{"Gmpy", "[", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "z"}], ")"}], 
    RowBox[{"-", "m"}]], "]"}]}]], "Input",
 CellChangeTimes->{{3.596043906710437*^9, 3.596043917467052*^9}, {
  3.596112406274493*^9, 3.5961124270386806`*^9}},
 CellLabel->"In[8]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"m", "=", "0"}], "\[Infinity]"], 
   RowBox[{
    RowBox[{"Binomial", "[", 
     RowBox[{
      RowBox[{"-", "b"}], ",", "m"}], "]"}], " ", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", "t"}], ")"}], 
         RowBox[{
          RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
        SuperscriptBox["t", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "a"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "z"}], ")"}], "m"], " ", 
        RowBox[{"(", 
         TagBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "z"}], ")"}], 
            RowBox[{"-", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "t"}], " ", "z"}], ")"}], "m"]}],
          ColourMaths`Green\[Breve]Colour,
          DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
          SelectWithContents->True], ")"}]}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}],
     ColourMaths`Integrate\[Breve]Inert,
     DefaultBaseStyle->{
      Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
     SelectWithContents->True]}]}],
  ColourMaths`Sum\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.5960439753443623`*^9, 3.596112430477877*^9},
 CellLabel->"Out[8]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   TagBox[
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"m", "=", "0"}], "\[Infinity]"], 
     RowBox[{
      RowBox[{"Binomial", "[", 
       RowBox[{
        RowBox[{"-", "b"}], ",", "m"}], "]"}], " ", 
      TagBox[
       RowBox[{
        SubsuperscriptBox["\[Integral]", "0", "1"], 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"1", "-", "t"}], ")"}], 
           RowBox[{
            RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
          SuperscriptBox["t", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "a"}]], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "z"}], ")"}], "m"], " ", 
          RowBox[{"(", 
           TagBox[
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "z"}], ")"}], 
              RowBox[{"-", "m"}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "t"}], " ", "z"}], ")"}], "m"]}],
            ColourMaths`Green\[Breve]Colour,
            DefaultBaseStyle->{FontColor -> RGBColor[0, 1, 0]},
            SelectWithContents->True], ")"}]}], 
         RowBox[{"\[DifferentialD]", "t"}]}]}],
       ColourMaths`Integrate\[Breve]Inert,
       DefaultBaseStyle->{
        Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
       SelectWithContents->True]}]}],
    ColourMaths`Sum\[Breve]Inert,
    DefaultBaseStyle->{
     Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
    SelectWithContents->True], "//", "Rout1"}], "//", 
  RowBox[{"GG1", "[", "PowerExpand", "]"}]}]], "Input",
 CellChangeTimes->{{3.596043972390194*^9, 3.5960439993517356`*^9}, {
   3.5961121489877768`*^9, 3.596112153674045*^9}, 3.5961124433056107`*^9},
 CellLabel->"In[9]:="],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"m", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     RowBox[{"Binomial", "[", 
      RowBox[{
       RowBox[{"-", "b"}], ",", "m"}], "]"}], " ", 
     TagBox[
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", "1"], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"1", "-", "t"}], ")"}], 
         RowBox[{
          RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
        SuperscriptBox["t", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "a", "+", "m"}]], " ", 
        TagBox[
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "z"}], ")"}], "m"],
         ColourMaths`Red\[Breve]Colour,
         DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
         SelectWithContents->True], 
        RowBox[{"\[DifferentialD]", "t"}]}]}],
      ColourMaths`Integrate\[Breve]Inert,
      DefaultBaseStyle->{
       Background -> RGBColor[0.6919200000000001, 0.6919200000000001, 1]},
      SelectWithContents->True]}]}],
   ColourMaths`Sum\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "//", "Rout1"}]], "Input",
 CellChangeTimes->{{3.596112533924794*^9, 3.596112539109091*^9}},
 CellLabel->"In[10]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"m", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "z"}], ")"}], "m"], " ", 
     RowBox[{"Binomial", "[", 
      RowBox[{
       RowBox[{"-", "b"}], ",", "m"}], "]"}], " ", 
     TagBox[
      TagBox[
       RowBox[{
        SubsuperscriptBox["\[Integral]", "0", "1"], 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"1", "-", "t"}], ")"}], 
           RowBox[{
            RowBox[{"-", "1"}], "-", "a", "+", "c"}]], " ", 
          SuperscriptBox["t", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "a", "+", "m"}]]}], 
         RowBox[{"\[DifferentialD]", "t"}]}]}],
       ColourMaths`Integrate\[Breve]Inert,
       DefaultBaseStyle->{
        Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
       SelectWithContents->True],
      ColourMaths`Red\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]},
      SelectWithContents->True]}]}],
   ColourMaths`Sum\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "//", 
  RowBox[{
  "Red\[Breve]ColourAdjust", "[", "Beta\[Breve]Function\[Breve]Definition", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5961125540229435`*^9, 3.5961125544909706`*^9}},
 CellLabel->"In[11]:="],

Cell[BoxData[
 TagBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"m", "=", "0"}], "\[Infinity]"], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "m"], " ", 
    RowBox[{"Binomial", "[", 
     RowBox[{
      RowBox[{"-", "b"}], ",", "m"}], "]"}], " ", 
    RowBox[{"\[CapitalBeta]", "[", 
     RowBox[{
      RowBox[{"a", "+", "m"}], ",", 
      RowBox[{
       RowBox[{"-", "a"}], "+", "c"}]}], "]"}]}]}],
  ColourMaths`Sum\[Breve]Inert,
  DefaultBaseStyle->{
   Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
  SelectWithContents->True]], "Output",
 CellChangeTimes->{3.5961125570051146`*^9},
 CellLabel->"Out[11]="]
}, Open  ]],

Cell["", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Highlight Formulae", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True,
 CellChangeTimes->{{3.601316171774992*^9, 3.6013161824466023`*^9}}],

Cell[TextData[{
 "Most serious mathematical derivations number certain formulae so that these \
can be referred to at subsequent points in the text. While ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does permit equation numbering, it supplies no way to actually access an \
equation by number. Although it would probably be possible to create such a \
mechanism, another problem associated with relying on equation numbering, is \
that inserting an extra numbered equation into a notebook will cause the \
other equation numbers to change."
}], "Text",
 CellChangeTimes->{{3.600799187666266*^9, 3.6007992542460737`*^9}, {
  3.6007992991816444`*^9, 3.6007995010361896`*^9}, {3.6008834229324102`*^9, 
  3.600883425319547*^9}}],

Cell["\<\
The ColourMaths package lets you highlight any cell and give it a name to \
identify it further down the notebook. This is done by clicking anywhere in \
the cell, and pressing the \[OpenCurlyDoubleQuote]Highlight\
\[CloseCurlyDoubleQuote] button on the palette. For example:\
\>", "Text",
 CellChangeTimes->{{3.60079951205182*^9, 3.600799588124171*^9}, {
  3.6008834563603225`*^9, 3.60088349687964*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{
   StyleBox["\"\<Atomic_hydrogen_ground_state\>\"",
    FontSize->17], "\n", 
   TagBox[
    RowBox[{"E\[Breve]Hydrogen", "\[Equal]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[DoubleStruckE]", "4"], " ", 
       SubscriptBox["m", "e"], " ", 
       SubscriptBox["m", "p"]}], 
      RowBox[{"32", " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox["\[HBar]", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["m", "e"], "+", 
         SubscriptBox["m", "p"]}], ")"}], " ", 
       SubsuperscriptBox["\[Epsilon]", "0", "2"]}]]}],
    ColourMaths`Private`HR2]}],
  ColourMaths`Private`HR1["Atomic_hydrogen_ground_state"]]], "Input",
 CellFrame->True,
 CellChangeTimes->{
  3.6007995971956897`*^9, {3.600800536208398*^9, 3.6008005771627407`*^9}, {
   3.600800619274149*^9, 3.600800624849468*^9}, 3.6008006933593864`*^9},
 Background->RGBColor[0.88, 1, 0.88]],

Cell["\<\
Once this highlight formula has been named in this way, it can be refereed to \
thus:\
\>", "Text",
 CellChangeTimes->{{3.6008007906749525`*^9, 3.6008008446910424`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Get\[Breve]Highlight\[Breve]Formula", "[", 
  "\"\<Atomic_hydrogen_ground_state\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.600800830341222*^9, 3.6008008554216557`*^9}}],

Cell[BoxData[
 RowBox[{"E\[Breve]Hydrogen", "\[Equal]", 
  FractionBox[
   RowBox[{
    SuperscriptBox["\[DoubleStruckE]", "4"], " ", 
    SubscriptBox["m", "e"], " ", 
    SubscriptBox["m", "p"]}], 
   RowBox[{"32", " ", 
    SuperscriptBox["\[Pi]", "2"], " ", 
    SuperscriptBox["\[HBar]", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["m", "e"], "+", 
      SubscriptBox["m", "p"]}], ")"}], " ", 
    SubsuperscriptBox["\[Epsilon]", "0", "2"]}]]}]], "Output",
 CellChangeTimes->{3.6008008751347837`*^9}]
}, Open  ]],

Cell["\<\
Using the following extra definitions (which might typically be included in \
the initialisation section of a notebook), we can access the formula in a \
more interesting way:\
\>", "Text",
 CellChangeTimes->{{3.600800886866455*^9, 3.6008009851370754`*^9}, {
  3.600883537127942*^9, 3.600883538263007*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"physicalConstants", "=", 
   RowBox[{"{", "\[IndentingNewLine]", "   ", 
    RowBox[{
     RowBox[{
      SubscriptBox["m", "e"], "->", "9.10938215`*^-31"}], ",", "\n", 
     "         ", 
     RowBox[{
      SubscriptBox["m", "p"], "->", "1.672621637`*^-27"}], ",", 
     "\[IndentingNewLine]", "   ", 
     RowBox[{
      SubscriptBox["\[Epsilon]", "0"], "->", "8.85419*^-12"}], ",", 
     "\[IndentingNewLine]", "   ", 
     RowBox[{"\[DoubleStruckE]", "->", "1.6021766*^-19"}], ",", 
     "\[IndentingNewLine]", "   ", 
     RowBox[{"\[HBar]", "->", "1.0545717*^-34"}]}], "}"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
  "Get\[Breve]Highlight\[Breve]Formula", "[", 
   "\"\<Atomic_hydrogen_ground_state\>\"", "]"}], "/.", 
  "physicalConstants"}]], "Input",
 CellChangeTimes->{{3.600801012772656*^9, 3.600801023448267*^9}}],

Cell[BoxData[
 RowBox[{"E\[Breve]Hydrogen", "\[Equal]", 
  "2.1786846654235426`*^-18"}]], "Output",
 CellChangeTimes->{
  3.6008010240012984`*^9, 3.6008040387917347`*^9, {3.600804084487348*^9, 
   3.6008040916187563`*^9}, 3.6008041667000504`*^9}]
}, Open  ]],

Cell["\<\
If you prefer to number your equations, you can always give them a numerical \
name, e.g. \[OpenCurlyDoubleQuote]23\[CloseCurlyDoubleQuote] or \
\[OpenCurlyDoubleQuote](23)\[CloseCurlyDoubleQuote] or \
\[OpenCurlyDoubleQuote]2.1.7\[CloseCurlyDoubleQuote] .\
\>", "Text",
 CellChangeTimes->{{3.601299269974264*^9, 3.6012993453975782`*^9}, {
  3.6012993780054436`*^9, 3.60129937829346*^9}, {3.6013163401846247`*^9, 
  3.601316340503643*^9}}],

Cell["\<\
Highlight formulae can also be accessed from a menu using \
Pick\[Breve]Highlight\[Breve]Formula:\
\>", "Text",
 CellChangeTimes->{{3.6013163470220156`*^9, 3.6013164078864965`*^9}, 
   3.601366891518342*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pick\[Breve]Highlight\[Breve]Formula", "[", "]"}]], "Input",
 CellChangeTimes->{{3.6013164117757196`*^9, 3.601316411799721*^9}},
 CellLabel->"In[4]:="],

Cell[BoxData[
 RowBox[{"E\[Breve]Hydrogen", "\[Equal]", 
  FractionBox[
   RowBox[{
    SuperscriptBox["\[DoubleStruckE]", "4"], " ", 
    SubscriptBox["m", "e"], " ", 
    SubscriptBox["m", "p"]}], 
   RowBox[{"32", " ", 
    SuperscriptBox["\[Pi]", "2"], " ", 
    SuperscriptBox["\[HBar]", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["m", "e"], "+", 
      SubscriptBox["m", "p"]}], ")"}], " ", 
    SubsuperscriptBox["\[Epsilon]", "0", "2"]}]]}]], "Output",
 CellChangeTimes->{3.601316416624997*^9},
 CellLabel->"Out[4]="]
}, Open  ]],

Cell["\<\
Certain other functions, such as Left\[Breve]Side and Right\[Breve]Side can \
also take the string name of a suitable formula. For example:\
\>", "Text",
 CellChangeTimes->{{3.601316433276949*^9, 3.601316480620657*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Right\[Breve]Side", "[", "\"\<Atomic_hydrogen_ground_state\>\"", 
  "]"}]], "Input",
 CellChangeTimes->{{3.6013164877660656`*^9, 3.601316512807498*^9}},
 CellLabel->"In[5]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["\[DoubleStruckE]", "4"], " ", 
   SubscriptBox["m", "e"], " ", 
   SubscriptBox["m", "p"]}], 
  RowBox[{"32", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   SuperscriptBox["\[HBar]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["m", "e"], "+", 
     SubscriptBox["m", "p"]}], ")"}], " ", 
   SubsuperscriptBox["\[Epsilon]", "0", "2"]}]]], "Output",
 CellChangeTimes->{3.6013165138505573`*^9},
 CellLabel->"Out[5]="]
}, Open  ]],

Cell["\<\
Note that these functions access the highlight formulae from the notebook,not \
from the kernel. This means that provided the ColourMaths package is loaded, \
it is not necessary to re-execute a notebook to pick up these formulae.\
\>", "Text",
 CellChangeTimes->{{3.6013165261962643`*^9, 3.6013166214917145`*^9}, {
  3.601316687659499*^9, 3.6013166997311897`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Future enhancements", "SubsectionIcon",
 PageWidth->WindowWidth,
 ShowGroupOpener->True],

Cell[TextData[{
 "Clearly the concept of using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in this way could be expanded in many directions, with specialised sets of \
transformation rules, etc being added as required."
}], "Text",
 CellChangeTimes->{{3.5960443152978067`*^9, 3.596044395536396*^9}, 
   3.596119130155077*^9, 3.6013669227481284`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
Saveable->False,
Selectable->True,
WindowSize->{811, 741},
WindowMargins->{{Automatic, 511}, {Automatic, 46}},
WindowFrame->"Normal",
Visible->True,
DockedCells->FEPrivate`FrontEndResource["FEExpressions", "HelpViewerToolbar"],
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, 
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"]}, {
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"], Inherited, 
   Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}, {
  Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowCellBracket->True,
ShowSelection->True,
TaggingRules->{
 "SearchTextTranslated" -> "", "SearchText" -> "", "LinkTrails" -> ""},
ShowCellLabel->False,
ShowSyntaxStyles->False,
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.25, 1.25 Inherited],
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Action.Definition"->{
  Cell[11268, 314, 15553, 368, 87, "Text",
   CellTags->"Action.Definition"],
  Cell[49698, 1451, 399, 11, 87, "Text",
   CellTags->"Action.Definition"]},
 "Precedence of Operators"->{
  Cell[36271, 972, 113, 2, 87, "Text",
   CellTags->"Precedence of Operators"]},
 "Must use palette"->{
  Cell[43033, 1211, 316, 7, 87, "Text",
   CellTags->"Must use palette"]},
 "WorkingForm.Definition"->{
  Cell[51189, 1507, 404, 11, 87, "Text",
   CellTags->"WorkingForm.Definition"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Action.Definition", 159404, 4465},
 {"Precedence of Operators", 159589, 4470},
 {"Must use palette", 159693, 4473},
 {"WorkingForm.Definition", 159797, 4476}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 141, 2, 108, "Chapter"],
Cell[CellGroupData[{
Cell[733, 28, 56, 1, 78, "Section"],
Cell[792, 31, 1044, 20, 141, "Text"],
Cell[1839, 53, 16, 0, 27, "Text"],
Cell[CellGroupData[{
Cell[1880, 57, 90, 2, 46, "SubsectionIcon"],
Cell[1973, 61, 887, 14, 127, "Text"],
Cell[2863, 77, 271, 4, 27, "Text"],
Cell[3137, 83, 145, 3, 30, "Input"],
Cell[3285, 88, 1040, 20, 163, "Text"],
Cell[4328, 110, 999, 17, 163, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5364, 132, 105, 2, 87, "SubsectionIcon"],
Cell[5472, 136, 561, 9, 87, "Text"],
Cell[CellGroupData[{
Cell[6058, 149, 372, 9, 87, "Input"],
Cell[6433, 160, 361, 11, 87, "Output"]
}, Open  ]],
Cell[6809, 174, 289, 5, 87, "Text"],
Cell[CellGroupData[{
Cell[7123, 183, 497, 12, 87, "Input"],
Cell[7623, 197, 355, 9, 87, "Output"]
}, Open  ]],
Cell[7993, 209, 275, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[8293, 220, 288, 10, 87, "Input"],
Cell[8584, 232, 250, 7, 87, "Output"]
}, Open  ]],
Cell[8849, 242, 260, 6, 87, "Text"],
Cell[CellGroupData[{
Cell[9134, 252, 514, 14, 87, "Input"],
Cell[9651, 268, 556, 15, 87, "Output"]
}, Open  ]],
Cell[10222, 286, 175, 4, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10434, 295, 108, 2, 87, "SubsectionIcon"],
Cell[10545, 299, 720, 13, 87, "Text"],
Cell[11268, 314, 15553, 368, 87, "Text",
 CellTags->"Action.Definition"],
Cell[26824, 684, 241, 4, 87, "Text"],
Cell[27068, 690, 3030, 74, 87, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30135, 769, 94, 1, 87, "SubsectionIcon"],
Cell[30232, 772, 373, 6, 87, "Text"],
Cell[CellGroupData[{
Cell[30630, 782, 485, 14, 87, "Input"],
Cell[31118, 798, 749, 22, 87, "Output"]
}, Open  ]],
Cell[31882, 823, 187, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[32094, 831, 502, 15, 87, "Input"],
Cell[32599, 848, 445, 13, 87, "Output"]
}, Open  ]],
Cell[33059, 864, 154, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[33238, 871, 536, 16, 87, "Input"],
Cell[33777, 889, 551, 16, 87, "Output"]
}, Open  ]],
Cell[34343, 908, 146, 3, 87, "Text"],
Cell[34492, 913, 436, 8, 87, "Text"],
Cell[34931, 923, 331, 10, 87, "Input"],
Cell[35265, 935, 89, 2, 87, "Text"],
Cell[CellGroupData[{
Cell[35379, 941, 455, 12, 87, "Input"],
Cell[35837, 955, 276, 7, 87, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[36162, 968, 106, 2, 87, "SubsectionIcon"],
Cell[36271, 972, 113, 2, 87, "Text",
 CellTags->"Precedence of Operators"],
Cell[36387, 976, 165, 5, 87, "Input"],
Cell[36555, 983, 563, 15, 87, "Text"],
Cell[CellGroupData[{
Cell[37143, 1002, 554, 16, 87, "Input"],
Cell[37700, 1020, 433, 13, 87, "Output"]
}, Open  ]],
Cell[38148, 1036, 130, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[38303, 1043, 734, 22, 87, "Input"],
Cell[39040, 1067, 321, 10, 87, "Output"]
}, Open  ]],
Cell[39376, 1080, 129, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[39530, 1087, 690, 21, 87, "Input"],
Cell[40223, 1110, 254, 7, 87, "Output"]
}, Open  ]],
Cell[40492, 1120, 229, 4, 87, "Text"],
Cell[40724, 1126, 342, 6, 87, "Text"],
Cell[CellGroupData[{
Cell[41091, 1136, 329, 10, 87, "Input"],
Cell[41423, 1148, 466, 15, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41926, 1168, 369, 12, 87, "Input"],
Cell[42298, 1182, 444, 14, 87, "Output"]
}, Open  ]],
Cell[42757, 1199, 137, 3, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42931, 1207, 99, 2, 87, "SubsectionIcon"],
Cell[43033, 1211, 316, 7, 87, "Text",
 CellTags->"Must use palette"],
Cell[CellGroupData[{
Cell[43374, 1222, 366, 10, 87, "Input"],
Cell[43743, 1234, 216, 6, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[43996, 1245, 374, 11, 87, "Input"],
Cell[44373, 1258, 333, 10, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44743, 1273, 417, 12, 87, "Input"],
Cell[45163, 1287, 422, 13, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45622, 1305, 699, 21, 87, "Input"],
Cell[46324, 1328, 256, 8, 87, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[46629, 1342, 98, 2, 87, "SubsectionIcon"],
Cell[46730, 1346, 421, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[47176, 1357, 354, 12, 87, "Input"],
Cell[47533, 1371, 283, 8, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47853, 1384, 381, 12, 87, "Input"],
Cell[48237, 1398, 213, 7, 87, "Output"]
}, Open  ]],
Cell[48465, 1408, 169, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[48659, 1415, 685, 20, 87, "Input"],
Cell[49347, 1437, 240, 7, 87, "Output"]
}, Open  ]],
Cell[49602, 1447, 93, 2, 87, "Text"],
Cell[49698, 1451, 399, 11, 87, "Text",
 CellTags->"Action.Definition"],
Cell[CellGroupData[{
Cell[50122, 1466, 588, 18, 87, "Input"],
Cell[50713, 1486, 366, 13, 87, "Output"]
}, Open  ]],
Cell[51094, 1502, 44, 1, 87, "Caption"],
Cell[51141, 1505, 45, 0, 87, "Text"],
Cell[51189, 1507, 404, 11, 87, "Text",
 CellTags->"WorkingForm.Definition"],
Cell[CellGroupData[{
Cell[51618, 1522, 600, 17, 87, "Input"],
Cell[52221, 1541, 542, 16, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52800, 1562, 438, 14, 87, "Input"],
Cell[53241, 1578, 269, 9, 87, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[53559, 1593, 100, 2, 87, "SubsectionIcon"],
Cell[53662, 1597, 420, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[54107, 1608, 392, 11, 87, "Input"],
Cell[54502, 1621, 352, 10, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54891, 1636, 476, 16, 87, "Input"],
Cell[55370, 1654, 220, 7, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55627, 1666, 403, 12, 87, "Input"],
Cell[56033, 1680, 191, 6, 87, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[56273, 1692, 105, 2, 87, "SubsectionIcon"],
Cell[56381, 1696, 775, 15, 87, "Text"],
Cell[57159, 1713, 129, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[57313, 1720, 621, 18, 87, "Input"],
Cell[57937, 1740, 570, 18, 87, "Output"]
}, Open  ]],
Cell[58522, 1761, 407, 6, 87, "Text"],
Cell[58932, 1769, 51, 0, 87, "Text"],
Cell[CellGroupData[{
Cell[59008, 1773, 1550, 43, 87, "Input"],
Cell[60561, 1818, 523, 18, 87, "Output"]
}, Closed]],
Cell[61099, 1839, 137, 3, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61273, 1847, 117, 2, 87, "SubsectionIcon"],
Cell[61393, 1851, 1142, 17, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62572, 1873, 149, 3, 87, "SubsectionIcon"],
Cell[62724, 1878, 1183, 19, 87, "Text"],
Cell[CellGroupData[{
Cell[63932, 1901, 854, 24, 87, "Input"],
Cell[64789, 1927, 734, 20, 87, "Output"]
}, Open  ]],
Cell[65538, 1950, 141, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[65704, 1957, 829, 24, 87, "Input"],
Cell[66536, 1983, 284, 8, 87, "Output"]
}, Open  ]],
Cell[66835, 1994, 648, 15, 87, "Text"],
Cell[67486, 2011, 221, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[67732, 2019, 397, 12, 87, "Input"],
Cell[68132, 2033, 156, 4, 87, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[68337, 2043, 92, 2, 87, "SubsectionIcon"],
Cell[68432, 2047, 1355, 28, 87, "Text"],
Cell[CellGroupData[{
Cell[69812, 2079, 388, 12, 87, "Input"],
Cell[70203, 2093, 498, 16, 87, "Output"]
}, Open  ]],
Cell[70716, 2112, 407, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[71148, 2123, 389, 12, 87, "Input"],
Cell[71540, 2137, 438, 14, 87, "Output"]
}, Open  ]],
Cell[71993, 2154, 178, 4, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72208, 2163, 101, 2, 87, "SubsectionIcon"],
Cell[72312, 2167, 354, 6, 87, "Text"],
Cell[CellGroupData[{
Cell[72691, 2177, 697, 23, 87, "Input"],
Cell[73391, 2202, 306, 10, 87, "Output"]
}, Open  ]],
Cell[73712, 2215, 158, 3, 87, "Text"],
Cell[73873, 2220, 442, 13, 87, "Input"],
Cell[74318, 2235, 138, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[74481, 2242, 479, 15, 87, "Input"],
Cell[74963, 2259, 353, 11, 87, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[75365, 2276, 113, 2, 87, "SubsectionIcon"],
Cell[75481, 2280, 16, 0, 87, "Text"],
Cell[75500, 2282, 49, 0, 87, "Text"],
Cell[75552, 2284, 131, 4, 87, "Input"],
Cell[75686, 2290, 1058, 18, 87, "Text"],
Cell[76747, 2310, 557, 16, 87, "Input"],
Cell[77307, 2328, 506, 8, 87, "Text"],
Cell[77816, 2338, 549, 10, 87, "Text"],
Cell[78368, 2350, 318, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[78711, 2361, 482, 16, 87, "Input"],
Cell[79196, 2379, 215, 8, 87, "Output"]
}, Open  ]],
Cell[79426, 2390, 181, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[79632, 2398, 668, 21, 87, "Input"],
Cell[80303, 2421, 397, 12, 87, "Output"]
}, Open  ]],
Cell[80715, 2436, 553, 14, 87, "Text"],
Cell[81271, 2452, 352, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[81648, 2463, 781, 22, 87, "Input"],
Cell[82432, 2487, 740, 22, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83209, 2514, 795, 23, 87, "Input"],
Cell[84007, 2539, 701, 21, 87, "Output"]
}, Open  ]],
Cell[84723, 2563, 270, 5, 87, "Text"],
Cell[84996, 2570, 280, 5, 87, "Text"],
Cell[CellGroupData[{
Cell[85301, 2579, 948, 29, 87, "Input"],
Cell[86252, 2610, 1199, 40, 87, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87488, 2655, 1295, 41, 87, "Input"],
Cell[88786, 2698, 664, 21, 87, "Output"]
}, Open  ]],
Cell[89465, 2722, 121, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[89611, 2729, 679, 19, 87, "Input"],
Cell[90293, 2750, 726, 23, 87, "Output"]
}, Open  ]],
Cell[91034, 2776, 295, 6, 87, "Text"],
Cell[91332, 2784, 613, 16, 87, "Input"],
Cell[91948, 2802, 169, 3, 87, "Text"],
Cell[CellGroupData[{
Cell[92142, 2809, 698, 21, 87, "Input"],
Cell[92843, 2832, 722, 23, 87, "Output"]
}, Open  ]],
Cell[93580, 2858, 246, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[93851, 2866, 683, 21, 87, "Input"],
Cell[94537, 2889, 932, 31, 87, "Output"]
}, Open  ]],
Cell[95484, 2923, 186, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[95695, 2931, 712, 21, 87, "Input"],
Cell[96410, 2954, 1275, 41, 87, "Output"]
}, Open  ]],
Cell[97700, 2998, 564, 11, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[98301, 3014, 246, 6, 87, "SubsectionIcon"],
Cell[98550, 3022, 481, 8, 87, "Text"],
Cell[99034, 3032, 438, 13, 87, "Input"],
Cell[99475, 3047, 243, 4, 87, "Text"],
Cell[99721, 3053, 12237, 205, 87, "Input"],
Cell[111961, 3260, 391, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[112377, 3271, 611, 18, 87, "Input"],
Cell[112991, 3291, 334, 10, 87, "Output"]
}, Open  ]],
Cell[113340, 3304, 264, 5, 87, "Text"],
Cell[113607, 3311, 1286, 30, 87, "Input"],
Cell[114896, 3343, 1078, 26, 87, "Input"],
Cell[115977, 3371, 326, 6, 87, "Text"],
Cell[116306, 3379, 313, 5, 87, "Text"],
Cell[116622, 3386, 676, 21, 87, "Input"],
Cell[117301, 3409, 16491, 275, 87, "Input"],
Cell[133795, 3686, 294, 5, 87, "Text"],
Cell[134092, 3693, 435, 7, 87, "Text"],
Cell[134530, 3702, 1210, 35, 87, "Input"],
Cell[CellGroupData[{
Cell[135765, 3741, 1233, 35, 87, "Input"],
Cell[137001, 3778, 1161, 36, 87, "Output"]
}, Open  ]],
Cell[138177, 3817, 277, 5, 87, "Text"],
Cell[CellGroupData[{
Cell[138479, 3826, 1537, 44, 87, "Input"],
Cell[140019, 3872, 1170, 36, 87, "Output"]
}, Open  ]],
Cell[141204, 3911, 187, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[141416, 3919, 1548, 46, 87, "Input"],
Cell[142967, 3967, 1641, 49, 87, "Output"]
}, Open  ]],
Cell[144623, 4019, 1903, 53, 87, "Input"],
Cell[146529, 4074, 1355, 39, 87, "Input"],
Cell[CellGroupData[{
Cell[147909, 4117, 1470, 43, 87, "Input"],
Cell[149382, 4162, 690, 22, 87, "Output"]
}, Open  ]],
Cell[150087, 4187, 16, 0, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[150140, 4192, 161, 3, 87, "SubsectionIcon"],
Cell[150304, 4197, 735, 13, 87, "Text"],
Cell[151042, 4212, 414, 7, 87, "Text"],
Cell[151459, 4221, 947, 26, 87, "Input"],
Cell[152409, 4249, 179, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[152613, 4257, 194, 3, 87, "Input"],
Cell[152810, 4262, 524, 15, 87, "Output"]
}, Open  ]],
Cell[153349, 4280, 316, 6, 87, "Text"],
Cell[153668, 4288, 636, 16, 87, "Input"],
Cell[CellGroupData[{
Cell[154329, 4308, 239, 6, 87, "Input"],
Cell[154571, 4316, 246, 5, 87, "Output"]
}, Open  ]],
Cell[154832, 4324, 449, 8, 87, "Text"],
Cell[155284, 4334, 217, 5, 87, "Text"],
Cell[CellGroupData[{
Cell[155526, 4343, 175, 3, 87, "Input"],
Cell[155704, 4348, 545, 16, 87, "Output"]
}, Open  ]],
Cell[156264, 4367, 229, 4, 87, "Text"],
Cell[CellGroupData[{
Cell[156518, 4375, 199, 4, 87, "Input"],
Cell[156720, 4381, 489, 15, 87, "Output"]
}, Open  ]],
Cell[157224, 4399, 375, 6, 87, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[157636, 4410, 94, 2, 87, "SubsectionIcon"],
Cell[157733, 4414, 360, 8, 87, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

