(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[      7148,        218]
NotebookOptionsPosition[      5058,        152]
NotebookOutlinePosition[      6843,        207]
CellTagsIndexPosition[      6767,        202]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "Define\[Breve]Adjustment",
 " ",
 StyleBox["(part of the ColourMath package)",
  FontSize->12]
}], "Chapter",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
   3.44731239346875*^9, 3.447312399453125*^9}, {3.447390701375*^9, 
   3.447390712625*^9}, {3.447390810796875*^9, 3.4473908173125*^9}, {
   3.4523472274375*^9, 3.452347231625*^9}, {3.59522847980902*^9, 
   3.595228496107952*^9}, 3.595251830631611*^9},
 CellTags->"SwapOperations"],

Cell["\<\
Define\[Breve]Adjustment[tag,rule or function,comment] \[Dash] adds a rule or \
function to the list of operations that are tried when the \"Adjust\" button \
is used. These operations are referred to by their tag, which must be a \
symbol.\
\>", "Text",
 CellChangeTimes->{{3.595254490316736*^9, 3.5952546225142975`*^9}}],

Cell["For example:", "Text",
 CellChangeTimes->{{3.447390957390625*^9, 3.447391254703125*^9}, {
  3.452347238875*^9, 3.452347243484375*^9}, {3.4523544591875*^9, 
  3.45235455875*^9}, {3.595251075106397*^9, 3.5952512686584673`*^9}, {
  3.595251437904148*^9, 3.595251441313343*^9}, {3.5952518370979805`*^9, 
  3.5952518926301565`*^9}, {3.595254630931779*^9, 3.595254631915835*^9}}],

Cell[BoxData[
 RowBox[{"<<", " ", "ColourMaths`"}]], "Input",
 CellChangeTimes->{{3.5952515116543665`*^9, 3.5952515167536583`*^9}},
 CellLabel->"In[1]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Define\[Breve]Adjustment", "[", 
   RowBox[{"Binomial\[Breve]Theorem", ",", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", "x_"}], ")"}], "p_"], "\[RuleDelayed]", 
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"k", "=", "0"}], "\[Infinity]"], 
       RowBox[{
        SuperscriptBox["x", "k"], " ", 
        RowBox[{"Binomial", "[", 
         RowBox[{"p", ",", 
          TagBox["k",
           ColourMaths`Flag\[Breve]Colour,
           DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 1]}]}], "]"}]}]}],
      ColourMaths`Sum\[Breve]Inert,
      DefaultBaseStyle->{
       Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
      SelectWithContents->True]}], ",", "\"\<Binomial theorem\>\""}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.595254711042361*^9, 3.595254729677427*^9}, {
  3.5952549729013386`*^9, 3.5952549957716465`*^9}},
 CellLabel->"In[2]:="],

Cell["\<\
In this example, one symbol, k, is flagged. This means that because it is a \
dummy variable, its name is ultimately left to the user. For example:\
\>", "Text",
 CellChangeTimes->{{3.595254678475498*^9, 3.595254694106392*^9}, {
  3.5952547350027313`*^9, 3.5952548036426573`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   TagBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", "x"}], ")"}], "s"],
    ColourMaths`Red\[Breve]Colour,
    DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 0]}], "]"}], "//", 
  RowBox[{"Red\[Breve]ColourAdjust", "[", 
   RowBox[{"Binomial\[Breve]Theorem", ",", 
    RowBox[{
     TagBox["k",
      ColourMaths`Flag\[Breve]Colour,
      DefaultBaseStyle->{FontColor -> RGBColor[1, 0, 1]}], "\[Rule]", "m"}]}],
    "]"}]}]], "Input",
 CellChangeTimes->{{3.59525481051305*^9, 3.5952548430039086`*^9}, {
  3.5952548747507243`*^9, 3.595254902429308*^9}, {3.5952550177179017`*^9, 
  3.5952550421953015`*^9}},
 CellLabel->"In[3]:="],

Cell[BoxData[
 RowBox[{"f", "[", 
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"m", "=", "0"}], "\[Infinity]"], 
    RowBox[{
     SuperscriptBox["x", "m"], " ", 
     RowBox[{"Binomial", "[", 
      RowBox[{"s", ",", "m"}], "]"}]}]}],
   ColourMaths`Sum\[Breve]Inert,
   DefaultBaseStyle->{
    Background -> RGBColor[0.7440000000000001, 0.7440000000000001, 1]},
   SelectWithContents->True], "]"}]], "Output",
 CellChangeTimes->{3.5952549041764073`*^9, 3.5952550426953306`*^9},
 CellLabel->"Out[3]="]
}, Open  ]],

Cell[TextData[{
 "Click",
 Cell[BoxData[
  RowBox[{" ", 
   TagBox[
    ButtonBox[
     PaneSelectorBox[{False->"\<\"here\"\>", True->
      StyleBox["\<\"here\"\>", "HyperlinkActive"]}, Dynamic[
       CurrentValue["MouseOver"]],
      BaseStyle->{"Hyperlink"},
      BaselinePosition->Baseline,
      FrameMargins->0,
      ImageSize->Automatic],
     BaseStyle->"Link",
     ButtonData->"paclet:ColourMaths",
     ButtonNote->"paclet:ColourMaths"],
    Annotation[#, "paclet:ColourMaths", "Hyperlink"]& ]}]]],
 " for general information about the ColourMaths package"
}], "Text"]
}, Open  ]]
},
Saveable->False,
Selectable->True,
WindowSize->{717, 741},
WindowMargins->{{Automatic, 583}, {-20, Automatic}},
WindowFrame->"Normal",
Visible->True,
DockedCells->FEPrivate`FrontEndResource["FEExpressions", "HelpViewerToolbar"],
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, 
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"]}, {
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"], Inherited, 
   Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}, {
  Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowCellBracket->True,
ShowSelection->True,
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> {2014, 2, 14, 12, 10, 50.0672924`9.452128958511638}, "context" -> 
    "ColourMaths`", "keywords" -> {}, "index" -> True, "label" -> 
    "ColourMaths Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "ColourMaths Package", "status" -> "", "summary" -> "", "synonyms" -> {}, 
    "title" -> "Define\:02d8Adjustment", "type" -> "Symbol", "uri" -> 
    "ColourMaths/ref/DefineAdjustment"}, "SearchTextTranslated" -> ""},
ShowCellLabel->False,
ShowSyntaxStyles->False,
Magnification->1.,
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "SwapOperations"->{
  Cell[567, 22, 477, 11, 85, "Chapter",
   CellTags->"SwapOperations"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"SwapOperations", 6671, 196}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 477, 11, 85, "Chapter",
 CellTags->"SwapOperations"],
Cell[1047, 35, 332, 6, 39, "Text"],
Cell[1382, 43, 379, 5, 23, "Text"],
Cell[1764, 50, 154, 3, 27, "Input"],
Cell[1921, 55, 982, 26, 78, "Input"],
Cell[2906, 83, 290, 5, 39, "Text"],
Cell[CellGroupData[{
Cell[3221, 92, 695, 19, 29, "Input"],
Cell[3919, 113, 526, 15, 48, "Output"]
}, Open  ]],
Cell[4460, 131, 582, 18, 23, "Text"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

